/*
 * train
 * Copyright (C) 1999  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */
#ifndef __CONFIG_H      /* don't change these lines. They're to stop the */
#define __CONFIG_H      /* compiler getting into infinite include loops */
/*
 * config.h
 *
 * This file contains all the configuration information which is specific
 * to a particular layout.
 *
 * YOU WILL NEED TO MODIFY THIS FILE TO SUIT YOUR SPECIFIC LAYOUT.
 * Start at the top of this file, work your way down towards the bottom and
 * answer the questions (change the parameters etc) as you go.
 *
 * Before you start:
 *  - Design your layout and identify all turnouts. (Draw a diagram!!!)
 *  - Break your layout into a number of different blocks.
 *  - If you will have more than one train controller card, think about a
 *    sensible allocation of track blocks to cards. Ask yourself
 *    what the main traffic routes will be and try to group the blocks along
 *    these routes onto a single train controller card.
 *  - Number your turnouts consecutively starting at 1. If you leave numbers
 *    "unassigned", you will need to leave the corresponding wires on the
 *    corresponding controller card unconnected.
 *  - Number your track blocks consecutively starting at 1. If you leave
 *    numbers "unassigned", you will need to leave the corresponding wires on
 *    the corresponding cotroller card unconnected.
 ******************************************************************************
 * Uncomment any of the following definitions to turn on various types of
 * debugging options. Normally, you would not have any turned on. If you run
 * into trouble with the software (ie crashes or not working properly), then
 * you could use these debugging options to help you out.
 *
 * The PARANOID option turns on a whole bunch of software sanity (and bug
 * detecting) checks and should normally not be required unless you modify the
 * code or don't trust it.
 *
 * The SIMULATE_PORT option will turn off the code which writes to the parallel
 * port interface. This makes fundamental debugging easier but will not provide
 * any dynamic input ring stimulus as there will be no trains moving around the
 * layout!
 */

#define BASIC_DEBUG            /* set to display basic debug messages */
/* #define MESSAGE_TRACE          debug msgs (not always printed in order)*/
/* #define SEM_TRACE              set to debug semaphores */
/* #define SHM_TRACE              set to debug shared memory */
/* #define PORT_DEBUG             set to debug port controller */
/* #define STATUS_SCAN_DEBUG      set to debug status scanning process */
/* #define TIMER_DEBUG            set to debug timekeeper process */
/* #define DISPATCHER_DEBUG       set to debug dispatcher process */
#define POINTS_DEBUG           /* set to debug the points process */
#define POWER_DEBUG            /* set to debug the power process */
/* #define SIG_HANDLER_DEBUG      set to debug signal handlerss */
#define CONTROLLER_DEBUG       /* set to debug the controller process */
#define ENGINEER_DEBUG         /* set to debug the engineer process */
#define PARANOID               /* set to enable pedantic error checking */
#define SIMULATE_PORT          /* set if controllers not connected in ring */

/******************************************************************************
 * User definable time constants:
 *
 * Each of the general purpose and block detection inputs is debounced by the
 * software. The software reads the inputs at a rate of Scan_Interval (defined
 * as 0.2 Seconds by default in train.c) and after the first reading, waits for
 * another CHANGE_PERSISTENCE number of consecutive reads of the same state
 * before recognising the change. The value of CHANGE_PERSISTENCE can be from
 * zero (no debouncing) up to any positive integer.
 *
 * Note that train.c has a kludge in it to allow for funny timing in the case
 * of block entry. The persistence is reduced for block entry as opposed to 
 * block exit. See train.c for details.
 */

#define CHANGE_PERSISTENCE  2  /* # ADDITIONAL reads before recognising chng */

/*
 * The ring keepalive mechanism is used to tri-state all point motor and relay
 * drivers if the software stops working. Normally, this timeout is 20 seconds
 * or more. If a train is stopped and there is no change to any of the ring
 * settings, there is a low possibility that the ring activity will cease for
 * more than 20 seconds. If so, a "refresh" needs to take place. This happens
 * after the keepalive interval expires
 */

#define KEEPALIVE_INTERVAL  10 /* seconds. to refresh hardware ring reset */

/******************************************************************************
 * A note about point and block numbering.....
 *
 * NORMALLY, (if you build the circuit exactly as published)....
 * Point 1 or Block 1 refers to the 1st point or block connections on the first
 * train controller card. Point 10 or Block 10 refers to the last point or
 * block connections on the first train controller card.
 *
 * Point 11 or Block 11 refers to the 1st point or block on the SECOND train
 * controller card. Point 20 or Block 20 refers to the last point or block on
 * the SECOND train controller card.
 *
 * etc.
 *
 * ANOTHER NOTE ABOUT DEFINING BLOCKS WHICH INCLUDE POINTS
 * Later in this file of definitons, the concept of "centre of a block" will
 * become important. You will need to measure the length (in mm or similar)
 * from the nominal "centre" of each block to each exit point of the block.
 *
 * The train control software has been written with a fundamental assumption
 * about points and blocks: the points end of each turnout (the end where
 * there is just ONE pair of rails) MUST MUST MUST be closest to the "centre"
 * of the track block. Saying this another way, starting at the nominal
 * "centre" of a block, you should be able to head in either direction and
 * exit the track block by ANY of the possible paths (in that direction)
 * WITHOUT again changing direction.
 *
 * You can electrically join two or more sets of points into the one block
 * but if you do, you must ensure that the points end of each of the turnouts
 * is always closest to the centre of the block.
 *
 * <--- one block            --->     <--- one block     --->
 *
 *       _\_        _/_       _/_
 *  _\______\______/____/____/_        _\______/_______\_____
 *             ^                                  ^
 *           centre                             centre
 *
 *          allowed                          not allowed
 *
 * The "centre" of the block does NOT have to be half way along the block.
 */

/******************************************************************************
 * Remember the diagram of your layout you made before.... In this section,
 * you will need to "express this diagram in words" so that the software
 * has a picture of it.
 *
 * This section can "get a little confusing". Please follow the following
 * instructions exactly, keep your wits about you and you'll be OK!
 *
 * 1.
 * YOU WILL CERTAINLY NEED TO MODIFY THIS STRUCTURE AS YOUR LAYOUT WILL ALMOST
 * CERTAINLY BE DIFFERENT TO MINE!!!
 *
 * 2.
 * Make sure that your layout diagram includes CORRECT numbers for each of the
 * points and track blocks. The numbers you allocate need to correspond to the
 * numbers of the point and block outputs on the train controller card.
 *
 * 3.
 * You will also need to determine the FORWARD and REVERSE directions for
 * each of the track blocks. The forward direction is the direction a train
 * will travel if the polarity relay for that block is NOT ENERGISED. The
 * reverse direction is the direction a train will travel if the polarity
 * relay IS ENERGISED. Don't worry too much if you get this wrong becuase it
 * will be apparent when you start to run your trains and you'll know by then
 * that you need to "debug" the entries in the following structure. :-)
 *
 * 4.
 * Measure the distance from the "centre" of each block to each one of the
 * block boundaries. You should measure using a small unit such as mm or
 * sixteenths of an inch. ONLY WHOLE NUMBERS ARE SUPPORTED. NOT FRACTIONS.
 *
 * 5.
 * BEFORE you try to modify or even READ the structure below, look at the
 * definitions for BLOCK_ELEMENT, ADJACENCY and POINT_SETTING (in that order)
 * in the file <<train.h>>. The structure below defines the layout in these
 * terms.
 *
 * NOTE NOTE NOTE: The <controllerN.h> file includes a static definition called
 * MAX_CASCADED_POINTS which is the maximum number of sets of points in one
 * track block. If you get compile time errors and you have MORE than 5 sets
 * of points in one of your track blocks, you'd better change this constant.
 *-----------------------------------------------------------------------------
 * For each block, there is an ADJACENCY list containing a complete list of
 * track segments which are adjacent to the present segment.
 *
 * The adjacency list is an array of ADJACENCY structure elements containing:
 *     Number of the Adjacent Segment,
 *     Length From Centre To Block Boundary Using This Exit,
 *     The electrical direction which needs to be set in order to reach the
 *       adjacent block.
 *     Point Setting List travelling from Block Centre to Exit
 *       (note that if there are no sets of points to cross before reaching
 *       the block boundary, the Point Setting List is null)
 *
 * The Point Setting List is an ordered array of the following pairs:
 *     Number of the set of points
 *     Setting for points to reach the specified exit (OPEN or CLOSED)
 *
 * NOTE NOTE
 * Blocks which are "dead ends" (finish in a set of buffers) are conspicuous
 * becuase one of their "polarities" definitions has a length of 0. The
 * definition which has a non zero value indicates the total length of the
 * block from entry point to buffers.
 *
 * It is important that the polarity with zero length is that polarity
 * which would cause the train to smash into the buffers. The other polarity
 * is that which would normally take the train out of the dead end block into
 * the rest of the train set.
 *
 * You need to manually calculate the MAX_BLOCK_LENGTH and specify it. The
 * run time checks use this to make sure your Block[] structure is sane.
 *
 * The STOPPING_UNCERTAINTY is the amount of overshoot your heaviest train
 * will have if you cut its power when it is running at max speed.
 */

#define MAX_BLOCK_LENGTH    365  /* used in cross checking */
#define STOPPING_UNCERTAINTY 25

BLOCK_STRUCT Block[] =
{
  { /* BLOCK 1 */
    "Back inner grade underpass",
    {
      { 2,  33, NORMAL_POLARITY,   {BOUNDARY}},
      {10,  33, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 2 */
    "NW inner grade",
    {
      {17,  31, NORMAL_POLARITY,   {{3,  OPEN}, {2, CLOSED}, BOUNDARY}},
      { 3,  55, NORMAL_POLARITY,   {{3,  OPEN}, {2, OPEN}, {7, CLOSED}, BOUNDARY}},
      { 4,  55, NORMAL_POLARITY,   {{3,  OPEN}, {2, OPEN}, {7, OPEN}, BOUNDARY}},
      {12,  55, NORMAL_POLARITY,   {{3,  CLOSED}, BOUNDARY}},
      { 6,  16, OPPOSITE_POLARITY, {{4,  CLOSED}, BOUNDARY}},
      { 1,  16, OPPOSITE_POLARITY, {{4,  OPEN}, BOUNDARY}}
    }
  },

  { /* BLOCK 3 */
    "Long front overpass",
    {
      {10,  182, NORMAL_POLARITY,   {BOUNDARY}},
      { 2,  183, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 4 */
    "Western triangle access",
    {
      { 5,  42, NORMAL_POLARITY,   {{6,  OPEN},   BOUNDARY}},
      {20,  42, NORMAL_POLARITY,   {{6,  CLOSED}, BOUNDARY}},
      { 2,  43, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 5 */
    "Western triangle vertex",
    {
      {10,  22, OPPOSITE_POLARITY, {BOUNDARY}},
      { 4,  32, NORMAL_POLARITY, {{5,  OPEN}, BOUNDARY}},
      {DEAD_END, 76, OPPOSITE_POLARITY, {{5,  CLOSED}, BOUNDARY}}
    }
  },

  { /* BLOCK 6 */
    "Rear overpass",
    {
      { 2,  66, NORMAL_POLARITY,   {BOUNDARY}},
      {18,  67, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 7 */
    "Rear Eastern grade access",
    {
      {10,  25, NORMAL_POLARITY,   {BOUNDARY}},
      {18,  25, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 8 */
    "Eastern triangle vertex",
    {
      {20,  24, NORMAL_POLARITY,   {BOUNDARY}},
      {10,  25, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 9 */
    "NE inner to outer cutover",
    {
      {10,  44, NORMAL_POLARITY,   {BOUNDARY}},
      {16,  44, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 10 */
    "Rear central interchange",
    {
      { 1,  16, NORMAL_POLARITY,   {{12, CLOSED}, BOUNDARY}},
      { 5,  32, NORMAL_POLARITY,   {{12, OPEN}, {15, CLOSED}, BOUNDARY}},
      { 8,  32, NORMAL_POLARITY,   {{12, OPEN}, {15, OPEN}, BOUNDARY}},
      { 9,  28, OPPOSITE_POLARITY, {{13, CLOSED}, {16, CLOSED}, BOUNDARY}},
      { 7,  12, OPPOSITE_POLARITY, {{13, OPEN}, BOUNDARY}},
      { 3,  28, OPPOSITE_POLARITY, {{13, CLOSED}, {16, OPEN}, BOUNDARY}},
    }
  },

  { /* BLOCK 11 */
    "Long back grade",
    {
      {12, 145, NORMAL_POLARITY,   {BOUNDARY}},
      {16, 145, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 12 */
    "SW corner",
    {
      {13,  44, NORMAL_POLARITY,   {BOUNDARY}},
      {11,  43, OPPOSITE_POLARITY, {{1,  CLOSED}, BOUNDARY}},
      { 2,  43, OPPOSITE_POLARITY, {{1,  OPEN}, BOUNDARY}}
    }
  },

  { /* BLOCK 13 */
    "SW run",
    {
      {14,  40, NORMAL_POLARITY,   {{9,  CLOSED}, {10, CLOSED}, BOUNDARY}},
      {19,  40, NORMAL_POLARITY,   {{9,  CLOSED}, {10, OPEN}, BOUNDARY}},
      {20,  25, NORMAL_POLARITY,   {{9,  OPEN},   BOUNDARY}},
      {12,  24, OPPOSITE_POLARITY, {{8,  CLOSED}, BOUNDARY}},
      {17,  24, OPPOSITE_POLARITY, {{8,  OPEN}, BOUNDARY}}
    }
  },

  { /* BLOCK 14 */
    "Front run",
    {
      {15,  36, NORMAL_POLARITY,   {BOUNDARY}},
      {13,  37, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 15 */
    "SE corner",
    {
      {16,  44, NORMAL_POLARITY,   {BOUNDARY}},
      {14,  44, OPPOSITE_POLARITY, {{20, CLOSED}, BOUNDARY}},
      {18,  44, OPPOSITE_POLARITY, {{20, OPEN}, BOUNDARY}}
    }
  },

  { /* BLOCK 16 */
    "East dead end",
    {
      {11,  16, NORMAL_POLARITY,   {{19, CLOSED},   BOUNDARY}},
      { 9,  16, NORMAL_POLARITY,   {{19, OPEN}, BOUNDARY}},
      {15,  16, OPPOSITE_POLARITY, {{18, CLOSED},   BOUNDARY}},
      {DEAD_END, 186, OPPOSITE_POLARITY, {{18, OPEN}, BOUNDARY}}
    }
  },

  { /* BLOCK 17 */
    "SW corner bypass",
    {
      {13,  50, NORMAL_POLARITY,   {BOUNDARY}},
      { 2,  50, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 18 */
    "Eastern Y and grade",
    {
      {15,  63, NORMAL_POLARITY,   {{14, CLOSED}, BOUNDARY}},
      {19,  18, NORMAL_POLARITY,   {{14, OPEN}, BOUNDARY}},
      { 7,  18, OPPOSITE_POLARITY, {{17, CLOSED}, BOUNDARY}},
      { 6,  18, OPPOSITE_POLARITY, {{17, OPEN}, BOUNDARY}}
    }
  },

  { /* BLOCK 19 */
    "Front Eastern Y access",
    {
      {18,  30, NORMAL_POLARITY,   {BOUNDARY}},
      {13,  30, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },

  { /* BLOCK 20 */
    "Front Western Y access",
    {
      { 4,  68, NORMAL_POLARITY,   {{11, OPEN},   BOUNDARY}},
      { 8,  40, NORMAL_POLARITY,   {{11, CLOSED}, BOUNDARY}},
      {13,  40, OPPOSITE_POLARITY, {BOUNDARY}}
    }
  },
} ;

/******************************************************************************
 * How do you want to map speed settings to combinations of DC / PWM power?
 *
 * Normally, you can choose to leave the following mappings unchanged. However,
 * you may find that your locomotives perform better (accelerate or decelerate
 * more smoothly) with different combinations. You are free to experiment.
 *
 * The calculated voltages are the AVERAGE convolutions of the PWM and DC level
 * as appearing on the track.
 */

#define MAX_SPEED    28     /* so the table MUST have 29 entries */

PSU_OBJECT Speed_Table[MAX_SPEED+1] =
{
/* {PWM setting, DC setting}  Logical Speed, Average Voltage */
  {0, 0},      /* speed = 0    0.0 */
  {1, 0},      /* speed = 1    0.2 */
  {2, 1},      /* speed = 2    0.6 */
  {2, 2},      /* speed = 3    1.0 */
  {3, 2},      /* speed = 4    1.4 */
  {4, 2},      /* speed = 5    1.8 */
  {3, 3},      /* speed = 6    2.2 */
  {4, 3},      /* speed = 7    2.5 */
  {3, 4},      /* speed = 8    2.9 */
  {4, 4},      /* speed = 9    3.2 */
  {5, 4},      /* speed = 10   3.5 */
  {6, 4},      /* speed = 11   3.8 */
  {7, 4},      /* speed = 12   4.1 */
  {8, 4},      /* speed = 13   4.3 */
  {9, 4},      /* speed = 14   4.6 */
  {10, 4},     /* speed = 15   4.9 */
  {11, 4},     /* speed = 16   5.2 */
  {12, 4},     /* speed = 17   5.5 */
  {13, 4},     /* speed = 18   5.8 */
  {14, 4},     /* speed = 19   6.0 */
  {15, 4},     /* speed = 20   6.3 */
  {15, 8},     /* speed = 21   6.6 */
  {15, 9},     /* speed = 22   6.8 */
  {15, 10},    /* speed = 23   7.7 */
  {15, 11},    /* speed = 24   8.6 */
  {15, 12},    /* speed = 25   9.5 */
  {15, 13},    /* speed = 26  10.5 */
  {15, 14},    /* speed = 27  11.4 */
  {15, 15},    /* speed = 28  12.3 */
} ;

#endif
