/*
 * train
 * Copyright (C) 1999  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */
#ifndef __CONTROLLER_H      /* don't change these lines. They're to stop the */
#define __CONTROLLER_H      /* compiler getting into infinite include loops */
/*
 * controller3.h
 *
 * This file contains all the configuration information which is specific
 * to the design of the standard train controller card.
 *
 * YOU WILL NOT NEED TO MODIFY THIS FILE IF YOU BUILD THE PUBLISHED CIRCUIT.
 *****************************************************************************/

#define DEFAULT_LOG_FILENAME   "train.log"
#define DEFAULT_ROUTE_FILENAME "route%d.txt" /* The %d is expanded by sprintf */

/*
 * On the assumption that there is ONE point (turnout) supply on each train
 * controller card, how many sets of point driver can one train controller card
 * support? The circuit as published has 20 point motor drivers which support
 * 10 sets of points.
 *
 * On the assumption that there are 2 programmable power sources per card,
 * how many track blocks will one train controller card support? The circuit
 * as published supports 10 track blocks (there are 10 direction / power source
 * relays and 10 track block detectors per card)
 */

#define NUM_POINTS_PER_CONTROLLER_CARD   10
#define NUM_BLOCKS_PER_CONTROLLER_CARD   10

/*
 * How many sets of points will your layout contain? Normally, this number
 * will be NUM_POINTS_PER_CONTROLLER_CARD * (number of controller cards)
 * but a smaller number is also OK (with a little care).
 *
 * How many different track blocks will the layout contain? Normally, this
 * number will be NUM_BLOCKS_PER_CONTROLLER_CARD * (number of controller cards)
 * but a smaller number is also OK (with a little more care).
 *
 * How many points can be cascaded between the centre of a block and the
 * furthest exit. This is defined in MAX_CASCADED_POINTS.
 */

#define NUM_POINTS                30 /* actual number in your layout */
#define NUM_BLOCKS                30 /* actual number in your layout */
#define MAX_CASCADED_POINTS       5  /* max num points in series */

/*
 * How many point supplies will your layout contain? Normally, this number
 * will be the same as the number of train controller cards.
 *
 * How many power sources will your layout contain? Normally, this will be
 * twice the number of train controller cards.
 *
 * How many current limiting power supplies will your layout contain? Normally,
 * this will be the same as the number of train controller cards. (A current
 * limiting power supply is what feeds the 16V and 8V regulators which in turn
 * feed the two DC and two PWM programmable power sources)
 *
 * The number of reservations constant is used by the controller process to
 * define the number of different engineer processes which could be waiting
 * upon any particular resource. If there is one controller card, one
 * reservation is required - so that one train can reserve a block which the
 * other train is occupying. For two controllers, three reservations are
 * required although it is unlikely that this number will ever be seen. For
 * three controller cards, 5 reservations and so on.
 *
 * How many differnet General Purpose Inputs will your layout contain? Normally,
 * this will be the same as 5 * (number of controller cards).
 */

#define NUM_POINTS_SUPPLIES        3 /* 1 for one controller, 2 for two, .. */
#define NUM_POWER_SOURCES          6 /* 2 for one controller, 4 for two, .. */
#define NUM_PSU_ALLOCATIONS        3 /* 1 for one controller, 2 for two, .. */
#define NUM_RESERVATIONS           5 /* 1 for one controller, 3 for two, .. */
#define NUM_OVERLOAD_SOURCES       3 /* 1 for one controller, 2 for two, .. */
#define NUM_GP_INPUTS             15 /* 5 for one controller, 10 for two, .. */
#define POINTS_CAPACITANCE     10000 /* Number of uF of capacitance you used */

#define NUM_SERIAL_OUT_BITS      168 /* 56 for one card, 112 for 2 cards, ...*/
#define NUM_SERIAL_IN_BITS        48 /* 16 for one card, 32 for 2 cards, ... */

/******************************************************************************
 * Now, you need to define the meaning of all the bits in the output shift
 * ring and the input shift ring. You will modify the following structures
 * to match the circuits that you build.
 *
 * If you build more than one train controller card, then you will need to
 * modify the structures to reflect the longer nature of your shift rings.
 * Assuming that all the cards you build are electrically identical, you can
 * simply replicate the bit orders provided (for one card) and renumber each
 * of the replicated fields so that no numbers are duplicated.
 *
 * The format of the structures is as follows:
 *    First entry corresponds to the first bits to be shifted out (or in)
 *    Last entry corresponds to the last bits to be shifted out (or in)
 *    (The table is terminated with the flag "END_******_RING." where
 *    ****** is either INPUT or OUTPUT)
 *
 * Note that the general format of each line is....
 *    <Type of object>,    <Logical identification number of the object>
 *
 * NOTE ALSO: If you want to make use of the general purpose inputs, then
 * you are welcomed to modify the source code to suit your needs. There is
 * no intrinsic support for the general purpose inputs provided as standard.
 */

#include "train.h"                /* for definition of Point_R, ..... */

SERIAL_OUT Output_Format[] =
{ /* The FIRST bit to be shifted out appears at the top of this table */

    /*
     * THE PARAMETERS FOR THE THIRD (highest numbered) CARD START HERE
     */

  {BLOCK_DIR,       30},         /* first group of 8 bits IC13 */
  {POINT_R,         27},
  {POINT_S,         28},
  {POINT_R,         28},
  {POINT_S,         29},
  {POINT_R,         29},
  {POINT_S,         30},
  {POINT_R,         30},

  {POINT_R,         26},         /* second group of 8 bits IC12 */
  {POINT_S,         27},
  {BLOCK_DIR,       27},
  {BLOCK_POWER_SRC, 28},
  {BLOCK_DIR,       28},
  {BLOCK_POWER_SRC, 29},
  {BLOCK_DIR,       29},
  {BLOCK_POWER_SRC, 30},

  {BLOCK_POWER_SRC, 26},          /* third group of 8 bits IC10 */
  {BLOCK_DIR,       26},
  {BLOCK_POWER_SRC, 27},
  {POINT_S,         24},
  {POINT_R,         24},
  {POINT_S,         25},
  {POINT_R,         25},
  {POINT_S,         26},

  {POINT_R,         21},          /* fourth group of 8 bits IC8 */
  {POINT_S,         22},
  {POINT_R,         22},
  {POINT_S,         23},
  {POINT_R,         23},
  {BLOCK_DIR,       24},
  {BLOCK_POWER_SRC, 25},
  {BLOCK_DIR,       25},

  {BLOCK_POWER_SRC, 21},          /* fifth group of 8 bits IC7 */
  {BLOCK_DIR,       21},
  {BLOCK_POWER_SRC, 22},
  {BLOCK_DIR,       22},
  {BLOCK_POWER_SRC, 23},
  {BLOCK_DIR,       23},
  {BLOCK_POWER_SRC, 24},
  {POINT_S,         21},

  {CONTROLLER_DC ,  6},          /* sixth group of 8 bits IC18 */
  {CONTROLLER_PWM,  6},

  {CONTROLLER_DC ,  5},          /* seventh group of 8 bits IC20 */
  {CONTROLLER_PWM,  5},

    /*
     * THE PARAMETERS FOR THE SECOND (middle numbered) CARD START HERE
     */

  {BLOCK_DIR,       20},         /* first group of 8 bits IC13 */
  {POINT_R,         17},
  {POINT_S,         18},
  {POINT_R,         18},
  {POINT_S,         19},
  {POINT_R,         19},
  {POINT_S,         20},
  {POINT_R,         20},

  {POINT_R,         16},         /* second group of 8 bits IC12 */
  {POINT_S,         17},
  {BLOCK_DIR,       17},
  {BLOCK_POWER_SRC, 18},
  {BLOCK_DIR,       18},
  {BLOCK_POWER_SRC, 19},
  {BLOCK_DIR,       19},
  {BLOCK_POWER_SRC, 20},

  {BLOCK_POWER_SRC, 16},          /* third group of 8 bits IC10 */
  {BLOCK_DIR,       16},
  {BLOCK_POWER_SRC, 17},
  {POINT_S,         14},
  {POINT_R,         14},
  {POINT_S,         15},
  {POINT_R,         15},
  {POINT_S,         16},

  {POINT_R,         11},          /* fourth group of 8 bits IC8 */
  {POINT_S,         12},
  {POINT_R,         12},
  {POINT_S,         13},
  {POINT_R,         13},
  {BLOCK_DIR,       14},
  {BLOCK_POWER_SRC, 15},
  {BLOCK_DIR,       15},

  {BLOCK_POWER_SRC, 11},          /* fifth group of 8 bits IC7 */
  {BLOCK_DIR,       11},
  {BLOCK_POWER_SRC, 12},
  {BLOCK_DIR,       12},
  {BLOCK_POWER_SRC, 13},
  {BLOCK_DIR,       13},
  {BLOCK_POWER_SRC, 14},
  {POINT_S,         11},

  {CONTROLLER_DC ,  4},          /* sixth group of 8 bits IC18 */
  {CONTROLLER_PWM,  4},

  {CONTROLLER_DC ,  3},          /* seventh group of 8 bits IC20 */
  {CONTROLLER_PWM,  3},

    /*
     * THE PARAMETERS FOR THE FIRST CARD START HERE
     */

  {BLOCK_DIR,       10},         /* first group of 8 bits IC13 */
  {POINT_R,          7},
  {POINT_S,          8},
  {POINT_R,          8},
  {POINT_S,          9},
  {POINT_R,          9},
  {POINT_S,         10},
  {POINT_R,         10},

  {POINT_R,          6},         /* second group of 8 bits IC12 */
  {POINT_S,          7},
  {BLOCK_DIR,        7},
  {BLOCK_POWER_SRC,  8},
  {BLOCK_DIR,        8},
  {BLOCK_POWER_SRC,  9},
  {BLOCK_DIR,        9},
  {BLOCK_POWER_SRC, 10},

  {BLOCK_POWER_SRC,  6},          /* third group of 8 bits IC10 */
  {BLOCK_DIR,        6},
  {BLOCK_POWER_SRC,  7},
  {POINT_S,          4},
  {POINT_R,          4},
  {POINT_S,          5},
  {POINT_R,          5},
  {POINT_S,          6},

  {POINT_R,          1},          /* fourth group of 8 bits IC8 */
  {POINT_S,          2},
  {POINT_R,          2},
  {POINT_S,          3},
  {POINT_R,          3},
  {BLOCK_DIR,        4},
  {BLOCK_POWER_SRC,  5},
  {BLOCK_DIR,        5},

  {BLOCK_POWER_SRC,  1},          /* fifth group of 8 bits IC7 */
  {BLOCK_DIR,        1},
  {BLOCK_POWER_SRC,  2},
  {BLOCK_DIR,        2},
  {BLOCK_POWER_SRC,  3},
  {BLOCK_DIR,        3},
  {BLOCK_POWER_SRC,  4},
  {POINT_S,          1},

  {CONTROLLER_DC ,   2},          /* sixth group of 8 bits IC18 */
  {CONTROLLER_PWM,   2},

  {CONTROLLER_DC ,   1},          /* seventh group of 8 bits IC20 */
  {CONTROLLER_PWM,   1},

  {END_OUTPUT_RING, 0}           /* This MUST be the last entry in the table */
} ;

SERIAL_IN Input_Format[] =
{ /* The FIRST bit to be shifted in appears at the top of this table */

    /*
     * THE PARAMETERS FOR THE THIRD (highest numbered) CARD START HERE
     */

  {BLOCK_DET,        29},         /* IC 11 pin 7 */
  {BLOCK_DET,        30},
  {POWER_OVERLOAD,    3},
  {GP_INPUT,         11},
  {GP_INPUT,         12},
  {GP_INPUT,         13},
  {GP_INPUT,         14},
  {GP_INPUT,         15},         /* IC 11 pin 15 */

  {BLOCK_DET,        21},         /* IC 9 pin 7 */
  {BLOCK_DET,        22},
  {BLOCK_DET,        23},
  {BLOCK_DET,        24},
  {BLOCK_DET,        25},
  {BLOCK_DET,        26},
  {BLOCK_DET,        27},
  {BLOCK_DET,        28},         /* IC 9 pin 15 */

    /*
     * THE PARAMETERS FOR THE SECOND (middle numbered) CARD START HERE
     */

  {BLOCK_DET,        19},         /* IC 11 pin 7 */
  {BLOCK_DET,        20},
  {POWER_OVERLOAD,    2},
  {GP_INPUT,          6},
  {GP_INPUT,          7},
  {GP_INPUT,          8},
  {GP_INPUT,          9},
  {GP_INPUT,         10},         /* IC 11 pin 15 */

  {BLOCK_DET,        11},         /* IC 9 pin 7 */
  {BLOCK_DET,        12},
  {BLOCK_DET,        13},
  {BLOCK_DET,        14},
  {BLOCK_DET,        15},
  {BLOCK_DET,        16},
  {BLOCK_DET,        17},
  {BLOCK_DET,        18},         /* IC 9 pin 15 */

    /*
     * THE PARAMETERS FOR THE FIRST CARD START HERE
     */

  {BLOCK_DET,         9},         /* IC 11 pin 7 */
  {BLOCK_DET,        10},
  {POWER_OVERLOAD,    1},
  {GP_INPUT,          1},
  {GP_INPUT,          2},
  {GP_INPUT,          3},
  {GP_INPUT,          4},
  {GP_INPUT,          5},         /* IC 11 pin 15 */

  {BLOCK_DET,         1},         /* IC 9 pin 7 */
  {BLOCK_DET,         2},
  {BLOCK_DET,         3},
  {BLOCK_DET,         4},
  {BLOCK_DET,         5},
  {BLOCK_DET,         6},
  {BLOCK_DET,         7},
  {BLOCK_DET,         8},         /* IC 9 pin 15 */

  {END_INPUT_RING,   0}          /* This MUST be the last entry in the table */
} ;

/******************************************************************************
 * How do you want to map speed settings to combinations of DC / PWM power?
 *
 * Normally, you can choose to leave the following mappings unchanged. However,
 * you may find that your locomotives perform better (accelerate or decelerate
 * more smoothly) with different combinations. You are free to experiment.
 */

#define MAX_SPEED    24     /* so the table MUST have 25 entries */

PSU_OBJECT Speed_Table[MAX_SPEED+1] =
{
#endif
