/*
 * train
 * Copyright (C) 1999  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 * train.h
 *
 * This file contains the definitions of the structures and constants which
 * will be used by the train controller software.
 *
 * YOU WILL NOT NORMALLY NEED TO MODIFY THIS FILE
 ******************************************************************************
 */

#ifndef  __TRAIN_H
#define  __TRAIN_H

typedef unsigned char   Uchar ;
typedef unsigned short  Ushort ;
typedef unsigned long   Ulong ;

#define DEAD_END  (NUM_BLOCKS+1)

/******************************************************************************
 * The following constants and structures relate to the timer process.
 *
 * TICK_FREQ is the frequency (in Hz) that the timer process ticks over. The 
 * higher the value, the more times per second that the process is run.
 * Values above 50 are tending towards being unrealistic.
 */

#define   TICK_FREQ     30       /* Hz */
#define   FRAC_TIME(A)  (A*100/TICK_FREQ)

struct TIMER_STRUCT
{
  Ushort              interval ; /* number of ticks remaining */
  int                 ref_id ;   /* magic number specific to the originator */
  int                 reply_to ; /* message queue to notify of timeout */
  struct TIMER_STRUCT *next ;    /* linked list of timers */
} ;

struct SYS_TIME_STRUCT
{
  Ushort             ticks ;
  Ulong              seconds ;
} ;

typedef struct TIMER_STRUCT TIMER ;
typedef struct SYS_TIME_STRUCT SYS_TIME ;

/******************************************************************************
 * The LOG_PRIORITY type is a list of the different possible priorities that a
 * log message could have.
 */

enum LOG_PRIORITY_TYPES
{
  NORM,           /* normal priority - log to file if file, otherwise screen */
  SCREEN,         /* Log to screen even if a file is defined */
  ERR,            /* this is an error, log to screen also */
  FILE_ONLY,      /* log message to file but not screen */
  LOG_ABORT       /* display this message and bail */
} ;

typedef enum LOG_PRIORITY_TYPES LOG_PRIORITY ;

/******************************************************************************
 * The RING_OBJECT_TYPES data type is a set of values representing each type of
 * electrical object which resides on the train controller shift ring.
 */

enum RING_OBJECT_OUTPUT_TYPES
{
  END_OUTPUT_RING, /* Signifies the end of the definition structure */
  POINT_R,         /* Next object is the reset coil of a point motor */
  POINT_S,         /* Next object is the set coil of a point motor */
  BLOCK_DIR,       /* Next object is the dir setting of a track block */
  BLOCK_POWER_SRC, /* Next object is the power source for a track block */
  CONTROLLER_DC,   /* Next object is the setting for a DC power controller */
  CONTROLLER_PWM,  /* Next object is the setting for a PWM power controller */

  SPEED            /* This parameter is only used in messages */
} ;

typedef enum RING_OBJECT_OUTPUT_TYPES RING_OUT_TYPE ;

enum RING_OBJECT_INPUT_TYPES
{
  END_INPUT_RING,  /* Signifies the end of the definition structure */
  BLOCK_DET,       /* Next object is a block detect input */
  GP_INPUT,        /* Next object is a general purpose input */
  POWER_OVERLOAD   /* Next object is an overload current trip status bit */
} ;

typedef enum RING_OBJECT_INPUT_TYPES RING_IN_TYPE ;

/******************************************************************************
 * The POINTS_OBJECT data type is the set of values representing the state of
 * each set of points.
 */

enum POINTS_OBJECT_VALUES
{
  UNKNOWN_STATE,      /* This value (0) is used by other enumerations too */
  OPEN,
  CLOSED
} ;

typedef enum POINTS_OBJECT_VALUES POINTS_OBJECT ;

/******************************************************************************
 * The STATE data type holds a binary state
 */

enum STATE_VALUES
{
  OFF,
  ON
} ;

typedef enum STATE_VALUES STATE ;

/******************************************************************************
 * The ALLOCATION data type is the set of values representing ALLOCATION
 */

enum ALLOCATION_VALUES
{
  RELEASE,
  OWN
} ;

typedef enum ALLOCATION_VALUES ALLOCATION ;

/******************************************************************************
 * The ACCELERATION data type describes the meaning of the speed setting
 *
 * Each of the acceleration constants is a multiple of FASTEST_TIME
 */

#define FASTEST_TIME 1.5  /* seconds to accelerate from 0 to max */

enum ACCELERATION_VALUES
{
  IMMEDIATELY,
  QUICKLY,                   /*  1.5 seconds */
  MODERATELY = QUICKLY+2,    /*  4.5 seconds */
  SLOWLY = MODERATELY+2,     /*  7.5 seconds */
  V_SLOWLY = SLOWLY+2,       /* 10.5 seconds */
  NUM_ACCELERATIONS,

  TEST_ACCEL = V_SLOWLY      /* this will affect the -tb option */
} ;

typedef enum ACCELERATION_VALUES ACCELERATION ;

/******************************************************************************
 * The ENGINEER_STATE is a set of the possible states which the engineer
 * process may be in at any one time.
 *
 * The ACC_STATE is the state of acceleration of the train.
 *
 * The DIR_STATE holds the direction of the locomotive - forwards or reverse.
 *   This is an important parameter becuase it governs how the power supplies
 *   need to be reserved and held by the engineer processes.
 */

enum ENGINEER_STATE_VALUES
{
  UNINITIALISED,
  READY_TO_START,
  RUNNING
} ;

enum DIRCHNG_STATE_VALUES
{
  NO_STATE,
  ENTRY,
  EXIT,
} ;

enum ACC_STATE_VALUES
{
  ACCELERATING,
  AT_SPEED,
  DECELERATING,
  STOPPED,              /* Values including and past STOPPED cause engineers */
  DECELERATING_TO_HALT, /* to suspend normal route interpretation */
  DECELERATING_TO_STOP  /* to suspend normal route interpretation */
} ;

enum DIR_STATE_VALUES
{
  FORWARDS,
  REVERSE
} ;

typedef enum ENGINEER_STATE_VALUES ENGINEER_STATE ;
typedef enum DIRCHNG_STATE_VALUES DIRCHNG_STATE ;
typedef enum ACC_STATE_VALUES ACC_STATE ;
typedef enum DIR_STATE_VALUES DIR_STATE ;

/******************************************************************************
 * CONTROLLER_SUBCMD is the specific sub command to execute
 */

enum CONTROLLER_SUBCMD_VALUES
{
  REQUEST,
  RESPONSE,
  DELAY,
  NOTIFICATION,
  RETURN
} ;

typedef enum CONTROLLER_SUBCMD_VALUES CONTROLLER_SUBCMD ;

/******************************************************************************
 * The TRACK_OBJECT data type is a structure holding a value representing the
 * selected power source and the polarity setting for a track segment.
 */

enum TRACK_POWER_SOURCE_VALUES
{
  PSU_1,
  PSU_2
} ;

typedef enum TRACK_POWER_SOURCE_VALUES TRACK_POWER_SOURCE ;

enum TRACK_POLARITY_VALUES
{
  NULL_POLARITY,
  NORMAL_POLARITY,
  OPPOSITE_POLARITY
} ;

typedef enum TRACK_POLARITY_VALUES TRACK_POLARITY ;

struct TRACK_OBJECT_VALUES
{
  TRACK_POWER_SOURCE psu ;
  TRACK_POLARITY     polarity ;
} ;

typedef struct TRACK_OBJECT_VALUES TRACK_OBJECT ;

/******************************************************************************
 * The PSU_OBJECT data type is the set of values representing the values
 * required to translate from a speed number to PWM / DC settings,
 *
 * The POWER_STATE_INFO type is a set of parameters which uniquely defines
 * the state that each power supply process is in.
 */

struct PSU_OBJECT_VALUES
{
  Uchar  pwm_setting ;
  Uchar  dc_setting ;
} ;

typedef struct PSU_OBJECT_VALUES PSU_OBJECT ;

struct POWER_STATE_INFO_STRUCT
{
  int          owner ;
  Ushort       speed ;
  Ushort       target_speed ;
  ACCELERATION acceleration ;
} ;

typedef struct POWER_STATE_INFO_STRUCT POWER_STATE_INFO ;

/******************************************************************************
 * The following structures are used to define the layout attributes including:
 *   how track blocks connect together to form the basic layout
 *   the lengths of each track block
 *   which sets of points are connected to each track block
 *   how points need to be configured to move from one track block to the next
 * etc
 *
 * The easiest way of understanding how to use these structures is to look at
 * an example and then think about it. Refer to <<config.h>> and expect to be
 * a little confused for a little while.
 */

#define MAX_ADJACENCIES        (MAX_CASCADED_POINTS+2)
#define MAX_NAME_LEN           32  /* characters */

struct POINT_SETTING_STRUCT
{
  Ushort        point_num ;    /* identity of the set of points (turnout) */
  POINTS_OBJECT direction ;    /* whether the point should be set or reset */
} ;

typedef struct POINT_SETTING_STRUCT POINT_SETTING ;
#define BOUNDARY  {0,UNKNOWN_STATE}

struct ADJACENCY_STRUCT
{
  Ushort          adj_seg ;   /* seg to which current seg is connected */
  Ushort          length ;    /* distance from centre to the boundary */
  TRACK_POLARITY  polarity ;  /* setting in order to reach this next segment */
  POINT_SETTING   points[MAX_CASCADED_POINTS] ;
                              /* list of point settings for this exit */
} ;

typedef struct ADJACENCY_STRUCT ADJACENCY ;

struct _BLOCK_STRUCT
{
  char          name [MAX_NAME_LEN] ;
  ADJACENCY     adj_seg [MAX_ADJACENCIES] ;
} ;

typedef struct _BLOCK_STRUCT BLOCK_STRUCT ;

/******************************************************************************
 * The following structures define a pro forma structure which will define the
 * bit order for the serial output and input rings.
 */

struct SERIAL_OUT_STRUCT
{
  RING_IN_TYPE  type ;
  Uchar         id ;
} ;

typedef struct SERIAL_OUT_STRUCT SERIAL_OUT ;

/*
struct SET_SERIAL_OUT_STRUCT
{
  SERIAL_OUT                     object ;
  union
  {
    POINTS_OBJECT                point_setting ;
    TRACK_POLARITY               polarity_setting ;
    TRACK_POWER_SOURCE           power_source ;
    PSU_OBJECT                   power_setting ;
  } options ;
} ;

typedef struct SET_SERIAL_OUT_STRUCT SET_SERIAL_OUT ;
*/
struct SERIAL_IN_STRUCT
{
  RING_IN_TYPE  type ;
  Uchar         id ;
} ;

typedef struct SERIAL_IN_STRUCT SERIAL_IN ;

/******************************************************************************
 * These definitions are used to specify different reasons that an engineer
 * can enter the wait state. They are bit fields intended for a 16 bit variable
 */

#define POINTS_WAIT      1
#define CONTROLLER_WAIT  2
#define HALT_WAIT        4

/******************************************************************************
 * The following structure is used to hold information so that the status
 * scanning process can send messages to different processes when various input
 * bits change.
 *
 * The status scanning process needs to know the message number and the message
 * queue id to which to send the message.
 */

struct STATUS_NOTIFICATION_STRUCT
{
  long sub_command ; /* the type of message to send */
  int  notify_id ;   /* message queue ID to which to send the notification */
} ;

typedef struct STATUS_NOTIFICATION_STRUCT NOTIFIER ;

/******************************************************************************
 * This route plan structure is used by each engineer to log their present
 * position and to register their intended next blocks en route.
 *
 * NUM_PSU_ALLOCATIONS is defined in controller.h
 */

#define MAX_TRAIN_BLOCKS    3 /* This must be less than or equal to MIN_NEXT_BLOCKS */
#define MAX_OCCUPIED_BLOCKS (MAX_TRAIN_BLOCKS+2)
#define NUM_STUB_BLOCKS     4 /* This must be equal to or greater than MAX_OCCUPIED_BLOCKS */
#define NUM_PREV_BLOCKS     (MAX_OCCUPIED_BLOCKS + NUM_STUB_BLOCKS)
#define MIN_NEXT_BLOCKS     6 /* This must be MORE than 1, pref 2*MAX_TRAIN_BLOCKS */
#define MAX_BLK_LOOKAHEAD   4
#define NUM_NEXT_BLOCKS     (MIN_NEXT_BLOCKS+MAX_BLK_LOOKAHEAD)
#define NUM_GRANTS          3 /* The max number of blocks granted at a time */
#define REVERSE_LOOKAHEAD   2 /* must be MORE than 1 & LESS than 0.5*MIN_NEXT_BLOCKS */

struct ROUTE_PLAN_STRUCT
{
  Ushort        train_num ;           /* The first entries in this structure */
  DIR_STATE     direction ;           /* are important to be mirrored by the */
  float         circumstantial_speed ;/* controller process                  */
  float         speed_scale ;         /* scales speed in route file per loco */
  float         last_speed ;
  ACC_STATE     acc_state ;
  ACCELERATION  acc ;
  DIRCHNG_STATE dirchng ;
  DIRCHNG_STATE stat_chng_event ;
  Ushort        expect_block ;
  Ushort        length ;
  Ushort        num_blocks ;          /* number of blocks occupied by train  */
  Ushort        num_rev ;             /* num blocks allocated to rear train  */
  Ushort        num_rev_route ;       /* num blocks assigned to rear route   */
  Ushort        blocks [NUM_PREV_BLOCKS] ;
  Ushort        num_next ;
  Ushort        next [NUM_NEXT_BLOCKS] ;
  float         next_speed [NUM_NEXT_BLOCKS] ;
  Ushort        num_reservations ;
  Ushort        PSU [NUM_PSU_ALLOCATIONS] ;
  Ushort        block_grants [NUM_GRANTS] ;
  Ushort        block_returns [NUM_GRANTS] ;
  Ushort        psu_grants [NUM_PSU_ALLOCATIONS] ;
  Ushort        num_stop ;
  Ulong         point_index ; /* only accessed by eng process */
  Ulong         point_wait ;
  Ushort        wait_fields ;
} ;

typedef struct ROUTE_PLAN_STRUCT ROUTE_PLAN ;

/******************************************************************************
 * DIFFERENT SYSTEM MESSAGES TYPES AND STRUCTURES ARE DEFINED HERE.
 *
 * Each type of message has a different set of message values and structure.
 */

typedef struct msgbuf      LINUX_MSG_BUF ;

#define LOG_MSG_SIZE    100     /* characters maximum */

struct LOG_MSG_STRUCT
{
  long         mtype ;             /* a unique id */
  LOG_PRIORITY pri ;
  char         msg [LOG_MSG_SIZE+1] ;
} ;

typedef struct LOG_MSG_STRUCT LOG_MSG ;

struct GENERIC_MSG_STRUCT
{
  long       mtype ;             /* a unique id */
  int        sub_command ;       /* additional message classification */
  int        reply_to ;          /* the queue id to which to send replies */
} ;

#define MSG_OVERHEAD  (sizeof(long))  /* the mtype field is overhead to LINUX */
typedef struct GENERIC_MSG_STRUCT  GENERIC_MSG ;

#include "config.h"             /* for NUM_SERIAL_*_BITS */

enum MY_MESSAGE_TYPES
{
  LOG_MESSAGE=1,                /* messages to the logging process      */
  CLOSE_LOG,                    /* stop logging                         */
  PARALLEL_PORT_RESPONSE,       /* messages from port driver process    */
  PARALLEL_PORT_REQUEST,        /* messages to port driver process      */
  TICK_EVENT,                   /* the timekeeper is getting a tick     */
  TIMEOUT_REQUEST,              /* messages to the timekeeper           */
  TIMEOUT_NOTIFICATION,         /* messages from the timekeeper         */
  DISPATCHER_REQUEST,           /* messages to the dispatcher process   */
  POINTS_REQUEST,               /* engineer to one of the points procs  */
  HANDOVER_REQUEST,             /* engineer to a power supply process   */
  SHUTDOWN_REQUEST,             /* controller to a power supply process */
  SET_SPEED_REQUEST,            /* engineer to a power supply process   */
  BLOCK_NOTIFICATION,           /* messages from the scanner process    */
  SPEED_NOTIFICATION,           /* dispatcher to a train process        */
  POINT_NOTIFICATION,           /* points proc to a train process       */
  CONTROLLER_MESSAGE,           /* messages between eng & controller    */
  INITIALISE_ENGINEER,          /* user input initialising the engineer */
  CHANGE_SPEED_SCALE,           /* user input changing the speed scale  */
  START_ENGINEER,               /* user input starting all engineers    */
  HALT_ENGINEER,                /* user input stopping a single engineer*/
  STOP_ENGINEER,                /* user input stopping all engineers    */
  NUM_MY_MESSAGES
} ;

typedef enum MY_MESSAGE_TYPES MSG_TYPE ;

/*-----------------------------------------------------------------------------
 * Info relating to the port driver process
 */

enum PORT_DRIVER_MSG_VALUES
{
  LED1_OFF=1,
  LED1_ON,
  LED2_OFF,
  LED2_ON,
  SEND_COMMAND,
  RECEIVE_STATUS,
  INITIALISE_PORT_DRIVER
} ;

struct PORT_IN_MSG_STRUCT
{
  GENERIC_MSG  header ;
  Uchar        input_bytes  [NUM_SERIAL_IN_BITS] ;
} ;

struct PORT_OUT_MSG_STRUCT
{
  GENERIC_MSG  header ;
  Uchar        output_bytes [NUM_SERIAL_OUT_BITS] ;
} ;

typedef struct PORT_IN_MSG_STRUCT  PORT_IN_MSG ;
typedef struct PORT_OUT_MSG_STRUCT PORT_OUT_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the timekeeper process
 */

struct TIMER_MSG_STRUCT
{
  GENERIC_MSG  command ;
  Ushort       interval ;          /* measured in TICKS */
} ;

typedef struct TIMER_MSG_STRUCT   TIMER_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the status scanner process
 */

struct STATUS_MSG_SET_STRUCT
{
  GENERIC_MSG  header ;
  SERIAL_IN    object ;
  NOTIFIER     notify ;
} ;

typedef struct STATUS_MSG_SET_STRUCT  SET_STATUS_MSG ;

struct STATUS_MSG_NOTIFY_STRUCT
{
  GENERIC_MSG  header ;
  RING_IN_TYPE type ;
  Ushort       id ;
  Uchar        state ;
} ;

typedef struct STATUS_MSG_NOTIFY_STRUCT  NOTIFY_STATUS_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the dispatcher process
 */

struct DISP_MSG_STRUCT
{
  GENERIC_MSG  header ;
  SERIAL_OUT   object ;
} ;

typedef struct DISP_MSG_STRUCT  DISP_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the points process
 */

struct POINTS_MSG_STRUCT
{
  GENERIC_MSG    command ;
  POINT_SETTING  setting ;
  Ulong          index ;
} ;

typedef struct POINTS_MSG_STRUCT   POINTS_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the power process
 */

struct POWER_MSG_STRUCT
{
  GENERIC_MSG    command ;
  TRACK_POLARITY polarity ;
  Ushort         speed ;
  ACCELERATION   acceleration ;
} ;

typedef struct POWER_MSG_STRUCT   POWER_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the controller process
 */

struct CONTROLLER_MSG_STRUCT
{
  GENERIC_MSG    command ;
  Ushort         train_number ;
} ;

typedef struct CONTROLLER_MSG_STRUCT   CONTROLLER_MSG ;

/*-----------------------------------------------------------------------------
 * Info relating to the engineer processes
 */

struct ENGINEER_MSG_STRUCT
{
  GENERIC_MSG    command ;
  Ushort         speed ;
  Ushort         psu ;
} ;

typedef struct ENGINEER_MSG_STRUCT   ENGINEER_MSG ;

struct INIT_ENGINEER_MSG_STRUCT
{
  GENERIC_MSG    command ;
  Ushort         route_id ;
  Ushort         num_blocks ;
  Ushort         block [MAX_TRAIN_BLOCKS] ;
  float          speed_scale ;
} ;

typedef struct INIT_ENGINEER_MSG_STRUCT   INIT_ENGINEER_MSG ;

/*-----------------------------------------------------------------------------
 * All the message unions are defined below - these have to be at the end of
 * the message definitions becuase the compiler doesn't like forward references
 * to structures.
 */

union _PORT_MSG_UNION
{
  GENERIC_MSG   command ;
  PORT_IN_MSG   in_info ;
  PORT_OUT_MSG  out_info ;
} ;

union _STATUS_MSG_UNION
{
  GENERIC_MSG       command ;
  SET_STATUS_MSG    set ;
  NOTIFY_STATUS_MSG notify ;
  TIMER_MSG         timer ;
} ;

union _DISP_MSG_UNION
{
  GENERIC_MSG   command ;
  DISP_MSG      request ;
  TIMER_MSG     timer ;
} ;

union _POWER_MSG_UNION
{
  GENERIC_MSG       command ;
  POWER_MSG         power ;
  TIMER_MSG         timer ;
  ENGINEER_MSG      engineer ;
} ;

union _CONTROLLER_MSG_UNION
{
  GENERIC_MSG       command ;
  CONTROLLER_MSG    controller ;
  ENGINEER_MSG      engineer ;
  NOTIFY_STATUS_MSG status ;
} ;

union _ENGINEER_MSG_UNION
{
  GENERIC_MSG         command ;
  ENGINEER_MSG        engineer ;
  CONTROLLER_MSG      controller ;
  INIT_ENGINEER_MSG   init ;
  NOTIFY_STATUS_MSG   status ;
  TIMER_MSG           timer ;
  POINTS_MSG          points ;
} ;

union  _ALL_MESSAGES
{
  GENERIC_MSG         command ;
  PORT_IN_MSG         in_info ;
  PORT_OUT_MSG        out_info ;
  TIMER_MSG           timer_msg ;
  SET_STATUS_MSG      setstatus_msg ;
  NOTIFY_STATUS_MSG   notstatus_msg ;
  DISP_MSG            disp_msg ;
  POINTS_MSG          points_msg ;
  POWER_MSG           power_msg ;
  CONTROLLER_MSG      controller_msg ;
  ENGINEER_MSG        engineer_msg ;
  INIT_ENGINEER_MSG   init_engineer_msg ;
} ;

typedef union _PORT_MSG_UNION        PORT_MSG ;
typedef union _STATUS_MSG_UNION      STATUS_MSG_UNION ;
typedef union _DISP_MSG_UNION        DISP_MSG_UNION ;
typedef union _POWER_MSG_UNION       POWER_MSG_UNION ;
typedef union _CONTROLLER_MSG_UNION  CONTROLLER_MSG_UNION ;
typedef union _ENGINEER_MSG_UNION    ENGINEER_MSG_UNION ;
typedef union _ALL_MESSAGES          ALL_MESSAGES ;

/******************************************************************************
 * Process definitions
 *
 * In any train controller system, there are a number of fixed processes which
 * run regardless of the number of train controller cards which have been
 * implemented. These fixed processes are listed in the enumeration. There are
 * also a number of other processes which run, the number of these depending
 * on the number of train controller cards within the system. For example,
 * each train controller card has one points supply which needs indivitual
 * management plus the number of independent locomotives is a function of
 * the number of train controller cards: 1 more than the number of cards.
 */

enum FIXED_PROCESSES
{
  LOG_PROC,             /* This must be the first process */
  PORT_DRIVER_PROC,     /* This must be the second process */
  STATUS_SCAN_PROC,
  DISPATCHER_PROC,
  TIMEKEEPER_PROC,
  CONTROLLER_PROC,
  USER_INPUT_PROC,
  NUM_FIXED_PROC
} ;

#define FIRST_POINTS_PROC       NUM_FIXED_PROC
#define LAST_POINTS_PROC        (FIRST_POINTS_PROC+NUM_POINTS_SUPPLIES-1)

#define FIRST_POWER_PROC        (LAST_POINTS_PROC+1)
#define LAST_POWER_PROC         (FIRST_POWER_PROC+NUM_POWER_SOURCES-1)

/*
 * Processes numbered after the points and power processes will not be launched
 * when any of the test options are selected
 */

#define FIRST_ENGINEER_PROC     (LAST_POWER_PROC+1)
#define NUM_ENGINEER_PROCESSES  ((NUM_POWER_SOURCES > 2) ? (NUM_POWER_SOURCES-1) : 2)
#define LAST_ENGINEER_PROC      (FIRST_ENGINEER_PROC+NUM_ENGINEER_PROCESSES-1)

#define NUM_PROC                (LAST_ENGINEER_PROC+1)

/*
 * Some other macros and constants
 */

#define CARD_FROM_BLOCK(A)      ((A-1) / NUM_BLOCKS_PER_CONTROLLER_CARD)
#define CARD_FROM_PSU(A)        ((A-1) >> 1) /* assuming 2 PSUs per card */
#define PSU_NUMBER(A)           (((A-1) & 0x01) ? PSU_2 : PSU_1)

#define PORT_SLEEP              1                /* second */
#define USER_SLEEP              (PORT_SLEEP+2)   /* seconds */

/******************************************************************************
 * The following information is used to map process information to process id
 */

#define PROCESS_NAME_SIZE 11

struct PROC_INFO_STRUCT
{
  char     name [PROCESS_NAME_SIZE+1] ;
  int      pid ;
  int      msgq1 ;
  int      msgq2 ;
} ;

typedef struct PROC_INFO_STRUCT PROC_INFO ;

#endif
