/*
 * train
 * Copyright (C) 1999  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. http://www.gnu.org
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 * train.c
 *
 * Programme to control a model train set from the PC's parallel port using
 * the train controller circuit published by S.Keller-Tuberg.
 *
 * Written to run under the Linux operating system but could be easily hacked
 * to run under one of Linux's lesser counterparts.
 *
 * Compile using gcc -O2 -o train train.c
 *
 * Needs to be run with root priviledges. Most easily achieved by running
 * (if compiled by a non root user)
 *      chown root train
 *      chgrp root train
 * and then
 *      chmod 4755 train
 */

#include <sys/io.h>           /* for ioperm */
#include <signal.h>           /* for sigaction, signal, alarm, ... */
#include <sys/time.h>         /* for setitimer, ... */
#include <sys/ipc.h>          /* for ftok, semctl, ... */
#include <sys/msg.h>          /* for msgget, ... */
#include <sys/types.h>        /* for wait, semctl, ... */
#include <sys/wait.h>         /* for wait */
#include <sys/shm.h>          /* for shmget, shmat, shmctl etc */
#include <sys/sem.h>          /* for semget, semop, semctl etc */
#include <stdio.h>
#include <unistd.h>           /* for calls to ioperm, sleep, usleep */
#include <stdlib.h>
#include <errno.h>            /* for definitions of sys call error numbers */
#include <sched.h>            /* for sched_setscheduler, ... */
#include <stdarg.h>           /* for calls to vsnprintf etc */
#include <string.h>           /* for strncpy, memcpy, memset, ... */
#include <ctype.h>            /* for toupper, ... */

/* Uncomment the following if your compiler complains about missing information */
/* #include <asm/io.h>           for inb, outb */


#include "controller.h"       /* for ring and controller specific info */
#include "config.h"           /* for layout specific configuration options */
#include "train.h"            /* for definitions of types and structures */

/******************************************************************************
 */

#define  BasePort     (0x378) /* Address of parallel port to be used */

#define  Cmd_Data_WR  (0x01)
#define  Cmd_Data_RD  (0x10)

#define  Stat_Data_WR (0x02)
#define  Stat_Data_RD (0x20)

#define  Latch_WR     (0x04)
#define  Latch_RD     (0x40)

#define  Clock_WR     (0x08)
#define  Clock_RD     (0x08)

#define  Power_On     (0x20)
#define  Led1         (0x40)
#define  Led2         (0x80)

#define  Ok           (0)
#define  Bad          (-1)

/******************************************************************************
 * All the message queue IDs, semaphore IDs and shared memory segment IDs are
 * global and are declared here. All other variables are local and known only
 * to the process which defines them.
 */

static int   logfile_msgq ;
static int   port_driver_msgq ;
static int   scan_msgq ;
static int   status_recv_msgq ;
static int   dispatcher_msgq ;
static int   timer_msgq ;
static int   controller_msgq ;
static int   test_msgq ;
static int   points_msgq [NUM_POINTS_SUPPLIES] ;
static int   power_msgq [NUM_POWER_SOURCES] ;
static int   engineer_msgq [NUM_ENGINEER_PROCESSES] ;
static int   user_msgq ;

/******************************************************************************
 * This section of declarations are for the pointers to shared memory and
 * semaphores.
 */

static          int       Sys_Time_id ;
static          SYS_TIME  *Sys_Time ;
static          int       Notifier_Sem ;
static          int       Not_Sem ;
static          int       Notifier_id ;
static          NOTIFIER  (*Notifier)[] ;           /* pointer to array of NOTIFIER */
static          int       Not_Block_Det_id ;
static          NOTIFIER  *(*Not_Block_Det)[] ;     /* pointer to array of pointers to NOTIFIER */
static          int       Not_GP_Input_id ;
static          NOTIFIER  *(*Not_GP_Input)[] ;      /* pointer to array of pointers to NOTIFIER */
static          int       Not_Pow_Overload_id ;
static          NOTIFIER  *(*Not_Pow_Overload)[] ;  /* pointer to array of pointers to NOTIFIER */

static          int       Block_Allocation_Sem ;
static          int       Block_Allocation_id ;
static          int       (*Block_Allocation)[] ;    /* pointer to array of int */

static          int       Block_From_Points_Sem ;
static          int       Block_From_Points_id ;
static          int       (*Block_From_Points)[] ;  /* pointer to array of int */

static          int       Power_State_Sem [NUM_POWER_SOURCES] ;
static          int       Power_State_id ;
static          POWER_STATE_INFO (*Power_State)[] ;  /* pointer to array of POWER_STATE_INFO */

static          int       Adjacency_Sem ;
static          int       Adjacency_id ;
static          ADJACENCY (*(*Adjacency)[NUM_BLOCKS][DEAD_END]) ;    /* pointer to array of pointers to ADJACENCY */

static          int        Plan_Sem [NUM_ENGINEER_PROCESSES] ;
static          int        Plan_id ;
static          ROUTE_PLAN (*Plan)[] ;                               /* pointer to array of pointers to ROUTE_PLAN */

static          int       Status_Indicators_Sem ;
static          int       Block_Occupancy_id ;
static          Uchar     (*Block_Occupancy)[] ;    /* pointer to array of Uchar */
static          int       GP_Inputs_id ;
static          Uchar     (*GP_Inputs)[] ;    /* pointer to array of Uchar */
static          int       Overload_Status_id ;
static          Uchar     (*Overload_Status)[] ;    /* pointer to array of Uchar */

static          int       Process_Info_Sem ;
static          int       Process_Info_id ;
static          PROC_INFO (*Proc_Info)[] ;         /* pointer to array of PROC_INFO */

/******************************************************************************
 * This area defines a bunch of things relating to files.
 *
 * The extract_filename routine has some definitions relating to MSDOS. If you
 * want to use this routine under DOS, these will make it easier for you.
 * The train software however has not been written to run in the DOS / WINDOWS
 * environment.
 */

#ifdef MSDOS
#define	Dir_Delimiter		'\\'
#define	Ext_Delimiter		'.'
#define	File_Name_Size		200
#else
#define	Dir_Delimiter		'/'
#define	Ext_Delimiter		'.'
#define	File_Name_Size		200
#endif

char	Directory [File_Name_Size] ;
char    *Filename, *Extension ;
char	Filename_String  [File_Name_Size] ;
char	Extension_String [File_Name_Size] ;

/*******************************************************************************
 * Here are other definitons which don't fall into the above categories.
 */

char    Process_Name [PROCESS_NAME_SIZE+1] ;
char    *Logfilename ;
int     Verbose ;
int     Error ;
int     Test_Points ;
int     Test_Supply ;
int     Test_Block ;
Ushort  Test_Block_Delay ;
int     Test_Ring ;
int     Reset_Ring ;
Ushort  Block_Reservation [NUM_BLOCKS][NUM_RESERVATIONS] ;
Ushort  PSU_Reservation [NUM_PSU_ALLOCATIONS][NUM_RESERVATIONS] ;

#define Delay_Period    35000 /* microseconds */

#define BLK_OWNER(A)    ((*Block_Allocation) [(A)-1])
#define POW_OWNER(A)    ((*Power_State) [(A)-1].owner)
#define ADJ(A, B)       ((*Adjacency)[(A)-1][(B)-1])
#define BLOCKS_POLARITY(FROM, TOWARDS)                (ADJ((FROM), (TOWARDS))->polarity)
#define OPPOSITE_BLOCKS_POLARITY(FROM, TOWARDS)       (((BLOCKS_POLARITY(FROM, TOWARDS)) == NORMAL_POLARITY) ? OPPOSITE_POLARITY : NORMAL_POLARITY)
#define HEADING_SAME_DIRECTION(PREV,CURRENT,NEXT)     (ADJ((CURRENT), (PREV))->polarity != ADJ((CURRENT), (NEXT))->polarity)

/*******************************************************************************
 * log_msg
 *
 * This routine is called from all over the place. It sends a message to the
 * logging process. For more information, see the logging process header below
 */

void
log_msg (const LOG_PRIORITY priority, const char *fmt, ...)
{
  int		n, m ;
  va_list	args ;
  LOG_MSG       msg ;

  va_start(args, fmt) ;
  n = sprintf (msg.msg, "%-*.*s:%4lu.%02hu: ", PROCESS_NAME_SIZE, PROCESS_NAME_SIZE, Process_Name, Sys_Time->seconds, (Ushort)(FRAC_TIME(Sys_Time->ticks))) ;

  if (priority == ERR)
  {
    strcat (msg.msg, "*") ;
    n = strlen (msg.msg) ;
  }

  m = vsnprintf (&msg.msg[n], LOG_MSG_SIZE-2-n, fmt, args) ; /* changed from -1 due to sig11*/

  if (m == -1)
  {
    log_msg (NORM, "Following msg truncated to %d chars", LOG_MSG_SIZE) ;
    n = LOG_MSG_SIZE-2 ;
  }
  else
    n += m ;

  msg.msg[n++] = '\n' ;
  msg.msg[n] = '\0' ;
  va_end(args) ;
  msg.mtype = LOG_MESSAGE ;
  msg.pri = priority ;

  while (msgsnd (logfile_msgq, (LINUX_MSG_BUF *)&msg, n+sizeof(msg.pri)+1, 0) == -1)
  {
    if (errno != EINTR)
    {
      perror ("Unexpected error when writing to log msg queue") ;
      fprintf (stderr, "Failed message: %s", (char *)&msg.msg[0]) ;
      fprintf (stderr, "%s: Msg_buf log msg error location: %p\n", Process_Name, &msg) ;
      _exit (2) ;
    }

    usleep (Delay_Period) ;
  }
}

/*******************************************************************************
 * extract_filename
 *
 * takes a system file specification and sets the global variables <Directory>,
 * <Filename> and <Extension>.
 *
 * eg.... c:\top\second\third\file.ext
 *
 * Directory == "c:\top\second\third\"
 * Filename == "file"
 * Extension == ".ext"
 */

void
extract_filename (const char *fspec)
{
  unsigned long	i, last_file, last_ext ;

  strncpy (Directory, fspec, File_Name_Size-1) ;
  strncpy (Filename_String, fspec, File_Name_Size-1) ;
  strncpy (Extension_String, fspec, File_Name_Size-1) ;

  last_file = last_ext = 0 ;

  for (i=0 ; fspec[i] != '\0' ; i++)
  {
    if (fspec[i] == Dir_Delimiter)
      last_file = i+1 ;

    if (fspec[i] == Ext_Delimiter)
    {
      last_ext = i ;
      Filename_String [i] = '\0' ;
    }
  }

  if ((last_ext < last_file) || (last_ext == 0))
  {
    last_ext = 0 ;
    Extension_String [0] = '\0' ;
  }

  Directory [last_file] = '\0' ;
  Filename = &Filename_String [last_file] ;
  Extension = &Extension_String [last_ext] ;
}

/******************************************************************************
 * create_mesg, read_mesg, send_mesg, rm_mesg
 * create_sem, lock_sem, unlock_sem, rm_sem
 * create_shm, rm_shm
 *
 * These are housekeeping routines to make the tasks of managing Linux's
 * interprocess communications a little easier.
 *
 * The read_mesg routine is blocking and will cause a process to go to
 * sleep until a message arrives. Same for lock_sem.
 *
 * The send_mesg routine is also potentially blocking if there is no room left
 * in the kernel's message buffer area for another message to be written.
 */

void
set_process_priority (const int policy, const int priority)
{
  struct sched_param param ;

  param.sched_priority = sched_get_priority_min(policy) + priority ;

  if (sched_setscheduler (0, policy, &param) == -1)
  {
    log_msg (ERR, "Error setting scheduler. Are you root or suid root?") ;
    _exit (0) ;
  }

  if (sched_setparam (0, &param) == -1)
  {
    log_msg (ERR, "Error setting process priority to %d", param.sched_priority) ;
    _exit (0) ;
  }

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "Process priority=%d, scheduler=%d", param.sched_priority, policy) ;
  #endif
}

int
create_mesg (const char token)
{
  key_t         key ;
  int           msgq_id ;
  ALL_MESSAGES  null_buffer ;

  /*
   * In order to open a message queue, we first need a magic number with which
   * the operating system can identify the queue. We make this key using the
   * ftok call which convolutes the major and minor inode numbers of the
   * current durectory "." with another token which is specific to this
   * software.
   *
   * When we open the message queue, we also check to see if it is empty.
   * If not empty, we assume that it is still hanging around from a previous
   * (crashed) instance of this programme and we need to clear out all the
   * old messages before this programme can work properly.
   */

  key = ftok (".", token) ;

  if ((msgq_id = msgget (key, IPC_CREAT | 0600)) == -1)
  {
    perror ("Can't create message queue") ;
    fprintf (stderr, "Process: %s, key: %d\n", Process_Name, key) ;
    exit (3) ;
  }

  #ifdef MESSAGE_TRACE
    log_msg (FILE_ONLY, "Created message queue token '%c', id %d", token, msgq_id) ;
  #endif

  while (msgrcv (msgq_id, &null_buffer, sizeof(null_buffer)-MSG_OVERHEAD, 0, IPC_NOWAIT|MSG_NOERROR) != -1) ;

  if (errno != ENOMSG)
  {
    perror ("unexpected error when flushing message queue") ;
    fprintf (stderr, "Process: %s, queue id: %d\n", Process_Name, msgq_id) ;
    exit (4) ;
  }

  return (msgq_id) ;
}

int
create_sem (const char token)
{
  key_t         key ;
  int           sem_id ;
  int           arg ;

  key = ftok (".", token) ;

  if ((sem_id = semget (key, 1, IPC_CREAT | 0600)) == -1)
  {
    perror ("Can't create semaphore") ;
    fprintf (stderr, "Process: %s, key: %d\n", Process_Name, key) ;
    exit (5) ;
  }

  #ifdef SEM_TRACE
    log_msg (FILE_ONLY, "Created semaphore token '%c', id %d", token, sem_id) ;
  #endif

  /*
   * The semaphore is created. But it could have been hanging
   * around from a previous instance of this programme.
   *
   * When a semaphore is created, its count should be initialised to 0. A
   * count of zero means that all processes which call unlock will be suspended
   * until another process sets the count above 0.
   *
   * Because the semaphore may not be newly created (such as if there was a
   * previous instance of the semaphore haning around from a previous crash)
   * then the count may not be 0 upon return. The only reliable fix for this
   * I have found is as follows.
   */

  arg = 0 ;

  if (semctl (sem_id, 0, SETVAL, arg) == -1)
  {
    perror ("Can't initialise semaphore to 0") ;
    fprintf (stderr, "Process: %s, key: %d, id: %d\n", Process_Name, key, sem_id) ;
    exit (6) ;
  }

  return (sem_id) ;
}

int
create_shm (const char token, const int size, void **shm_addr)
{
  key_t         key ;
  int           shm_id ;

  key = ftok (".", token) ;

  if ((shm_id = shmget (key, size, IPC_CREAT | 0600)) == -1)
  {
    perror ("Can't create shared memory segment") ;
    fprintf (stderr, "Process: %s, key: %d, token '%c'\n", Process_Name, key, token) ;
    exit (7) ;
  }

  #ifdef SHM_TRACE
    log_msg (FILE_ONLY, "Created shared memory segment token '%c', Shared Memory id %d", token, shm_id) ;
  #endif

  if ((*shm_addr = (void *)shmat(shm_id, NULL, 0)) == (void *)-1)
  {
    perror ("Can't attach to shared memory segment") ;
    fprintf (stderr, "Process: %s, key: %d\n", Process_Name, key) ;
    exit (8) ;
  }

  return (shm_id) ;
}

void
read_mesg (const int queue, LINUX_MSG_BUF *buf, const int max_msg_len)
{
  int len ;

  while ((len = msgrcv (queue, buf, max_msg_len-MSG_OVERHEAD, 0, 0)) == -1)
  {
    if (errno != EINTR)
    {
      perror ("Unexpected error while waiting for msg") ;
      log_msg (ERR, "Queue id=%d, msg_buf loc=%#p, msg len=%d, expected len=%d", queue, buf, len, max_msg_len) ;

      #ifdef PARANOID
        if (errno == E2BIG)
	{
	  #define SIZE_A 1000
	  unsigned char array[SIZE_A] ;
	  GENERIC_MSG   *p ;

	  p = (GENERIC_MSG *)array ;

	  if ((len = msgrcv (queue, array, SIZE_A-MSG_OVERHEAD, 0, MSG_NOERROR)) != -1)
	    log_msg (ERR, "Actual msg size=%d, type=%d, sub_cmd=%d, rep_to=%d", len, p->mtype, p->sub_command, p->reply_to) ;
	}
      #endif

      _exit (0) ;
    }

    usleep (Delay_Period) ; /* delay a while before trying again */
  }

  #ifdef MESSAGE_TRACE
    if (len > sizeof(long))
      log_msg (FILE_ONLY, "%d byte message read from queue id %d", len, queue) ;
  #endif
}

void
send_mesg (const int queue, LINUX_MSG_BUF *buf, const int msg_len)
{
  #ifdef MESSAGE_TRACE
    if (msg_len > sizeof(long))
      log_msg (FILE_ONLY, "%d byte message sent to queue id=%d", msg_len, queue) ;
  #endif

  while (msgsnd (queue, buf, msg_len-MSG_OVERHEAD, 0) == -1)
  {
    if (errno != EINTR)
    {
      perror ("Unexpected error when writing to message queue") ;
      log_msg (ERR, "Queue write error id: %d, msg_buf location: %#p", queue, buf) ;
      _exit (0) ;
    }

    usleep (Delay_Period) ; /* delay a while before trying again */
  }
}

void
unlock_sem (const int sem)
{
  struct sembuf unlock ;

  unlock.sem_num = 0 ;
  unlock.sem_op  = 1 ;
  unlock.sem_flg = 0 ;

  if (semop(sem, &unlock, 1) == -1)
  {
    perror ("Could not unlock semaphore") ;
    log_msg (ERR, "Sem unlock error id=%d", sem) ;
    _exit (0) ;
  }

  #ifdef SEM_TRACE
    log_msg (FILE_ONLY, "Unlocked semaphore id=%d", sem) ;
  #endif
}

void
lock_sem (const int sem)
{
  struct sembuf lock ;
  int           ret ;

  lock.sem_num =  0 ;
  lock.sem_op  = -1 ;
  lock.sem_flg =  0 ;

  /*
   * Neet to implement the following loop as a while loop with explicit delay
   * - because we have many different processes in the system with different
   * priorities. If a lower priority process gets the semaphore and a higher
   * priority process starts waiting, then the lower priority process will
   * never get a chance to give up the semaphore!!
   */

  while ((ret = semop(sem, &lock, 1)) != 0)
  {
    if (errno != EAGAIN)
    {
      perror ("Could not lock semaphore") ;
      log_msg (ERR, "Sem lock error id=%d", sem) ;
      _exit (0) ;
    }

    #ifdef SEM_TRACE
      log_msg (FILE_ONLY, "Looping for sem id=%d", sem) ;
    #endif

    usleep (Delay_Period) ; /* wait a while before trying again */
  }

  #ifdef SEM_TRACE
    log_msg (FILE_ONLY, "Locked semaphore id=%d", sem) ;
  #endif
}

void
rm_mesg (const int queue)
{
  if (msgctl (queue, IPC_RMID, 0) != 0)
  {
    perror ("Couldn't remove a message queue") ;
    fprintf (stderr, "Process: %s, Queue id=%d", Process_Name, queue) ;
    exit (13) ;
  }

  #ifdef MESSAGE_TRACE
    log_msg (FILE_ONLY, "Removed message queue id=%d", queue) ;
  #endif
}

void
rm_sem (const int sem)
{
  if (semctl (sem, 0, IPC_RMID, 0) != 0)
  {
    perror ("Couldn't remove a semaphore") ;
    fprintf (stderr, "Process %s, queue id=%d\n", Process_Name, sem) ;
    exit (14) ;
  }

  #ifdef SEM_TRACE
    log_msg (FILE_ONLY, "Removed semaphore id=%d", sem) ;
  #endif
}

void
rm_shm (const int shm)
{
  if (shmctl (shm, IPC_RMID, NULL) != 0)
  {
    perror ("Couldn't remove a shared memory segment") ;
    fprintf (stderr, "Process %s, shared memory id=%d\n", Process_Name, shm) ;
    exit (15) ;
  }

  #ifdef SHM_TRACE
    log_msg (FILE_ONLY, "Removed shared memory segment id=%d", shm) ;
  #endif
}

void
set_process_info (const char *name, const int pid, const int msgq1, const int msgq2)
{
  Ushort i ;

  lock_sem (Process_Info_Sem) ;

  for (i=0 ; i < NUM_PROC ; i++)
    if ((*Proc_Info)[i].name[0] == '\0')
      break ;

  if (i >= NUM_PROC)
  {
    log_msg (ERR, "Can't find Proc_Info entry for %s", name) ;
    unlock_sem (Process_Info_Sem) ;
    _exit (0) ;
  }

  memcpy ((*Proc_Info)[i].name, name, PROCESS_NAME_SIZE) ;
  (*Proc_Info)[i].pid = pid ;
  (*Proc_Info)[i].msgq1 = msgq1 ;
  (*Proc_Info)[i].msgq2 = msgq2 ;
  unlock_sem (Process_Info_Sem) ;
}

static char err_str[25] ;

char *
proc_from_msgq (const int msgq)
{
  Ushort i ;

  for (i=0 ; i < NUM_PROC ; i++)
    if (((*Proc_Info)[i].msgq1 == msgq) || ((*Proc_Info)[i].msgq2 == msgq))
      break ;

  if (i >= NUM_PROC)
  {
    sprintf (err_str, "Unknown (msgq=%d)", msgq) ;
    return (err_str) ;
  }

  return ((*Proc_Info)[i].name) ;
}

char *
proc_from_pid (const int pid)
{
  Ushort i ;

  for (i=0 ; i < NUM_PROC ; i++)
    if ((*Proc_Info)[i].pid == pid)
      break ;

  if (i >= NUM_PROC)
  {
    sprintf (err_str, "Unknown (pid=%d)", pid) ;
    return (err_str) ;
  }

  return ((*Proc_Info)[i].name) ;
}

#ifdef BASIC_DEBUG

void
decode_io_string (const Ushort num_bits, const Uchar *ptr, const char *format)
{
  #define num_bytes    (((num_bits-1) / 8) + 1)
  #define residue_bits (num_bits % 8)

  Uchar   c, hex [num_bytes] ;
  char    string [(num_bytes*2)+1] ;
  Ushort  i, j, k ;

  for (i=0 ; i < num_bytes ; i++)
  {
    c = 0 ;
    k = ((i == 0) && (residue_bits != 0)) ? residue_bits : 8 ;

    for (j=0 ; j < k ; j++)
    {
      c <<= 1 ;

      if (*ptr++ != 0)
	c |= 1 ;
    }

    hex[i] = c ;
  }

  for (i=0 ; i < num_bytes ; i++)
    sprintf (&string[i*2], "%02x", hex[i]) ;

  string[i*2] = '\0' ;
  log_msg (FILE_ONLY, format, string) ;
}

void
show_blocks (volatile ROUTE_PLAN *ptr)
{
  char    current_blks [NUM_PREV_BLOCKS*3+1] ;
  char    next_blks [NUM_NEXT_BLOCKS*3+1] ;
  char    psus [NUM_PSU_ALLOCATIONS*3+1] ;
  Ushort  i, j ;

  #define None_Str  " (none)"

  strcpy (current_blks, None_Str) ;
  strcpy (next_blks, current_blks) ;
  strcpy (psus, current_blks) ;

  for (j=0, i=ptr->num_blocks ; i-- > 0 ;)
    j += sprintf (&current_blks[j], " %d", ptr->blocks[i]) ;

  for (j=0, i=0 ; i < ptr->num_next ; i++)
    j += sprintf (&next_blks[j], " %d", ptr->next[i]) ;

  for (j=0, i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
    if (ptr->PSU[i] != 0)
      j += sprintf (&psus[j], " %d", ptr->PSU[i]) ;

  log_msg (FILE_ONLY, "Train=%hu on%s len=%hu moving to%s with PSU%s", ptr->train_num, current_blks, ptr->length, next_blks, psus) ;
}

#endif

/*****************************************************************************
 * The following routines simplify the task of formatting and sending messages
 * A number of different routines are defined for different types of messages.
 */

void
send_port_req_msg (const int rep_to, const enum PORT_DRIVER_MSG_VALUES cmd, const Uchar *output)
{
  PORT_MSG   msg ;
  int        size = sizeof(GENERIC_MSG) ;

  if (cmd == SEND_COMMAND)
  {
    memcpy (msg.out_info.output_bytes, output, NUM_SERIAL_OUT_BITS * sizeof (*output)) ;
    size = sizeof(msg) ;

    #ifdef PORT_DEBUG
      decode_io_string (NUM_SERIAL_OUT_BITS, output, "Sending output string '%s'") ;
    #endif
  }

  msg.command.mtype = PARALLEL_PORT_REQUEST ;
  msg.command.sub_command = cmd ;
  msg.command.reply_to = rep_to ;

  #ifdef PORT_DEBUG
    log_msg (FILE_ONLY, "send_port_msg being sent") ;
  #endif

  send_mesg (port_driver_msgq, (LINUX_MSG_BUF *)(&msg), size) ;
}

void
send_dispatcher_req_msg (const int rep_to, const Uchar obj_id, const RING_OUT_TYPE obj_type, const STATE state)
{
  DISP_MSG   msg ;

  msg.header.mtype = DISPATCHER_REQUEST ;
  msg.header.sub_command = state ;
  msg.header.reply_to = rep_to ;
  msg.object.id = obj_id ;
  msg.object.type = obj_type ;

  #ifdef DISPATCHER_DEBUG
    log_msg (FILE_ONLY, "send_dispatcher_req_msg being sent") ;
  #endif

  send_mesg (dispatcher_msgq, (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
}

void
send_timeout_req_msg (int rep_to, Ushort interval, int *id)
{
  TIMER_MSG  msg ;

  msg.command.mtype = TIMEOUT_REQUEST ;
  msg.command.sub_command = (int)id ;
  msg.command.reply_to = rep_to ;
  msg.interval = interval;

  #ifdef TIMER_DEBUG
    log_msg (FILE_ONLY, "send_timeout_req_msg being sent") ;
  #endif

  send_mesg (timer_msgq, (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
}

void
send_points_msg (Ushort points_id, POINTS_OBJECT state, Ulong index, int rep_to)
{
  POINTS_MSG   msg ;
  int          send_queue ;

  #ifdef PARANOID
    if ((points_id == 0) || (points_id > NUM_POINTS))
    {
      log_msg (ERR, "Set_points: tried to set illegal numbered points=%hu", points_id) ;
      _exit (0) ;
    }

    if ((state != OPEN) && (state != CLOSED))
    {
      log_msg (ERR, "Set_points: tried to set illegal points state=%d\n", state) ;
      _exit (0) ;
    }
  #endif

  msg.command.mtype = POINTS_REQUEST ;
  msg.command.reply_to = rep_to ;
  msg.setting.point_num = points_id ;
  msg.setting.direction = state ;
  msg.index = index ;
  send_queue = points_msgq [(points_id - 1) / NUM_POINTS_PER_CONTROLLER_CARD] ;

  #ifdef POINTS_DEBUG
    log_msg (FILE_ONLY, "send_points_msg being sent to %s, point_num=%d, dir=%d", proc_from_msgq(send_queue), msg.setting.point_num, msg.setting.direction) ;
  #endif

  send_mesg (send_queue, (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
}

void
send_power_msg (Ushort power_id, int rep_to, MSG_TYPE cmd, int subcmd, Ushort speed, ACCELERATION acceleration)
{
  POWER_MSG_UNION  msg ;

  #ifdef PARANOID
    if ((power_id == 0) || (power_id > NUM_POWER_SOURCES))
    {
      log_msg (ERR, "send_power_msg: %s addressed illegal PSU number=%d", proc_from_msgq(rep_to), power_id) ;
      _exit (0) ;
    }
  #endif

  msg.command.mtype = cmd ;
  msg.command.sub_command = subcmd ;
  msg.command.reply_to = rep_to ;
  msg.power.speed = speed ;
  msg.power.acceleration = acceleration ;

  #ifdef POWER_DEBUG
    log_msg (FILE_ONLY, "send_power_msg being sent to PSU=%d, speed=%hu, accel=%d", power_id, speed, acceleration) ;
  #endif

  send_mesg (power_msgq [power_id - 1], (LINUX_MSG_BUF *)(&msg), sizeof(POWER_MSG)) ;
}

void
send_controller_msg (int send_to, int rep_to, MSG_TYPE cmd, CONTROLLER_SUBCMD subcmd, Ushort train_number)
{
  CONTROLLER_MSG msg ;

  #ifdef CONTROLLER_DEBUG
    char *c, *s ;
  #endif

  if (cmd == CONTROLLER_MESSAGE)
  {
    #ifdef PARANOID
      if ((train_number == 0) || (train_number > NUM_ENGINEER_PROCESSES))
      {
	log_msg (ERR, "snd_ctrlr_msg: Referenced illegal train number (%d)", train_number) ;
	_exit (0) ;
      }
    #endif

    #ifdef CONTROLLER_DEBUG
      c = "CNTRLR_BLK" ;

      switch (subcmd)
      {
        case REQUEST:
	  s = "REQ" ;
	  break ;

        case RESPONSE:
	  s = "RESP" ;
	  break ;

        case DELAY:
	  s = "DELAY" ;
	  break ;

        case NOTIFICATION:
	  s = "NOTIF" ;
	  break ;

        case RETURN:
	  s = "RETURN" ;
	  break ;

        default:
	  log_msg (ERR, "Unkown snd_ctrlr_msg subcmd=%d", subcmd) ;
	  _exit (0) ;
      }
    #endif

  }
  #ifdef PARANOID
    else
    {
      log_msg (ERR, "snd_ctrlr_msg: Referenced illegal cmd (%d)", cmd) ;
      _exit (0) ;
    }
  #endif

  #ifdef CONTROLLER_DEBUG
    log_msg (FILE_ONLY, "snd_ctrlr_msg cmd=%s sub=%s sent to %s", c, s, proc_from_msgq(send_to)) ;
  #endif

  msg.command.mtype = cmd ;
  msg.command.sub_command = subcmd ;
  msg.command.reply_to = rep_to ;
  msg.train_number = train_number ;
  send_mesg (send_to, (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
}

void
send_engineer_msg (int send_to, int rep_to, MSG_TYPE cmd, int sub)
{
  ENGINEER_MSG_UNION  msg ;

  #ifdef PARANOID
    if ((cmd != START_ENGINEER) && (cmd != STOP_ENGINEER))
    {
      log_msg (ERR, "send_engineer_msg: Specified bad command (%d). rep_to=%d", cmd, rep_to) ;
      _exit (0) ;
    }
  #endif

  #ifdef ENGINEER_DEBUG
    log_msg (FILE_ONLY, "send_engineer_msg being sent to %s. type=%d", proc_from_msgq(send_to), cmd) ;
  #endif

  msg.command.mtype = cmd ;
  msg.command.sub_command = sub ;
  msg.command.reply_to = rep_to ;
  send_mesg (send_to, (LINUX_MSG_BUF *)(&msg), sizeof (msg)) ;
}

/******************************************************************************
 * This routine is a debugging routine which is called if a process dies. It
 * can also be called at any other time.
 *
 * WARNING: Snapshot allocation does not use semaphores to ensure that other
 * processes are not using the memory at the same time. This gets around
 * problems of trying to lock semaphores when another process has taken
 * the semaphore and subsequently died or hung up.
 */

void
snapshot_allocation (int where)
{
  Ushort i ;

  log_msg (where, "Snapshot of block allocations:") ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
    if (BLK_OWNER(i+1) != 0)
      log_msg (where, "Block=%hu allocated to %s", i+1, proc_from_msgq(BLK_OWNER(i+1))) ;

  log_msg (where, "Snapshot of PSU allocations:") ;

  for (i=0 ; i < NUM_POWER_SOURCES ; i++)
    if (POW_OWNER(i+1) != 0)
      log_msg (where, "PSU=%hu allocated to %s", i+1, proc_from_msgq(POW_OWNER(i+1))) ;
}

/******************************************************************************
 * log_process
 *
 * This process is a single point of message logging.
 *
 * This process receives messages and logs them to a file. This may seem a
 * dumb way to do it - but it seemed to be the only way I could reliably get
 * many different processes to simulataneously be writing to a single file
 * and havinf the outputs appear in order and without loss. I know that this
 * could have been done by opening the file in O_SYNC mode (which like stderr,
 * blocks writers until the bytes are actually written to the hardware) but
 * this would not have been good in a real time application where processes
 * need to run quickly.
 *
 * There is probably a better way to do this under Linux of which I am not
 * aware.
 */

int
log_process (const char *filename)
{
  FILE               *logfile ;
  int                len ;
  LOG_MSG            msg ;

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), logfile_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), logfile_msgq, 0) ;

  if (filename != (char *)0)
  {
    printf ("LOG messages being written to %s\n", filename) ;

    if ((logfile = fopen (filename, "w")) == NULL)
    {
      fprintf (stderr, "* Can't open <%s> for writing\n", filename) ;
      logfile = stdout ;
    }
  }
  else
    logfile = stdout ;

  for (;;)
  {
    while ((len = msgrcv (logfile_msgq, (LINUX_MSG_BUF *)&msg, sizeof(msg)-MSG_OVERHEAD, 0, 0)) == -1)
    {
      if (errno != EINTR)
      {
	perror ("Unexpected error while waiting for log msg") ;
	sprintf (msg.msg, "Msg_buf wait error: errno=%#x, loc=%p, msg len=%d, expected len=%d\n", errno, &msg, len, sizeof(msg)) ;
	msg.mtype = LOG_MESSAGE ;
	msg.pri = LOG_ABORT ;
	break ;
      }
    }

    if (msg.mtype == LOG_MESSAGE)
    {
      /*
       * Write the message to the log file and flush it to disk immediately.
       * This will cause this process to block for a short time but will
       * also ensure that if the programme crashes or is abnormally killed,
       * that as many messages as possible will have been written to disk
       *
       * If writing to disk rather than STDOUT, the logfile should always
       * conclude with the close message - this indicates that no messages
       * have been lost because the file was closed before the output
       * buffers were flushed.
       */

      if ((msg.pri == FILE_ONLY) && (logfile == stdout))
	continue ;

      if ((msg.pri == ERR) || (msg.pri == LOG_ABORT))
      {
	fprintf (stderr, "\r%s", msg.msg) ;

	if ((logfile == stdout) && (msg.pri != LOG_ABORT))
	  continue ;
      }

      if ((msg.pri == SCREEN) && (logfile != stdout))
      {
	fputs (msg.msg, stdout) ;
	fflush (stdout) ;
      }

      fputs (msg.msg, logfile) ;
      fflush (logfile) ;

      if (msg.pri == LOG_ABORT)
	goto log_abort ;
    }
   else if (msg.mtype == CLOSE_LOG)
    {
log_abort:
      if (logfile != stdout)
      {
	fputs ("\nLogfile closed normally\n", logfile) ;
	fclose (logfile) ;
      }

      _exit (0) ;
    }
    else
      log_msg (LOG_ABORT, "Illegal message type received %d", msg.mtype) ;
  }
}

/*****************************************************************************
 * port_driver_process
 *
 * This routine is the basis for the port controller process. It is run
 * at a higher priority than the normal process because very short delays
 * are called for from time to time and we don't really want the Linux
 * scheduler to start running another process if the other process cannot
 * be preemptively interrupted.
 */

#define Max_Count   (5000)    /* maximum allowable loops before exiting */

__inline__ int
wait_for_high (const register Uchar bits)
{
  register Ushort counter ;

  for (counter=Max_Count ; counter!=0 ; counter--)
  {
    if (inb(BasePort+1) & bits)
      return (Ok) ;
  }

  return (Bad) ;
}

__inline__ int
wait_for_low (const register Uchar bits)
{
  register Ushort counter ;

  for (counter=Max_Count ; counter!=0 ; counter--)
  {
    if ((inb(BasePort+1) & bits) == 0)
      return (Ok) ;
  }

  return (Bad) ;
}

void
fail_abort (short failure)
{
  log_msg (ERR, "The ring is open or not functioning: %hd", failure) ;

  #ifndef SIMULATE_PORT
    if (ioperm(BasePort, 3, 0))
      perror ("Port Driver: ioperm call failed to close parallel port") ;
  #endif

  _exit (21) ;
}

int
port_driver_process (void)
{
  volatile Uchar     output_contents ;
  PORT_MSG           port_driver_msg ;
  Ushort             bit ;
  Uchar              *ptr ;

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), port_driver_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), port_driver_msgq, 0) ;
  send_port_req_msg (port_driver_msgq, INITIALISE_PORT_DRIVER, (void *)0) ;

  #ifndef SIMULATE_PORT
    /*
     * Open the printer port for access
     */

    if (ioperm(BasePort, 3, 1))
    {
      perror ("Port Driver: ioperm call failed to open parallel port") ;
      return (0) ;
    }
  #endif

  set_process_priority (SCHED_FIFO, 3) ;

  /*
   * Ensure that the clock and latch bits are OFF
   * Also ensure that the power control bit is ON (to enable the ring)
   */

  output_contents = Power_On ;

  #ifndef SIMULATE_PORT
    outb (output_contents, BasePort) ;
  #endif

  sleep (PORT_SLEEP) ;

  for (;;)
  {
    read_mesg (port_driver_msgq, (LINUX_MSG_BUF *)(&port_driver_msg), sizeof(port_driver_msg)) ;

    #ifdef PARANOID
      if (port_driver_msg.command.mtype != PARALLEL_PORT_REQUEST)
      {
	log_msg (ERR, "Bad msg recd type: %d, subcmd: %#0x, reply_to: %d", port_driver_msg.command.mtype, port_driver_msg.command.sub_command, port_driver_msg.command.reply_to) ;
	return (0) ;
      }
    #endif

    switch (port_driver_msg.command.sub_command)
    {
      case LED1_OFF:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received LED1_OFF message") ;
        #endif

	output_contents &= ~Led1 ;
        #ifndef SIMULATE_PORT
	  outb (output_contents, BasePort) ;
        #endif
        break ;

      case LED1_ON:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received LED1_ON message") ;
        #endif

	output_contents |= Led1 ;
        #ifndef SIMULATE_PORT
	  outb (output_contents, BasePort) ;
        #endif
        break ;

      case LED2_OFF:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received LED2_OFF message") ;
        #endif

	output_contents &= ~Led2 ;
        #ifndef SIMULATE_PORT
	  outb (output_contents, BasePort) ;
        #endif
        break ;

      case LED2_ON:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received LED2_ON message") ;
        #endif

	output_contents |= Led2 ;

        #ifndef SIMULATE_PORT
	  outb (output_contents, BasePort) ;
        #endif
        break ;

      case INITIALISE_PORT_DRIVER:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received INITIALISE message - emulating SEND_COMMAND") ;
        #endif

	memset ((void *)(port_driver_msg.out_info.output_bytes), 0, sizeof(port_driver_msg.out_info.output_bytes)) ;

      case SEND_COMMAND:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received SEND_COMMAND message") ;
        #endif

	/*
	 * Firstly, ensure that the clock and latch bits are off. Then shift
	 * the bits out one at a time.
	 */

	output_contents &= ~(Clock_WR | Latch_WR) ;

        #ifndef SIMULATE_PORT
	  outb (output_contents, BasePort) ;

	  if (wait_for_low (Clock_RD | Latch_RD))
	    fail_abort(1) ;
        #endif

        #ifdef PORT_DEBUG
	  decode_io_string (NUM_SERIAL_OUT_BITS, port_driver_msg.out_info.output_bytes, "Outputting serial string '%s'") ;
        #endif

	ptr = port_driver_msg.out_info.output_bytes ;

	for (bit=NUM_SERIAL_OUT_BITS; bit > 0 ; bit--, ptr++)
	{
	  /*
	   * Toggle the command data bit if necessary
	   */

	  if ((output_contents & Cmd_Data_WR) != (*ptr & Cmd_Data_WR))
	  {
            #ifndef SIMULATE_PORT
	      output_contents ^= Cmd_Data_WR ;
	      outb (output_contents, BasePort) ;

	      /*
	       * Because the command data ring does not propagate the bit
	       * around the ring to the Cmd_Data_RD pin without clocking, all
	       * we can do to ensure that the bit is stable is to delay a
	       * little while before proceeding. That's what we do now - for a
	       * couple of microseconds.
	       */

	      usleep (30) ; /* microseconds */
            #endif
	  }

	  /*
	   * Issue a clock pulse: positive then negative transition. Exit with
	   * Clock LOW.
	   */

          #ifndef SIMULATE_PORT
	    output_contents |= Clock_WR ;
	    outb (output_contents, BasePort) ;

	    if (wait_for_high (Clock_RD))
	      fail_abort(2) ;

	    output_contents &= ~Clock_WR ;
	    outb (output_contents, BasePort) ;

	    if (wait_for_low (Clock_RD))
	      fail_abort(3) ;
          #endif
	}

	/*
	 * All the command bits have been shifted out to the train controllers.
	 * The command needs to be latched before it can be activated!
	 *
	 * The output protocol is to toggle the latch bit while the clock is
	 * LOW.
	 */

        #ifndef SIMULATE_PORT
	  output_contents |= Latch_WR ;
	  outb (output_contents, BasePort) ;

	  if (wait_for_high (Latch_RD))
	    fail_abort(4) ;

	  output_contents &= ~Latch_WR ;
	  outb (output_contents, BasePort) ;

	  if (wait_for_low (Latch_RD))
	    fail_abort(5) ;
        #endif

	if (Reset_Ring != 0)
	  _exit (0) ;

        break ;

      case RECEIVE_STATUS:
        #ifdef PORT_DEBUG
	  log_msg (FILE_ONLY, "Received RECEIVE_STATUS message") ;
        #endif

        #ifndef SIMULATE_PORT
	  /*
	   * Firstly, ensure that the clock bit is ON and latch bit is OFF.
	   *
	   * The reading protocol requires that the Latch_WR command is toggled
	   * when the Clock_WR bit is ON.
	   *
	   * Finally, return the clock bit to zero so that we can start shifting.
	   */

	  output_contents |= Clock_WR ;
	  output_contents &= ~Latch_WR ;
	  outb (output_contents, BasePort) ;

	  if (wait_for_high (Clock_RD))
	    fail_abort(6) ;

	  if (wait_for_low (Latch_RD))
	    fail_abort(7) ;

	  output_contents |= Latch_WR ;
	  outb (output_contents, BasePort) ;

	  if (wait_for_high (Latch_RD))
	    fail_abort(8) ;

	  output_contents &= ~Latch_WR ;
	  outb (output_contents, BasePort) ;

	  if (wait_for_low (Latch_RD))
	    fail_abort(9) ;

	  output_contents &= ~Clock_WR ;
	  outb (output_contents, BasePort) ;

	  if (wait_for_low (Clock_RD))
	    fail_abort(10) ;

	  for (bit=0 ; bit < NUM_SERIAL_IN_BITS ; bit++)
	  {
	    /*
	     * Read the data bit, then clock the shift register
	     */

	    port_driver_msg.in_info.input_bytes [bit] = (inb(BasePort+1) & Stat_Data_RD) ? 0x00 : 0xff ; /* Invert the logic */
	    output_contents |= Clock_WR ;
	    outb (output_contents, BasePort) ;
	    
	    if (wait_for_high (Clock_RD))
	      fail_abort(11) ;

	    output_contents &= ~Clock_WR ;
	    outb (output_contents, BasePort) ;

	    if (wait_for_low (Clock_RD))
	      fail_abort(12) ;
	  }
        #else
	  for (bit=NUM_SERIAL_IN_BITS; bit-- > 0 ;)
	    port_driver_msg.in_info.input_bytes [bit] = 0x00 ;
        #endif

        #ifdef PORT_DEBUG
	  decode_io_string (NUM_SERIAL_IN_BITS, port_driver_msg.in_info.input_bytes, "Read input string '%s'") ;
        #endif

	port_driver_msg.command.mtype = PARALLEL_PORT_RESPONSE ;
	send_mesg (port_driver_msg.command.reply_to, (LINUX_MSG_BUF *)(&port_driver_msg), sizeof(PORT_IN_MSG)) ;
        break ;

      default:
	#ifdef PARANOID
          log_msg (ERR, "Bad subcmd recd (type: %d, subcmd: %d, reply_to: %d", port_driver_msg.command.mtype, port_driver_msg.command.sub_command, port_driver_msg.command.reply_to) ;
	  return (0) ;
	#endif
	break ;
    }
  }
}

/******************************************************************************
 * status_scanner_process
 *
 * This routine is the basis for the status scanning process. It makes use of
 * the port controller process to read the port.
 *
 * Each process in Linux has its own timers available to it. The timer for
 * this process goes off at a regular interval and triggers the scanning
 * process.
 *
 * The constant Scan_Interval defines how frequently the scan process is
 * triggered. The value is not too critical but should be frequent enough
 * such that for the shortest track block segment and the fastest locomotive
 * speed, there are at least 10 scans or so while the train is within the
 * track block. Normally, 0.3 seconds is more than fast enough. There is no
 * real penalty for setting this timer too fast, except that the processor
 * load is increased.
 *
 * Because of the way that the scanner works, a small amount of time will be
 * lost each time a scan is conducted. So the real scan interval will be more
 * like 350ms than 300ms.
 */

void
request_notification (const RING_IN_TYPE type, const Ushort id, int msgq, const STATE state, int sub_cmd)
{
  NOTIFIER *ptr=(void *)0 ; /* avoid compiler warning by setting to 0 */

  #ifdef STATUS_SCAN_DEBUG
    log_msg (FILE_ONLY, "req_not called: type=%d, id=%hu, state=%s", type, id, (state == OFF) ? "OFF" : "ON") ;
  #endif

  lock_sem (Not_Sem) ;

  switch (type)
  {
    case BLOCK_DET:
      #ifdef PARANOID
        if ((id == 0) || (id > NUM_BLOCKS))
        {
	  log_msg (ERR, "Bad block number for request_notification() (%d)", id) ;
	  _exit (0) ;
	}
      #endif

      #ifdef STATUS_SCAN_DEBUG
        log_msg (FILE_ONLY, "%s notification of BLOCK_DET %d", (state == OFF) ? "Disabling" : "Enabling", id) ;
      #endif

      ptr = (*Not_Block_Det)[id - 1] ;
      break ;

    case GP_INPUT:
      #ifdef PARANOID
        if ((id == 0) || (id > NUM_GP_INPUTS))
        {
	  log_msg (ERR, "Bad GP input number for request_notification() (%d)", id) ;
	  _exit (0) ;
        }
      #endif

      #ifdef STATUS_SCAN_DEBUG
        log_msg (FILE_ONLY, "%s notification of GP_INPUT %d", (state == OFF) ? "Disabling" : "Enabling", id) ;
      #endif

      ptr = (*Not_GP_Input)[id - 1] ;
      break ;

    case POWER_OVERLOAD:
      #ifdef PARANOID
        if ((id == 0) || (id > NUM_OVERLOAD_SOURCES))
	{
          log_msg (ERR, "Bad overload source number for request_notification() (%d)", id) ;
          _exit (0) ;
        }
      #endif

      #ifdef STATUS_SCAN_DEBUG
        log_msg (FILE_ONLY, "%s notification of POWER_OVERLOAD %d", (state == OFF) ? "Disabling" : "Enabling", id) ;
      #endif

      ptr = (*Not_Pow_Overload)[id - 1] ;
      break ;

    #ifdef PARANOID
      default:
	log_msg (ERR, "Bad type specified for request_notification() (%d)", type) ;
	_exit (0) ;
	break ;
    #endif
  }

  unlock_sem (Not_Sem) ;
  lock_sem (Notifier_Sem) ;

  if (((*ptr).notify_id != msgq) && (((*ptr).notify_id != 0) || ((*ptr).sub_command != 0)))
  {
    log_msg (ERR, "req_not: type=%d id=%d. Either existing sub_cmd=%d or not_id=%d non zero", id, sub_cmd, (*ptr).sub_command, (*ptr).notify_id) ;
    _exit (0) ;
  }

  if (state == OFF)
  {
    msgq = 0 ;
    sub_cmd = 0 ;
  }

  (*ptr).sub_command = sub_cmd ;
  (*ptr).notify_id = msgq ;
  unlock_sem (Notifier_Sem) ;
}

/*
 * debounce delay = CHANGE_PERSISTENCE * Scan_Interval seconds
 */

#define  Scan_Interval      0.2     /* seconds */
#define  Scan_Blink_Period  0.35    /* seconds */

Uchar
process_input_bits (Uchar *input_bytes, Uchar val, RING_IN_TYPE *input_type, Ushort *input_id, Uchar *input_state, Uchar *input_count, Uchar **status_indicator)
{
  STATUS_MSG_UNION  status_msg ;
  Ushort            bit ;
  Ushort            turn_led_on = 0 ;
  Ushort            per ;

  /*
   * The following klude has been added because I found that the block
   * detectors sometimes are slower to flag an entry than they are to release
   * the block. This leads to occasional missignalling. The kludge ensures that
   * block entry is noticed a little faster than exit
   */

  if (val != 0)
    per = (CHANGE_PERSISTENCE > 1) ? CHANGE_PERSISTENCE-2 : CHANGE_PERSISTENCE ;
  else
    per = CHANGE_PERSISTENCE ;

  for (bit=0 ; bit<NUM_SERIAL_IN_BITS ; bit++)
  {
    /*
     * We're only processing one of the states this time through.
     * process_input_bits must be called twice to process all bits
     */

    if (input_bytes[bit] != val)
      continue ;

    if (input_state[bit] != input_bytes[bit])
    {
      if (++input_count[bit] > per) /* debouncing */
      {
	turn_led_on = 1 ;
	input_state[bit] = input_bytes[bit] ;
	input_count[bit] = 0 ;

        #ifdef STATUS_SCAN_DEBUG
	  log_msg (FILE_ONLY, "Bit=%d changed to %d", bit, input_state[bit]) ;
        #endif

	lock_sem (Status_Indicators_Sem) ;
	*(status_indicator[bit]) = input_bytes[bit] ;
	unlock_sem (Status_Indicators_Sem) ;
	lock_sem (Notifier_Sem) ;

	if ((*Notifier)[bit].notify_id != 0)
	{
          #ifdef STATUS_SCAN_DEBUG
            log_msg (FILE_ONLY, "Notifying %s with sub_cmd=%d", proc_from_msgq((*Notifier)[bit].notify_id), (*Notifier)[bit].sub_command) ;
          #endif

	  status_msg.command.mtype = BLOCK_NOTIFICATION ;
	  status_msg.command.sub_command = (*Notifier)[bit].sub_command ;
	  status_msg.notify.type = input_type[bit] ;
	  status_msg.notify.id = input_id[bit] ;
	  status_msg.notify.state = input_state[bit] ;
	  send_mesg ((*Notifier)[bit].notify_id, (LINUX_MSG_BUF *)(&status_msg), sizeof(NOTIFY_STATUS_MSG)) ;
	}

	unlock_sem (Notifier_Sem) ;
      }
    }
    else
    {
      #ifdef STATUS_SCAN_DEBUG
        if (input_count[bit] != 0)
	  log_msg (FILE_ONLY, "Bouncing input_bit[%d]. Bounce count=%d", bit, input_count[bit]) ;
      #endif

      input_count[bit] = 0 ;
    }
  }

  return (turn_led_on) ;
}

int
status_scanner_process (void)
{
  STATUS_MSG_UNION  status_msg ;
  PORT_MSG          port_msg ;
  RING_IN_TYPE      input_type [NUM_SERIAL_IN_BITS] ;
  Ushort            input_id [NUM_SERIAL_IN_BITS] ;
  Uchar             input_state [NUM_SERIAL_IN_BITS] ;
  Uchar             input_count [NUM_SERIAL_IN_BITS] ;
  Uchar             *status_indicator[NUM_SERIAL_IN_BITS] ; /* array of pointers to Uchar */
  Ushort            bit, i ;
  Uchar             turn_led_on ;
  int               prompt_a_scan ;
  SERIAL_IN         *in_ptr ;

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Main msg=%d, Scan response msg=%d", getpid(), scan_msgq, status_recv_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), scan_msgq, status_recv_msgq) ;
  set_process_priority (SCHED_RR, 1) ;

  /*
   * Initialise the pointer arrays to NULL. These will be reset a little later
   * with real information.
   */

  memset ((void *)(*Not_Block_Det), 0, NUM_BLOCKS*sizeof((*Not_Block_Det)[0])) ;
  memset ((void *)(*Block_Occupancy), 0, NUM_BLOCKS*sizeof((*Block_Occupancy)[0])) ;
  memset ((void *)(*Not_GP_Input), 0, NUM_GP_INPUTS*sizeof((*Not_GP_Input)[0])) ;
  memset ((void *)(*GP_Inputs), 0, NUM_GP_INPUTS*sizeof((*GP_Inputs)[0])) ;
  memset ((void *)(*Not_Pow_Overload), 0, NUM_OVERLOAD_SOURCES*sizeof((*Not_Pow_Overload)[0])) ;
  memset ((void *)(*Overload_Status), 0, NUM_OVERLOAD_SOURCES*sizeof((*Overload_Status)[0])) ;

  for (i=0 ; i<NUM_SERIAL_IN_BITS ; i++)
  {
    input_type[i] = END_INPUT_RING ;      /* A null value */
    input_id[i] = 0xffff ;
  }

  /*
   * Initialise the input ring pointer arrays by reading the input ring format
   * structure defined in <<controller.h>>
   *
   * Do some basic error checking along the way to catch human mistakes in
   * setting up this table.
   *
   * There are many arrays of data and pointers; they are used as follows:
   *
   * input_state[] holds state of each debounced input ring bit
   * input_count[] is the count of number of times bit has changed
   *
   * input_type[] identifies the class of each of the bits in the input ring
   * input_id[]   identifies the item number within this class
   *
   * status_indicator[] points to a mirror array of input bit states. Each
   *                    of these secondary arrays are arranged by type of
   *                    bit (eg block detect, power overload, GP input)
   *                    rather than the order of bits in the input ring
   */

  for (i=0, in_ptr=Input_Format ; in_ptr->type != END_INPUT_RING ; in_ptr++, i++)
  {
    #ifdef STATUS_SCAN_DEBUG
      log_msg (FILE_ONLY, "Input_Format[%2d] - type=%d, id=%d", i, in_ptr->type, in_ptr->id) ;
    #endif

    switch (in_ptr->type)
    {
      case BLOCK_DET:
	#ifdef PARANOID
	  if ((in_ptr->id == 0) || (in_ptr->id > NUM_BLOCKS))
	  {
	    log_msg (ERR, "Bad block detect number (%d)", in_ptr->id) ;
	    return (0) ;
	  }

	  if ((*Not_Block_Det)[in_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one block_det %d entry in Input_Format", in_ptr->id) ;
	    return (0) ;
	  }
	#endif

	(*Not_Block_Det)[in_ptr->id - 1] = &(*Notifier)[i] ;
	status_indicator[i] = &(*Block_Occupancy)[in_ptr->id - 1] ;
	input_type[i] = BLOCK_DET ;
	input_id[i] = in_ptr->id ;
	break ;

      case GP_INPUT:
	#ifdef PARANOID
	  if ((in_ptr->id == 0) || (in_ptr->id > NUM_GP_INPUTS))
	  {
	    log_msg (ERR, "Bad GP input number (%d)", in_ptr->id) ;
	    return (0) ;
	  }

	  if ((*Not_GP_Input)[in_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one gp_input %d entry in Input_Format", in_ptr->id) ;
	    return (0) ;
	  }
	#endif

	(*Not_GP_Input)[in_ptr->id - 1] = &(*Notifier)[i] ;
	status_indicator[i] = &(*GP_Inputs)[in_ptr->id - 1] ;
	input_type[i] = GP_INPUT ;
	input_id[i] = in_ptr->id ;
	break ;

      case POWER_OVERLOAD:
	#ifdef PARANOID
	  if ((in_ptr->id == 0) || (in_ptr->id > NUM_OVERLOAD_SOURCES))
	  {
	    log_msg (ERR, "Bad overload source number (%d)", in_ptr->id) ;
	    return (0) ;
	  }

	  if ((*Not_Pow_Overload)[in_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one power_overload %d entry in Input_Format", in_ptr->id) ;
	    return (0) ;
	  }
	#endif

	(*Not_Pow_Overload)[in_ptr->id - 1] = &(*Notifier)[i] ;
	status_indicator[i] = &(*Overload_Status)[in_ptr->id - 1] ;
	input_type[i] = POWER_OVERLOAD ;
	input_id[i] = in_ptr->id ;
	break ;

      case END_INPUT_RING:
	#ifdef PARANOID
	  log_msg (ERR, "Unexpected end of Input_Format[] after %d entries.\nHave you configured <controller.h> correctly?", i-1) ;
	  return (0) ;
	#endif

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Unknown entry in Input_Format[%d]. Type=%d, id=%d\nHave you configured <controller.h> correctly?", i, in_ptr->type, in_ptr->id) ;
	  return (0) ;
	#endif
    }
  }

  #ifdef PARANOID
    if (i != NUM_SERIAL_IN_BITS)
    {
      log_msg (ERR, "Number of entries (%d) in Input_Format[] incorrect. Should be %d", i, NUM_SERIAL_IN_BITS) ;
      return (0) ;
    }
  #endif

  unlock_sem (Not_Sem) ;

  /*
   * Initialise the present state of the system by performing an initial
   * scan and populating the static database
   */

  send_port_req_msg (status_recv_msgq, RECEIVE_STATUS, (void *)0) ;
  read_mesg (status_recv_msgq, (LINUX_MSG_BUF *)(&port_msg), sizeof(PORT_IN_MSG)) ;

  #ifdef PARANOID
    if (port_msg.command.mtype != PARALLEL_PORT_RESPONSE)
    {
      log_msg (ERR, "Bad init msg (type: %d, subcmd: %d, reply_to: %d", port_msg.command.mtype, port_msg.command.sub_command, port_msg.command.reply_to) ;
      return (0) ;
    }
  #endif

  for (bit=0 ; bit<NUM_SERIAL_IN_BITS ; bit++)
  {
    input_state[bit] = port_msg.in_info.input_bytes[bit] ;
    input_count[bit] = 0 ;
    (*Notifier)[bit].sub_command = (*Notifier)[bit].notify_id = 0 ;
    *(status_indicator[bit]) = port_msg.in_info.input_bytes[bit] ;
  }

  #ifdef STATUS_SCAN_DEBUG
    decode_io_string (NUM_BLOCKS, *Block_Occupancy, "Initial Block_Occupancy='%s'") ;
    decode_io_string (NUM_GP_INPUTS, *GP_Inputs, "Initial GP_Inputs='%s'") ;
    decode_io_string (NUM_OVERLOAD_SOURCES, *Overload_Status, "Initial Overload_Status='%s'") ;
  #endif

  unlock_sem (Notifier_Sem) ;
  unlock_sem (Status_Indicators_Sem) ;

  /*
   * Send a message to the timekeeper process which will
   * trigger the first scanning of the ring.
   */

  send_timeout_req_msg (scan_msgq, TICK_FREQ*Scan_Interval, &prompt_a_scan) ;

  #ifdef STATUS_SCAN_DEBUG
    log_msg (FILE_ONLY, "Initialised") ;
  #endif

  /*
   * The main loop of the scan process gets executed each time the timer
   * above expires or a message is received. (The timer must be restarted each
   * each time)
   */

  for (;;)
  {
    read_mesg (scan_msgq, (LINUX_MSG_BUF *)(&status_msg), sizeof(status_msg)) ;

    switch (status_msg.command.mtype)
    {
      case TIMEOUT_NOTIFICATION:
	/*
	 * Timeouts could either be to prompt a scan or to turn off the LED
	 */

	if (status_msg.command.sub_command == (int)(&prompt_a_scan))
	{
	  /*
	   * Firstly, restart the scan trigger timer so that there is a
	   * next time.
	   */

	  send_timeout_req_msg (scan_msgq, TICK_FREQ*Scan_Interval, &prompt_a_scan) ;

          #ifdef STATUS_SCAN_DEBUG
	    log_msg (FILE_ONLY, "About to send scan message") ;
          #endif

	  send_port_req_msg (status_recv_msgq, RECEIVE_STATUS, (void *)0) ;
	  read_mesg (status_recv_msgq, (LINUX_MSG_BUF *)(&port_msg), sizeof(PORT_IN_MSG)) ;

	  #ifdef PARANOID
	    if (port_msg.command.mtype != PARALLEL_PORT_RESPONSE)
	    {
	      log_msg (ERR, "Bad subcmd recd (type: %d, subcmd: %d, reply_to: %d", port_msg.command.mtype, port_msg.command.sub_command, port_msg.command.reply_to) ;
	      return (0) ;
	    }
	  #endif

          #ifdef STATUS_SCAN_DEBUG
	    decode_io_string (NUM_SERIAL_IN_BITS, port_msg.in_info.input_bytes, "Received Scan Response string '%s'") ;
          #endif

	  /*
	   * We need to process the input bits to see what has changed. It is
	   * imperative that we process the transitions from  0->1 transitions
	   * before the 1->0 transitions. This is to ensure that when a train
	   * exits one block and enters the next block in the same scan
	   * interval, that the first informative message to be sent to the
	   * engineer process is the entry message.
	   */

	  turn_led_on  = process_input_bits (port_msg.in_info.input_bytes, 0xff, input_type, input_id, input_state, input_count, status_indicator) ;
	  turn_led_on |= process_input_bits (port_msg.in_info.input_bytes, 0x00, input_type, input_id, input_state, input_count, status_indicator) ;

	  if (turn_led_on)
	  {
	    /*
	     * If there have been any changes, we should send a message
	     * to turn on the LED and send a message to (re)start the
	     * timer.
	     */

	    send_port_req_msg (0, LED1_ON, (void *)0) ;
	    send_timeout_req_msg (scan_msgq, TICK_FREQ*Scan_Blink_Period, (int *)&turn_led_on) ;
	  }
	}
	else if (status_msg.command.sub_command == (int)(&turn_led_on))
	{
	  /*
	   * Turn off the "input changed" LED
	   */

	  send_port_req_msg (0, LED1_OFF, (void *)0) ;
	}
	else
	{
	  #ifdef PARANOID
	    log_msg (ERR, "Unknown timeout received: %#0x", status_msg.command.sub_command) ;
	    return (0) ;
	  #endif
	}
	break ;

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Unknown message received: %d, sub command: %d", status_msg.command.mtype, status_msg.command.sub_command) ;
	  return (0) ;
	#endif
	break ;
    }
  }
}

/******************************************************************************
 * dispatcher_process
 *
 * This routine is the basis for the ring output management. It makes use of
 * the port controller process to write to the port. The basic idea is that any
 * process wishing to send a command onto the output ring does so through this
 * dispatcher process. The dispatcher receives commands, but before sending the
 * command to the ring (a process which takes quite some processing and time)
 * the dispatcher holds off for a fixed period of time just in case other ring
 * commands are received. At worst, a command is delayed for a few hundred
 * milliseconds. At best, a number of otherwise independent commands can be
 * integrated into the one ring shifting operation.
 *
 * There are two other advantages to centralising the management of ring output
 * commands. Firstly, the data structures representing the instantaneous state
 * of the output ring are a critical resource which would need to be in a
 * semaphore protected shared memory block if they were to be globally
 * readable. This would be a pain and would make the code difficult to follow.
 * Secondly, because the format of the ring may change depending on exactly how
 * the hobbyist implements their circuit, putting the formatting information
 * into one place (here) will simplify modifications for alternate bit ordering
 * etc.
 */

#define  Hold_Interval        0.15    /* seconds-time to hold off sending cmd */
#define  Cmd_Blink_Period     0.35    /* seconds-time to flash LED */

int
dispatcher_process (void)
{
  Uchar          *power_ctrl_dc  [NUM_POWER_SOURCES] ;
  Uchar          *power_ctrl_pwm [NUM_POWER_SOURCES] ;
  Uchar          *point_r        [NUM_POINTS] ;
  Uchar          *point_s        [NUM_POINTS] ;
  Uchar          *block_dir      [NUM_BLOCKS] ;
  Uchar          *block_pwr      [NUM_BLOCKS] ;
  Uchar          output_ring     [NUM_SERIAL_OUT_BITS] ;
  DISP_MSG_UNION disp_msg ;
  int            led_timeout, keepalive_timeout ;
  int            i, j, timer_on ;
  SERIAL_OUT     *out_ptr ;
  PSU_OBJECT     *speed ;
  Uchar          *spd_ptr ;

  #ifdef PARANOID
    int          conf_err ;
  #endif

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), dispatcher_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), dispatcher_msgq, 0) ;
  set_process_priority (SCHED_RR, 2) ;

  /*
   * Initialise the pointer arrays to NULL. These will be reset a little later
   * with real information.
   */

  memset ((void *)power_ctrl_dc, 0, NUM_POWER_SOURCES*sizeof(power_ctrl_dc[0])) ;
  memset ((void *)power_ctrl_pwm, 0, NUM_POWER_SOURCES*sizeof(power_ctrl_pwm[0])) ;
  memset ((void *)point_r, 0, NUM_POINTS*sizeof(point_r[0])) ;
  memset ((void *)point_s, 0, NUM_POINTS*sizeof(point_s[0])) ;
  memset ((void *)block_dir, 0, NUM_BLOCKS*sizeof(block_dir[0])) ;
  memset ((void *)block_pwr, 0, NUM_BLOCKS*sizeof(block_pwr[0])) ;
  memset ((void *)output_ring, 0, NUM_SERIAL_OUT_BITS*sizeof(output_ring[0])) ;

  /*
   * Initialise the output ring by setting each field to a known value. At the
   * same time, create arrays of pointers pointing into the output_ring array
   * so that when it comes to changing values, this can be done without a
   * further (slow) reference to the Output_Format array.
   */

  i = j = 0 ;
  out_ptr = Output_Format ;

  #ifdef PARANOID
    conf_err = 0 ;
  #endif

  for (; out_ptr->type != END_OUTPUT_RING ; out_ptr++, i++)
  {
    #ifdef DISPATCHER_DEBUG
      log_msg (FILE_ONLY, "Output_Format[%2d] - ring offset=%d, type=%d, id=%d", i, j, out_ptr->type, out_ptr->id) ;
    #endif

    /*
     * initialise point controllers, relays and power sources to logic 0
     */

    switch (out_ptr->type)
    {
      case POINT_R:
	#ifdef PARANOID
	  if ((out_ptr->id == 0) || (out_ptr->id > NUM_POINTS))
	  {
	    log_msg (ERR, "Bad Point_R number (%d)", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }

	  if (point_r[out_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one point_r %d entry in Ouptut_Format", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }
	#endif

	point_r [out_ptr->id - 1] = &output_ring [j++] ;
	break ;

      case POINT_S:
	#ifdef PARANOID
	  if ((out_ptr->id == 0) || (out_ptr->id > NUM_POINTS))
	  {
	    log_msg (ERR, "Bad Point_S number (%d)", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }

	  if (point_s[out_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one point_s %d entry in Ouptut_Format", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }
	#endif

	point_s [out_ptr->id - 1] = &output_ring [j++] ;
	break ;

      case BLOCK_DIR:
	#ifdef PARANOID
	  if ((out_ptr->id == 0) || (out_ptr->id > NUM_BLOCKS))
	  {
	    log_msg (ERR, "Bad Block_Dir number (%d)", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }

	  if (block_dir[out_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one block_dir %d entry in Ouptut_Format", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }
	#endif

	block_dir [out_ptr->id - 1] = &output_ring [j++] ;
	break ;

      case BLOCK_POWER_SRC:
	#ifdef PARANOID
	  if ((out_ptr->id == 0) || (out_ptr->id > NUM_BLOCKS))
	  {
	    log_msg (ERR, "Bad Block_Power_Src number (%d)", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }

	  if (block_pwr[out_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one block_power_source %d entry in Ouptut_Format", out_ptr->id) ;
	    conf_err++ ;
	    j++ ;
	    break ;
	  }
	#endif

	block_pwr [out_ptr->id - 1] = &output_ring [j++] ;
	break ;

      case CONTROLLER_DC:
	#ifdef PARANOID
	  if ((out_ptr->id == 0) || (out_ptr->id > NUM_POWER_SOURCES))
	  {
	    log_msg (ERR, "Bad Controller_DC number (%d)", out_ptr->id) ;
	    conf_err++ ;
	    j += 4 ;
	    break ;
	  }

	  if (power_ctrl_dc[out_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one pwr_ctrl_dc %d entry in Ouptut_Format", out_ptr->id) ;
	    conf_err++ ;
	    j += 4 ;
	    break ;
	  }
	#endif

	power_ctrl_dc [out_ptr->id - 1] = &output_ring [j] ;
	j += 4 ;
	break ;

      case CONTROLLER_PWM:
	#ifdef PARANOID
	  if ((out_ptr->id == 0) || (out_ptr->id > NUM_POWER_SOURCES))
	  {
	    log_msg (ERR, "Bad Controller_PWM number (%d)", out_ptr->id) ;
	    conf_err++ ;
	    j += 4 ;
	    break ;
	  }

	  if (power_ctrl_pwm[out_ptr->id - 1] != NULL)
	  {
	    log_msg (ERR, "More than one power_ctrl_pwm %d entry in Ouptut_Format", out_ptr->id) ;
	    conf_err++ ;
	    j += 4 ;
	    break ;
	  }
	#endif

	power_ctrl_pwm [out_ptr->id - 1] = &output_ring [j] ;
	j += 4 ;
	break ;

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Unknown entry in Output_Format[%d]. Type=%d, id=%d", i, out_ptr->type, out_ptr->id) ;
	  conf_err++ ;
	  break ;
	#endif
    }
  }

  #ifdef PARANOID
    if (j != NUM_SERIAL_OUT_BITS)
    {
      log_msg (ERR, "Number of bits in Output_Format[] (%d) incorrect (%d)", j, NUM_SERIAL_OUT_BITS) ;
      return (0) ;
    }

    if ((sizeof(Speed_Table)/sizeof(PSU_OBJECT)) != (MAX_SPEED+1))
    {
      log_msg (ERR, "Size of Speed_Table] (%d) incorrect", sizeof(Speed_Table)/sizeof(PSU_OBJECT)) ;
      return (0) ;
    }

    for (i=0 ; i < NUM_POWER_SOURCES ; i++)
    {
      if (power_ctrl_dc[i] == NULL)
      {
	log_msg (ERR, "CONTROLLER_DC[%d] has not been set in <controller.h> Output_Format", i) ;
	conf_err++ ;
      }

      if (power_ctrl_pwm[i] == NULL)
      {
	log_msg (ERR, "CONTROLLER_PWM[%d] has not been set in <controller.h> Output_Format", i) ;
	conf_err++ ;
      }
    }

    for (i=0 ; i < NUM_POINTS ; i++)
    {
      if (point_r[i] == NULL)
      {
	log_msg (ERR, "POINT_R[%d] has not been set in <controller.h> Output_Format", i) ;
	conf_err++ ;
      }

      if (point_s[i] == NULL)
      {
	log_msg (ERR, "POINT_S[%d] has not been set in <controller.h> Output_Format", i) ;
	conf_err++ ;
      }
    }

    for (i=0 ; i < NUM_BLOCKS ; i++)
    {
      if (block_dir[i] == NULL)
      {
	log_msg (ERR, "BLOCK_DIR[%d] has not been set in <controller.h> Output_Format", i) ;
	conf_err++ ;
      }

      if (block_pwr[i] == NULL)
      {
	log_msg (ERR, "BLOCK_POWER_SRC[%d] has not been set in <controller.h> Output_Format", i) ;
	conf_err++ ;
      }
    }

    if (conf_err != 0)
    {
      log_msg (ERR, "%d errors in Output_Format[]. Aborting.", conf_err) ;
      return (0) ;
    }
  #endif

  #ifdef DISPATCHER_DEBUG
    log_msg (FILE_ONLY, "Initialisation complete") ;
  #endif

  if ((Test_Ring != 0) && ((Test_Supply != 0) || (Test_Block != 0) || (Test_Points != 0)))
  {
    log_msg (ERR, "TEST_RING is mutually exclusive with other tests - ignoring") ;
    Test_Ring = 0 ;
  }

  if (Test_Ring != 0)
  {
    log_msg (FILE_ONLY, "Performing Ring Test") ;

    if (Test_Ring == 1)
    {
      log_msg (NORM, "Toggling furthest bit first, closest bit last") ;

      for (i=0 ; i < NUM_SERIAL_OUT_BITS ; i++)
      {
	output_ring [i] ^= 0xff ;
	log_msg (NORM, "Setting ring bit %d to %d", i, output_ring[i]) ;
	send_port_req_msg (0, SEND_COMMAND, output_ring) ;
	send_port_req_msg (0, ((i%2)==0)? LED2_ON : LED2_OFF, (void *)0) ;
	sleep (1) ;
	output_ring [i] ^= 0xff ;
      }
    }
    else
    {
      log_msg (NORM, "Sending an sliding one in three pattern") ;

      for (j=0 ; j<120 ; j++)
      {
	for (i=0 ; i < NUM_SERIAL_OUT_BITS ; i++)
	  output_ring [i] = ((j+i)%3 == 0) ? 0xff : 0x00 ;

	log_msg (NORM, "Sliding test loop %d", j) ;
	send_port_req_msg (0, SEND_COMMAND, output_ring) ;
	send_port_req_msg (0, ((j%2)==0)? LED2_ON : LED2_OFF, (void *)0) ;
	sleep (2) ;
      }
    }

    log_msg (NORM, "Ring test complete") ;
    return (0) ;
  }

  /*
   * The dispatcher process works by infinitely waiting for a message
   * from one of the other processes. The dispatcher receives the
   * message and acts on it. Then it waits for another message.
   *
   * There are two types of message: a command message and a timeout message.
   *
   * The timeout message is simplest. When received, a command is sent out onto
   * the ring. The timeout is not restarted.
   *
   * The command message triggers a change in one or more ring bits. If a
   * timeout is not yet started, then a timeout is started. If a timeout is
   * already running, then no further action is taken.
   *
   * The process of delaying before we send commands onto the ring ensures that
   * the ring is never over loaded and that the maximum time before a command
   * is sent is kept to a finite minimum.
   */

  #ifdef DISPATCHER_DEBUG
    log_msg (FILE_ONLY, "Starting ring keep alive timer") ;
  #endif
	
  send_timeout_req_msg (dispatcher_msgq, TICK_FREQ*KEEPALIVE_INTERVAL, &keepalive_timeout) ;

  for (timer_on=0 ;;)
  {
    read_mesg (dispatcher_msgq, (LINUX_MSG_BUF *)(&disp_msg), sizeof(disp_msg)) ;

    if (disp_msg.command.mtype == DISPATCHER_REQUEST)
    {
      /*
       * One of the train processes is asking for a command to be sent to the
       * ring.
       */

      switch (disp_msg.request.object.type)
      {
        case POINT_R:
          #ifdef DISPATCHER_DEBUG
            log_msg (FILE_ONLY, "Point_R msg recd (setting: %d, point id: %d)", disp_msg.command.sub_command, disp_msg.request.object.id) ;
          #endif

	  #ifdef PARANOID
	    if ((disp_msg.command.sub_command != OFF) && (disp_msg.command.sub_command != ON))
	    {
	      log_msg (ERR, "Bad Point_R cmd (%d)", disp_msg.command.sub_command) ;
	      return (0) ;
	    }

	    if ((disp_msg.request.object.id == 0) || (disp_msg.request.object.id > NUM_POINTS))
	    {
	      log_msg (ERR, "Bad Point_R id (%d)", disp_msg.request.object.id) ;
	      return (0) ;
	    }
	  #endif

	  *point_r [disp_msg.request.object.id - 1] = (disp_msg.command.sub_command == ON) ? 0xff : 0x00 ;
	  break ;

        case POINT_S:
          #ifdef DISPATCHER_DEBUG
            log_msg (FILE_ONLY, "Point_S msg recd (setting: %d, point: %d)", disp_msg.command.sub_command, disp_msg.request.object.id) ;
          #endif

	  #ifdef PARANOID
	    if ((disp_msg.command.sub_command != OFF) && (disp_msg.command.sub_command != ON))
	    {
	      log_msg (ERR, "Bad Point_S cmd (%d)", disp_msg.command.sub_command) ;
	      return (0) ;
	    }

	    if ((disp_msg.request.object.id == 0) || (disp_msg.request.object.id > NUM_POINTS))
	    {
	      log_msg (ERR, "Bad Point_S id (%d)", disp_msg.request.object.id) ;
	      return (0) ;
	    }
	  #endif

	  *point_s [disp_msg.request.object.id - 1] = (disp_msg.command.sub_command == ON) ? 0xff : 0x00 ;
	  break ;

        case BLOCK_DIR:
          #ifdef DISPATCHER_DEBUG
            log_msg (FILE_ONLY, "Block_Dir msg recd (setting=%d, block=%d)", disp_msg.command.sub_command, disp_msg.request.object.id) ;
          #endif

	  #ifdef PARANOID
	    if ((disp_msg.command.sub_command != NORMAL_POLARITY) && (disp_msg.command.sub_command != OPPOSITE_POLARITY))
	    {
	      log_msg (ERR, "Bad Block_Dir cmd=%d", disp_msg.command.sub_command) ;
	      return (0) ;
	    }

	    if ((disp_msg.request.object.id == 0) || (disp_msg.request.object.id > NUM_BLOCKS))
	    {
	      log_msg (ERR, "Bad Block_Dir id=%d", disp_msg.request.object.id) ;
	      return (0) ;
	    }
	  #endif

	  *block_dir [disp_msg.request.object.id - 1] = (disp_msg.command.sub_command == OPPOSITE_POLARITY) ? 0xff : 0x00 ;
	  break ;

        case SPEED:
          #ifdef DISPATCHER_DEBUG
            log_msg (FILE_ONLY, "Speed msg recd: speed=%d, psu=%d", disp_msg.command.sub_command, disp_msg.request.object.id) ;
          #endif

	  #ifdef PARANOID
	    if (disp_msg.command.sub_command > MAX_SPEED)
	    {
	      log_msg (ERR, "Bad Speed cmd=%d", disp_msg.command.sub_command) ;
	      return (0) ;
	    }

	    if ((disp_msg.request.object.id == 0) || (disp_msg.request.object.id > NUM_POWER_SOURCES))
	    {
	      log_msg (ERR, "Bad Speed id=%d", disp_msg.request.object.id) ;
	      return (0) ;
	    }
	  #endif

	  /*
	   * The following code is hard wired for 4 bit DC and PWM speed settings.
	   * You'll need to change it if you implement a different number of bits.
	   *
	   * For each power supply,
	   *   1st bit to be shifted out = LSB of PWM
	   *   4th bit to be shifted out = MSB of PWM
	   *   5th bit to be shifted out = LSB of DC
	   *   8th bit to be shifted out = MSB of DC
	   */

	  speed = &Speed_Table [disp_msg.command.sub_command] ;
	  j = speed->pwm_setting ;
	  spd_ptr = power_ctrl_pwm [disp_msg.request.object.id - 1] ;

	  for (i=4 ; i-- ; j>>=1)
	    *spd_ptr++ = (j & 0x01) ? 0xff : 0x00 ;

	  j = speed->dc_setting ;
	  spd_ptr = power_ctrl_dc [disp_msg.request.object.id - 1] ;

	  for (i=4 ; i-- ; j>>=1)
	    *spd_ptr++ = (j & 0x01) ? 0xff : 0x00 ;

	  break ;

        case BLOCK_POWER_SRC:
          #ifdef DISPATCHER_DEBUG
            log_msg (FILE_ONLY, "Block_Pwr_Src msg recd (psu=%d, block=%d)", disp_msg.command.sub_command, disp_msg.request.object.id) ;
          #endif

	  #ifdef PARANOID
	    if ((disp_msg.command.sub_command != PSU_1) && (disp_msg.command.sub_command != PSU_2))
	    {
	      log_msg (ERR, "Bad Block_Power_Src cmd (%d)", disp_msg.command.sub_command) ;
	      return (0) ;
	    }

	    if ((disp_msg.request.object.id == 0) || (disp_msg.request.object.id > NUM_BLOCKS))
	    {
	      log_msg (ERR, "Bad Block_Power_Src id=%d", disp_msg.request.object.id) ;
	      return (0) ;
	    }
	  #endif

	  *block_pwr [disp_msg.request.object.id - 1] = (disp_msg.command.sub_command == PSU_2) ? 0x00 : 0xff ;
	  break ;

        default:
	  #ifdef PARANOID
	    log_msg (ERR, "Bad sub cmd (type=%d, subcmd=%d, reply_to=%d", disp_msg.command.mtype, disp_msg.command.sub_command, disp_msg.command.reply_to) ;
	    return (0) ;
	  #endif
      }

      if (timer_on == 0)
      {
	/*
	 * There are presently no other commands which have been "queued".
	 * We'd better start the timer.
	 */

        #ifdef DISPATCHER_DEBUG
          log_msg (FILE_ONLY, "Dispatcher timer not presently running - about to start it") ;
        #endif
	
	send_timeout_req_msg (dispatcher_msgq, TICK_FREQ*Hold_Interval, &timer_on) ;
	timer_on = 1 ;
      }
    }
    else if (disp_msg.command.mtype == TIMEOUT_NOTIFICATION)
    {
      /*
       * Either it's time to send the cached message to the ring, or its time
       * to turn off the LED.
       */

      if (disp_msg.command.sub_command == (int)(&timer_on))
      {
        #ifdef DISPATCHER_DEBUG
          log_msg (FILE_ONLY, "Ring timer has expired and command about to be sent to the ring") ;
	  decode_io_string (NUM_SERIAL_OUT_BITS, output_ring, "Sending '%s'") ;
        #endif

	send_port_req_msg (0, SEND_COMMAND, output_ring) ;
	send_timeout_req_msg (dispatcher_msgq, TICK_FREQ*KEEPALIVE_INTERVAL, &keepalive_timeout) ;
	timer_on = 0 ;
	send_port_req_msg (0, LED2_ON, (void *)0) ;
	send_timeout_req_msg (dispatcher_msgq, TICK_FREQ*Cmd_Blink_Period, &led_timeout) ;
      }
      else if (disp_msg.command.sub_command == (int)(&led_timeout))
      {
	/*
	 * Turn off the "output sent" LED
	 */

        #ifdef DISPATCHER_DEBUG
          log_msg (FILE_ONLY, "LED timer has expired about to be turned off") ;
        #endif

	send_port_req_msg (0, LED2_OFF, (void *)0) ;
      }
      else if (disp_msg.command.sub_command == (int)(&keepalive_timeout))
      {
	/*
	 * Send a dummy command onto the ring just to keep the hardware
	 * reset mechanism from turning off the point and relay drivers
	 */

        #ifdef DISPATCHER_DEBUG
          log_msg (FILE_ONLY, "Keepalive timer has expired and ring is being refreshed") ;
        #endif

	send_port_req_msg (0, SEND_COMMAND, output_ring) ;
	send_timeout_req_msg (dispatcher_msgq, TICK_FREQ*KEEPALIVE_INTERVAL, &keepalive_timeout) ;
	send_port_req_msg (0, LED2_ON, (void *)0) ;
	send_timeout_req_msg (dispatcher_msgq, TICK_FREQ*Cmd_Blink_Period, &led_timeout) ;
      }
      else
      {
	#ifdef PARANOID
	  log_msg (ERR, "Bad timeout msg recd (subcmd: %d, reply_to: %d", disp_msg.command.sub_command, disp_msg.command.reply_to) ;
	  return (0) ;
	#endif
      }
    }
    else
    {
      #ifdef PARANOID
        log_msg (ERR, "Bad msg recd type: %d, subcmd: %#0x, reply_to: %d", disp_msg.command.mtype, disp_msg.command.sub_command, disp_msg.command.reply_to) ;
	return (0) ;
      #endif
    }
  }
}

/******************************************************************************
 * timekeeper_process
 *
 * This routine is the basis for the timekeeper process.
 *
 * There are probably many ways I could have implemented timers in this
 * project. I have chosen to write a centralised timer management process (this
 * process) which is entirely message based.
 *
 * Other processes wishing to start a timer send a message to this process.
 * The message will contain a magic number identifying the specific timer which
 * is to be started. (The magic number is usually the address of a process
 * signicicant local variable).
 *
 * This process checks to see if a timer with that magic ID already exists
 * and if so, restarts it - using the specified timeout. If not, a new timer is
 * created and added to the queue.
 *
 * If a process wishes to stop a timer, it sends a timer restart message with
 * a new timeout of 0. The timer process then deletes the timeout from the
 * queue (silently).
 *
 * This process uses a periodic alarm at the rate of TICK_FREQ Hz.
 * This sets the minimum granularity of the timers.
 */

#define Num_Timers 40

void
time_sig_handler (int sig)
{
  long   timer_msg_type ;  /* This is a VERY SHORT message - 1 field */

  #ifdef SIG_HANDLER_DEBUG
    log_msg (FILE_ONLY, "Time_sig_handler: Timer expired") ;
  #endif

  timer_msg_type = TICK_EVENT ;
  send_mesg (timer_msgq, (LINUX_MSG_BUF *)(&timer_msg_type), sizeof(timer_msg_type)) ;
}

int
timekeeper_process (void)
{
  
  TIMER             timer_array [Num_Timers] ;
  TIMER             *shortest_timer ;
  TIMER             *free_timer ;
  TIMER             *end_free_timer_list ;
  struct itimerval  timer ;
  TIMER_MSG         timer_msg ;
  Ushort            i ;
  TIMER             *tmp, *tmp1, *tmp2 ;

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), timer_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), timer_msgq, 0) ;
  set_process_priority (SCHED_RR, 2) ;

  /*
   * Initialise the timer array and pointers
   */

  Sys_Time->seconds = 0 ; /* uptime = 0 */
  Sys_Time->ticks = 0 ;

  for (i=0 ; i < Num_Timers ; i++)
  {
    timer_array [i].interval = 0 ;
    timer_array [i].ref_id = 0 ;
    timer_array [i].reply_to = 0 ;
    timer_array [i].next = &timer_array[i+1] ;
  }

  timer_array [Num_Timers-1].next = NULL ;
  shortest_timer = (TIMER *)0 ;
  free_timer = &timer_array[0] ;
  end_free_timer_list = &timer_array[Num_Timers-1] ;

  /*
   * Define a periodic timer to serve as the heartbeat which triggers the
   * processing of timers. By setting it_interval to a non zero value,
   * the timer becomes a periodic timeout - each time the timer expires, it
   * generates a SIGALM and at the same time, restarts the timer.
   */

  signal (SIGALRM, &time_sig_handler) ;
  timer.it_interval.tv_sec = 0 ;
  timer.it_interval.tv_usec = 1000000 / TICK_FREQ ; /* microseconds */
  timer.it_value.tv_sec = 0 ;
  timer.it_value.tv_usec = 1000000 / TICK_FREQ ;    /* microseconds */
  setitimer (ITIMER_REAL, &timer, NULL) ;

  #ifdef TIMER_DEBUG
    log_msg (FILE_ONLY, "Initialised") ;
  #endif

  for (;;)
  {
    /*
     * This loop is where the timekeeper process receives requests
     * from other processes.
     *
     * Also, a tick message is received from the alarm handler each
     * time an interval expires. Ticks are processed in order and
     * could be queued after timeout request messages.
     */

    read_mesg (timer_msgq, (LINUX_MSG_BUF *)(&timer_msg), sizeof(timer_msg)) ;

    switch (timer_msg.command.mtype)
    {
      case TICK_EVENT:
	/*
	 * The periodic timer has expired
	 */

	if (++(Sys_Time->ticks) >= TICK_FREQ)
	{
	  Sys_Time->ticks = 0 ;
	  Sys_Time->seconds++ ;
	}

        if (shortest_timer != NULL)
	{
	  shortest_timer->interval-- ;

	  /*
	   * If the head timer just decrmtd to zero, we need to process it.
	   */

	  while (shortest_timer->interval == 0)
	  {
	    tmp = shortest_timer ;
	    timer_msg.command.mtype = TIMEOUT_NOTIFICATION ;
	    timer_msg.command.sub_command = tmp->ref_id ;
	    send_mesg (tmp->reply_to, (LINUX_MSG_BUF *)(&timer_msg), sizeof(GENERIC_MSG)) ;

            #ifdef TIMER_DEBUG
	      log_msg (FILE_ONLY, "Timeout message sent to %s", proc_from_msgq (tmp->reply_to)) ;
            #endif

	    shortest_timer = tmp->next ;
	    tmp->ref_id = 0 ;
	    tmp->reply_to = 0 ;
	    tmp->next = NULL ;

	    if (end_free_timer_list != NULL)
	    {
	      end_free_timer_list->next = tmp ;
	      end_free_timer_list = tmp ;
	    }
	    else /* The free timer list is empty */
	      end_free_timer_list = free_timer = tmp ;

            if (shortest_timer == NULL)
	      break ;
	  }
	}

	break ;

      case TIMEOUT_REQUEST:
        /*
	 * A valid request has been received. First, scan the list of active
	 * timers to see if a timer with the same id already exists. If so,
	 * restart the timer by removing it from the queue and reinserting
	 * it. Otherwise, just start a new timer.
	 */

        for (tmp1=NULL,tmp=shortest_timer ; tmp!=NULL ; tmp1=tmp,tmp=tmp->next)
	{
	  if ((timer_msg.command.sub_command == tmp->ref_id) && (timer_msg.command.reply_to == tmp->reply_to))
	  {
	    /*
	     * An existing timer has been identified. Remove it from the list
	     * and reset it. Be careful to fix the list up though.
	     */

            #ifdef TIMER_DEBUG
	      log_msg (FILE_ONLY, "Timer restart request received from %s, duration %d", proc_from_msgq (tmp->reply_to), timer_msg.interval) ;
            #endif

	    if (tmp1 == NULL)
	    {
	      /*
	       * Existing timer. Remove it from head of the list
	       */

	      shortest_timer = tmp->next ;
	    }
	    else
	    {
	      /*
	       * Existing timer. Remove it from within the timer list
	       */

	      tmp1->next = tmp->next ;
	    }

	    if (tmp->next != NULL)
	    {
	      /*
	       * Another timer follows the existing timer which was just
	       * removed. Add the remaining interval of the deleted timer to
	       * the following timer.
	       */

	      tmp->next->interval += tmp->interval ;
	    }

	    /*
	     * Reset the parameters on the deleted timer to the spec'd values
	     */

	    tmp->interval = timer_msg.interval ;
	    tmp->reply_to = timer_msg.command.reply_to ;
	    break ;
	  }
	}

	if (timer_msg.interval == 0)
	{
	  /*
	   * The request was to delete the timer. Add the entry to the
	   * free list and get out of here
	   */

          #ifdef TIMER_DEBUG
	    log_msg (FILE_ONLY, "Request was to delete timer") ;
          #endif

	  if (tmp != NULL)
	  {
	    tmp->ref_id = 0 ;
	    tmp->reply_to = 0 ;
	    tmp->next = NULL ;

	    if (end_free_timer_list != NULL)
	    {
	      end_free_timer_list->next = tmp ;
	      end_free_timer_list = tmp ;
	    }
	    else /* The free timer list is empty */
	      end_free_timer_list = free_timer = tmp ;
	  }

	  break ;
	}

	if (tmp == NULL)
	{
	  /*
	   * A new timer needs to be started. Find a free timer entry from the
	   * head of the free list and fill out the fields.
	   */

          #ifdef TIMER_DEBUG
            log_msg (FILE_ONLY, "New timer request received from %s, duration %d", proc_from_msgq (timer_msg.command.reply_to), timer_msg.interval) ;
          #endif

	  if ((tmp = free_timer) == NULL)
	  {
	    log_msg (ERR, "Tried to start a new timer but no free timer entries") ;
	    return (0) ;
	  }

	  free_timer = tmp->next ;

	  if (free_timer == NULL)
	    end_free_timer_list = NULL ;

	  tmp->ref_id = timer_msg.command.sub_command ;
	  tmp->reply_to = timer_msg.command.reply_to ;
	  tmp->interval = timer_msg.interval ;
	}

	/*
	 * Add the new timer (pointed to by tmp) into the timer list
	 */

	if (shortest_timer == NULL)
	{
	  /*
	   * add the new timer to an empty list
	   */

	  shortest_timer = tmp ;
	  tmp->next = NULL ;
	}
	else
	{
	  /*
	   * add the new timer into the list but make sure to add it in at the
	   * right place.
	   */

	  for (tmp2=NULL, tmp1=shortest_timer ; tmp1 != NULL ; tmp2=tmp1, tmp1=tmp1->next)
	  {
	    if (tmp->interval < tmp1->interval)
	    {
	      /*
	       * add the new timer into the middle of the timer list
	       */

	      tmp1->interval -= tmp->interval ;

	      if (tmp2 == NULL)
	      {
		/*
		 * actually, the new timer should be the first entry in the
		 * timer list and all other entries should shift along.
		 */

		shortest_timer = tmp ;
	      }
	      else
		tmp2->next = tmp ;

	      tmp->next = tmp1 ;
	      break ;
	    }
	    else
	      tmp->interval -= tmp1->interval ;
	  }

	  if (tmp1 == NULL)
	  {
	    /*
	     * add the new timer to the end of the list
	     */

	    if (tmp2 == NULL)
	      shortest_timer = tmp ;
	    else
	      tmp2->next = tmp ;

	    tmp->next = NULL ;
	  }
	}

	break ;

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Bad msg recd type: %d, subcmd: %#0x, from %s", timer_msg.command.mtype, timer_msg.command.sub_command, proc_from_msgq (timer_msg.command.reply_to)) ;
	  return (0) ;
	#endif
    }
  }
}

/******************************************************************************
 * points_process
 *
 * An instance of this process is started for each train controller card within
 * the system. Each points controller process can switch one point at a time
 * and after setting a point, must wait for the capacitor to recharge before
 * firing again. This process does the timing and management of the capacitor
 * discharge / recharge cycle.
 *
 * The routine set_points() is a globally callable routine intended to be
 * used by train processes. Using the set_points() routine, the train processes
 * do not need to concern themselves with which points process messages should
 * be directed towards.
 *
 * When you read the code for the points process, don't become confused about
 * the relationship between a partcular set of points and the block to which
 * it is connected to..... ANY set of points could be connected into ANY track
 * block. This means that a set of points under the control of one particular
 * train controller card could be within a track block which is being supplied
 * by a different train controller card!
 *
 * For this reason, EACH points process needs to know the state of ownership
 * for EACH track block in the trainset. This is achieved using global shared
 * memory and is managed by the controller process.
 *
 * Actually, the points processes don't need to know the state of ownership
 * of the track blocks - this feature has been included purely as a debugging
 * aid - so that errors are flagged if a train process tries to change points
 * settings out of turn or incorrectly.
 *
 * BTW. The formula for the Cap Charge Time is
 *       t = CapValue * DesiredVoltage / ChargeCurrent
 * I've added an additional 0.2 seconds (below) "just to be very conservative"
 */

#define  PHold_Interval  (0.4+Hold_Interval)  /* time to pulse the points */
#define  Cap_Charge_Time ((POINTS_CAPACITANCE*1e-6)*20/0.6)  /* seconds */
#define  PRecharge_Time  (Cap_Charge_Time+Hold_Interval+0.2) /* seconds */

int
points_process (const int points_number)
{
  POINTS_MSG      msg ;
  POINTS_OBJECT   state [NUM_POINTS_PER_CONTROLLER_CARD] ;
  int             i ;

  #ifdef PARANOID
    if (points_number >= NUM_POINTS_SUPPLIES)
    {
      log_msg (ERR, "Process started with illegal points number") ;
      return (0) ;
    }
  #endif

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), points_msgq[points_number]) ;
  #endif

  set_process_info (Process_Name, getpid(), points_msgq[points_number], 0) ;
  set_process_priority (SCHED_RR, 1) ;

  for (i=0 ; i < NUM_POINTS_PER_CONTROLLER_CARD ; i++)
    state[i] = UNKNOWN_STATE ;

  #ifdef POINTS_DEBUG
    log_msg (FILE_ONLY, "Initialised") ;
  #endif

  for (;;)
  {
    read_mesg (points_msgq[points_number], (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;

    switch (msg.command.mtype)
    {
      case POINTS_REQUEST:
	/*
	 * One of the train processes is asking for points action
	 */

	#ifdef POINTS_DEBUG
	  log_msg (FILE_ONLY, "Received POINTS_REQUEST from %s, point_num=%d, dir=%d", proc_from_msgq(msg.command.reply_to), msg.setting.point_num, msg.setting.direction) ;
	#endif

	#ifdef PARANOID
          if ((msg.setting.direction != OPEN) && (msg.setting.direction != CLOSED))
	  {
	    log_msg (ERR, "%s tried to set bad direction=%d on point=%hu", proc_from_msgq(msg.command.reply_to), msg.setting.direction, msg.setting.point_num) ;
	    return (0) ;
	  }
	#endif

	i = msg.setting.point_num - 1 ;

	#ifdef PARANOID
          if ((i / NUM_POINTS_PER_CONTROLLER_CARD) != points_number)
	  {
	    log_msg (ERR, "%s specd points=%hu controlled by another process=%hu", proc_from_msgq(msg.command.reply_to), points_number, msg.setting.point_num) ;
	    return (0) ;
	  }
	#endif

	lock_sem (Block_From_Points_Sem) ;
	lock_sem (Block_Allocation_Sem) ;

	if (BLK_OWNER((*Block_From_Points)[i]+1) != msg.command.reply_to)
	{
	  log_msg (ERR, "%s requested point=%hu which is in block=%hu owned by %s", proc_from_msgq(msg.command.reply_to), i+1, (*Block_From_Points)[i]+1, proc_from_msgq(BLK_OWNER((*Block_From_Points)[i]+1))) ;
	  return (0) ;
	}

	unlock_sem (Block_Allocation_Sem) ;
	unlock_sem (Block_From_Points_Sem) ;

	/*
	 * Check to see the current known state of the set of points. If we're
	 * trying to set it the same way that it already is, then simply ignore
	 * the request and move on. This speeds up the process of setting
	 * points because there is no time wasted setting a point to a state
	 * that it is already in.
	 */

	i %= NUM_POINTS_PER_CONTROLLER_CARD ;

	if (state [i] != msg.setting.direction)
	{
	  state[i] = msg.setting.direction ;

          #ifdef POINTS_DEBUG
	    log_msg (FILE_ONLY, "Point %hu set to %s by %s", msg.setting.point_num, (state[i] == OPEN)?"OPEN":"CLOSED", proc_from_msgq(msg.command.reply_to)) ;
          #endif

	  send_dispatcher_req_msg (points_msgq[points_number], msg.setting.point_num, (msg.setting.direction==OPEN)?POINT_R:POINT_S, ON) ;
	  usleep (PHold_Interval * 1000000) ;
	  send_dispatcher_req_msg (points_msgq[points_number], msg.setting.point_num, (msg.setting.direction==OPEN)?POINT_R:POINT_S, OFF) ;
	  state [i] = msg.setting.direction ;
	  usleep (PRecharge_Time * 1000000) ;
	}
	else
	{
          #ifdef POINTS_DEBUG
	    log_msg (FILE_ONLY, "Point %hu already in state %s", msg.setting.point_num, (state[i] == OPEN)?"OPEN":"CLOSED") ;
          #endif
	}

	if (msg.index > 0)
	{
	  i = msg.command.reply_to ;
	  msg.command.reply_to = points_msgq[points_number] ;
	  send_mesg (i, (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
	}
	break ;

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Bad msg recd type: %d, subcmd: %#0x, reply_to: %d", msg.command.mtype, msg.command.sub_command, msg.command.reply_to) ;
	  return (0) ;
	#endif
    }
  }
}

/******************************************************************************
 * power_process
 *
 * An instance of this process is started for programmable power supply within
 * the system. Each power process manages just its own programmable DC and PWM
 * power supplies and will only accept commands from the train which owns
 * that power supply at the time.
 *
 * When a train is about to move from the domain of one train controller card
 * into the domain of another, the power supply processes need to communicate
 * directly with each other in order to pass the first's state to the latter.
 *
 * The routine send_power_msg() is a globally callable routine intended to be
 * used by train processes. Using the send_power_msg() routine, the train
 * processes do not need to concern themselves with which power process
 * messages should be directed towards.
 *
 * NOTE ABOUT SETTING OWNERSHIP OF POWER SUPPLIES
 *
 * The controller process is responsible for setting the ownership of a power
 * supply process when it allocates a particular PSU to a process. But the power
 * process itself resets its ownership to 0 when it receives a shutdown message.
 *
 * The constant below (Acceleration_Constant) defines the acceleration rate for
 * the __FASTEST__ acceleation state. At 0.25*TICK_FREQ, a train will
 * accelerate from 0 to maximum speed in 6 seconds for a 24 increment speed
 * scale. The Very Slow acceleration rate will take 4 times longer than this.
 */

#define Acceleration_Constant ((FASTEST_TIME / MAX_SPEED) * TICK_FREQ) /* ticks per increment */

void
process_acceleration (const int supply_number, volatile POWER_STATE_INFO (*ptr), int *id)
{
  /*
   * Adjust speed according to acceleration
   */

  if (ptr->speed != ptr->target_speed)
  {
    if (ptr->target_speed > ptr->speed)
      ptr->speed++ ;
    else
      ptr->speed-- ;
  }

  /*
   * If now reached target, notify train. Else start a timer.
   */

  if (ptr->speed == ptr->target_speed)
  {
    POWER_MSG_UNION    msg ;

    #ifdef POWER_DEBUG
      log_msg (FILE_ONLY, "Reached target speed, notifying %s", proc_from_msgq (ptr->owner)) ;
    #endif

    msg.command.mtype = SPEED_NOTIFICATION ;
    msg.command.reply_to = power_msgq[supply_number] ;
    msg.engineer.speed = ptr->speed ;
    msg.engineer.psu = supply_number+1 ;
    send_mesg (ptr->owner, (LINUX_MSG_BUF *)(&msg), sizeof(ENGINEER_MSG)) ;
    ptr->acceleration = IMMEDIATELY ;
  }
  else
    send_timeout_req_msg (power_msgq[supply_number], Acceleration_Constant * ptr->acceleration, id) ;

  #ifdef POWER_DEBUG
    log_msg (FILE_ONLY, "New speed=%hu, accel=%hu, targ spd=%hu", ptr->speed, ptr->acceleration, ptr->target_speed) ;
  #endif

  send_dispatcher_req_msg (power_msgq[supply_number], supply_number+1, SPEED, ptr->speed) ;
}

int
power_process (const int supply_number)
{
           POWER_MSG_UNION    msg ;
  volatile POWER_STATE_INFO   (*ptr) ;
           POWER_STATE_INFO   *ptr2 ;

  #ifdef PARANOID
    if (supply_number >= NUM_POWER_SOURCES)
    {
      log_msg (ERR, "Process started with illegal supply number=%d", supply_number) ;
      return (0) ;
    }
  #endif

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), power_msgq[supply_number]) ;
  #endif

  set_process_info (Process_Name, getpid(), power_msgq[supply_number], 0) ;
  set_process_priority (SCHED_RR, 1) ;
  ptr = &(*Power_State)[supply_number] ;
  ptr->owner = 0 ; /* a dummy value which will precipitate an error */
  ptr->speed = ptr->target_speed = UNKNOWN_STATE ;
  ptr->acceleration = IMMEDIATELY ;

  #ifdef POWER_DEBUG
    log_msg (FILE_ONLY, "Initialised") ;
  #endif

  for (;;)
  {
    unlock_sem (Power_State_Sem [supply_number]) ;
    read_mesg (power_msgq[supply_number], (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
    lock_sem (Power_State_Sem [supply_number]) ;

    switch (msg.command.mtype)
    {
      case HANDOVER_REQUEST:
	/*
	 * A train is moving from the domain of one PSU to another
	 */

	if (ptr->owner != msg.command.reply_to)
	{
	  log_msg (ERR, "%s reqd a handover (from PSU=%hu) but this PSU owned by %s", proc_from_msgq(msg.command.reply_to), msg.command.sub_command, proc_from_msgq(ptr->owner)) ;
	  return (0) ;
	}

	#ifdef PARANOID
	  if ((msg.command.sub_command > NUM_POWER_SOURCES) || (msg.command.sub_command == 0))
	  {
	    log_msg (ERR, "%s reqd a handover from impossible PSU=%hu", proc_from_msgq(msg.command.reply_to), msg.command.sub_command) ;
	    return (0) ;
	  }
	#endif

	#ifdef POWER_DEBUG
	  log_msg (FILE_ONLY, "Received HANDOVER_REQUEST from %s for PSU=%hu", proc_from_msgq(msg.command.reply_to), msg.command.sub_command) ;
	#endif

	ptr2 = &(*Power_State)[msg.command.sub_command-1] ;
	lock_sem (Power_State_Sem [msg.command.sub_command-1]) ;

	if (ptr2->owner != msg.command.reply_to)
	{
	  log_msg (ERR, "%s reqd a handover (from PSU=%hu) which is owned by %s", proc_from_msgq(msg.command.reply_to), msg.command.sub_command, proc_from_msgq(ptr2->owner)) ;
	  return (0) ;
	}

	ptr->speed = ptr2->speed ;
	ptr->target_speed = ptr2->target_speed ;
	ptr->acceleration = ptr2->acceleration ;
	unlock_sem (Power_State_Sem [msg.command.sub_command-1]) ;

        #ifdef POWER_DEBUG
	  log_msg (FILE_ONLY, "Handover current speed=%hu, accel=%hu, target speed=%hu by %s", ptr->speed, ptr->acceleration, ptr->target_speed, proc_from_msgq (msg.command.reply_to)) ;
        #endif

	send_dispatcher_req_msg (power_msgq[supply_number], supply_number+1, SPEED, ptr->speed) ;

	/*
	 * If there is some acceleration to do, start a timer
	 */

	if (ptr->acceleration != IMMEDIATELY)
	  send_timeout_req_msg (power_msgq[supply_number], Acceleration_Constant * ptr->acceleration, (int *)&ptr) ;

	break ;

      case SHUTDOWN_REQUEST:
	/*
	 * Somebody is telling the PSU to stop the train immediately!
	 * Cancel the timer if there is one running. Then reset everything.
	 */

	#ifdef POWER_DEBUG
	  log_msg (FILE_ONLY, "Received SHUTDOWN_REQUEST from %s", proc_from_msgq(msg.command.reply_to)) ;
	#endif

	#ifdef PARANOID
	  if ((msg.command.reply_to != 0) && (ptr->owner != 0))
	  {
	    log_msg (ERR, "%s attempted PSU shut down (owned by %s)", proc_from_msgq(msg.command.reply_to), proc_from_msgq(ptr->owner)) ;
	    return (0) ;
	  }
	#endif

	if (ptr->acceleration != IMMEDIATELY)
	  send_timeout_req_msg (power_msgq[supply_number], 0, (int *)&ptr) ;

	ptr->speed = ptr->target_speed = UNKNOWN_STATE ;
	ptr->acceleration = IMMEDIATELY ;
	send_dispatcher_req_msg (power_msgq[supply_number], supply_number+1, SPEED, ptr->speed) ;
	break ;

      case SET_SPEED_REQUEST:
	/*
	 * One of the engineer processes is asking for PSU action
	 */

	#ifdef PARANOID
	  if (msg.power.speed > MAX_SPEED)
	  {
	    log_msg (ERR, "%s tried to set illegal speed=%hu", proc_from_msgq(msg.command.reply_to), msg.power.speed) ;
	    return (0) ;
	  }
	#endif

	#ifdef POWER_DEBUG
	  log_msg (FILE_ONLY, "Received SET_SPEED_REQUEST from %s, speed=%hu, acc=%hu", proc_from_msgq(msg.command.reply_to), msg.power.speed, msg.power.acceleration) ;
	#endif

	if (ptr->owner != msg.command.reply_to)
	{
	  log_msg (ERR, "%s tried to change this PSU owned by %s", proc_from_msgq(msg.command.reply_to), proc_from_msgq(ptr->owner)) ;
	  return (0) ;
	}

	/*
	 * If immediate, set the PSU and exit. Otherwise, set the PSU to
	 * an initial value and start the timer at a rate defined by
	 * the acceleration parameter.
	 */

	ptr->target_speed = msg.power.speed ;
	ptr->acceleration = msg.power.acceleration ;

	if (msg.power.acceleration == IMMEDIATELY)
	{
	  ptr->speed = ptr->target_speed ;

	  if (ptr->acceleration != IMMEDIATELY)
	    send_timeout_req_msg (power_msgq[supply_number], 0, (int *)&ptr) ;

	}

	process_acceleration (supply_number, ptr, (int *)&ptr) ;
	break ;

      case TIMEOUT_NOTIFICATION:
        /*
	 * Timeouts mean that an acceleration event has happened
	 */

	#ifdef POWER_DEBUG
	  log_msg (FILE_ONLY, "Acceleration timeout. Old speed=%hu", ptr->speed) ;
	#endif

        #ifdef PARANOID
          if (msg.command.sub_command != (int)(&ptr))
	  {
	    log_msg (ERR, "Timeout received from unknown source (%lx but expected %lx)", msg.command.sub_command, (int)&ptr) ;
	    return (0) ;
	  }
	#endif

	if (ptr->acceleration == IMMEDIATELY)
	{
	  /*
	   * A timeout was launched before we could kill the timer.
	   * Just ignore it.
	   */

	  break ;
	}

	process_acceleration (supply_number, ptr, (int *)&ptr) ;
	break ;

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Bad msg recd type=%d, subcmd=%#0x, reply_to=%d", msg.command.mtype, msg.command.sub_command, msg.command.reply_to) ;
	  return (0) ;
	#endif
    }
  }
}

/******************************************************************************
 * controller_process
 *
 * This routine is where the master controller resides. The controller process
 * is designed to be something like an air traffic controller - an intelligent
 * entity which has dialogue with all the individual trains. The trains request
 * permission from the controller before they do anything and the controller
 * either grants permission or directs the train to do something else instead.
 *
 * The controller process does not drive trains - it only coordinates them.
 *
 * INITIALISATION SEQUENCE: READ THIS BEFORE READING THE CONTROLLER AND
 * ENGINEER SOFTWARE SOURCES BELOW!!!!
 *
 * 1. The user input process prompts the human for information about each of
 *    the trains. It then sends an initialisation message to each of the
 *    engineer processes.
 *
 * 2. The engineer process receives the initialisation message. It opens the
 *    route file which the user has specified. Then it scans through the route
 *    file until all blocks the train is allegedly sitting on are found - in
 *    the correct order. It also reads ahead in the route file and "caches"
 *    the next few blocks. The engineer completes its initialisation by sending
 *    a notification message to the controller informing all of the train's
 *    current blocks, (implicitly asking the controller for a PSU to be
 *    allocated for the train's engine).
 *
 * 3. The controller process receives the block notification and reserves them
 *    for that engineer. A PSU is allocated to the engine's block at the same
 *    time. The controller responds to the engineer notifying the PSU.
 *
 * 4. The engineer receives the confirmation and, having confirmed that the
 *    blocks that the train and its engine are on are allocated to it, sends
 *    points messages to each of the points controllers to ensure that the
 *    points in the track blocks which have been assigned to the train are
 *    set to the correct initial values.  The engineer then waits on the user
 *    input process to tell it when to start moving the train.
 *
 * 5. The user input process initialises each engineer, one by one, as
 *    described above. When the last engineer is initialised, the user input
 *    process sends all engineers a start message.
 *
 * 6. Now that all engineers have received their block allocations for the
 *    blocks that their trains are on and have received PSU allocations for
 *    their engines, each engineer looks at the next block in their routes
 *    and sees if it is different from the blocks their train is presently
 *    on. Normally, this is the case. The engineers send new block request
 *    messages to the controller for their next intended block.
 *
 * 7. The controller receives the new block requests one by one and grants
 *    or denys them depending upon sensibility, train priority etc.
 *
 * 8. The engineers receive the controller's block grants and check to see
 *    if the next block requires a different power supply. If so, the
 *    engineer must send a PSU request message to the controller before
 *    continuing. Otherwise, if the PSU for the next block is also the PSU
 *    for the block on which the engine is located, the engineer can set
 *    the block polarity and PSU relays for the next block and then send
 *    a message to that particular power process to ask that the train be
 *    accelerated.
 *
 * 9. The train is now running normally and the above process continues.
 *    Engineers request new blocks and PSUs as the train moves forwards
 *    and they release old blocks and PSUs as the train leaves them.
 *    For the engineer processes in particular, there are many different
 *    types of message which could be received and from a large number of
 *    different sources. So each engineer needs to keep track of its
 *    state very carefully so that it knows what to expect next.
 */

void
snapshot_plan (volatile ROUTE_PLAN (*plan))
{
  Ushort i ;
  char *a, *b, *d ;

  if (plan->acc_state == ACCELERATING)
    a="ACCEL" ;
  else if (plan->acc_state == AT_SPEED)
    a="AT_SPEED" ;
  else if (plan->acc_state == DECELERATING)
    a="DECEL" ;
  else if (plan->acc_state == STOPPED)
    a="STOPPED" ;
  else if (plan->acc_state == DECELERATING_TO_HALT)
    a="DECEL_TO_HALT" ;
  else
    a="DECEL_TO_STOP" ;

  if (plan->acc == IMMEDIATELY)
    b = "IMMED" ;
  else if (plan->acc == QUICKLY)
    b = "QUICK" ;
  else if (plan->acc == MODERATELY)
    b = "MOD" ;
  else if (plan->acc == SLOWLY)
    b = "SLOW" ;
  else
    b = "VSLOW" ;

  d = (plan->direction == FORWARDS) ? "FWD" : "REV" ;

  log_msg (FILE_ONLY, "train_num=%hu, length=%hu, acc=%s %s, dir=%s, speed=%2.2f, last_speed=%hu", plan->train_num, plan->length, a, b, d, plan->circumstantial_speed, plan->last_speed) ;
  log_msg (FILE_ONLY, "num_blocks=%hu, num_rev=%hu, num_rev_route=%hu, num_next=%hu, num_reserv=%hu", plan->num_blocks, plan->num_rev, plan->num_rev_route, plan->num_next, plan->num_reservations) ;

  for (i=0 ; i < plan->num_rev_route ; i++)
    log_msg (FILE_ONLY, "blocks[%d]= %d", i, plan->blocks[i]) ;

  for (i=0 ; i < plan->num_next ; i++)
    log_msg (FILE_ONLY, "next[%d]= %d", i, plan->next[i]) ;

  for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
    log_msg (FILE_ONLY, "PSU[%d]= %d", i, plan->PSU[i]) ;

  for (i=0 ; i < NUM_GRANTS ; i++)
    log_msg (FILE_ONLY, "blk_grnt[%d]= %d", i, plan->block_grants[i]) ;

  for (i=0 ; i < NUM_GRANTS ; i++)
    log_msg (FILE_ONLY, "blk_retn[%d]= %d", i, plan->block_returns[i]) ;

  for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
    log_msg (FILE_ONLY, "psu_grnt[%d]= %d", i, plan->psu_grants[i]) ;
}

void
do_power_supply_test (void)
{
  CONTROLLER_MSG_UNION msg ;
  Ushort               i ;
  Ushort               num ;

  /*
   * Perform the power supply test
   */

  log_msg (ERR, "Performing supply test") ;
  sleep (2) ;

  for (i=0 ; i < NUM_POWER_SOURCES ; i++)
  {
    lock_sem (Power_State_Sem [i]) ;
    (*Power_State)[i].owner = test_msgq ;
    unlock_sem (Power_State_Sem [i]) ;
  }

  lock_sem (Block_Allocation_Sem) ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
    BLK_OWNER(i+1) = test_msgq ;

  unlock_sem (Block_Allocation_Sem) ;

  for (i=1 ; i <= NUM_POWER_SOURCES ; i++)
  {
    if (Test_Supply != -1)
      num = Test_Supply ;
    else
      num = i ;

    #ifdef BASIC_DEBUG
      log_msg (NORM, "Testing Power Supply #%d", num) ;
      log_msg (NORM, "About to set max speed in PSU %d", num) ;
    #endif

    /*
     * accelerate a lot slower than normal
     */

    send_power_msg (num, test_msgq, SET_SPEED_REQUEST, 0, MAX_SPEED, TEST_ACCEL) ;
    read_mesg (test_msgq, (LINUX_MSG_BUF *)&msg, sizeof(msg)) ;

    #ifdef PARANOID
      if (msg.command.mtype != SPEED_NOTIFICATION)
	log_msg (ERR, "Expected to receive a SPEED_NOTIFICATION response but received %d", msg.command.mtype) ;
      else if (msg.engineer.speed != MAX_SPEED)
	log_msg (ERR, "Speed supposed to be MAX_SPEED (%d) but power process respondend with %d", MAX_SPEED, msg.engineer.speed) ;
    #endif

    #ifdef BASIC_DEBUG
      log_msg (NORM, "About to set 0 speed in PSU %d", num) ;
    #endif

    send_power_msg (num, test_msgq, SET_SPEED_REQUEST, 0, 0, TEST_ACCEL) ;
    read_mesg (test_msgq, (LINUX_MSG_BUF *)&msg, sizeof(msg)) ;

    #ifdef PARANOID
      if (msg.command.mtype != SPEED_NOTIFICATION)
	log_msg (ERR, "Expected to receive a SPEED_NOTIFICATION response but received %d", msg.command.mtype) ;
      else if (msg.engineer.speed != 0)
	log_msg (ERR, "Speed supposed to be 0 but power process respondend with %d", msg.engineer.speed) ;
    #endif
  }
}

void
do_block_test (Ushort delay)
{
  Ushort               i ;
  Ushort               num ;

  /*
   * Perform the Block test
   */

  log_msg (NORM, "Performing block test") ;
  sleep (2) ;

  for (i=0 ; i < NUM_POWER_SOURCES ; i++)
  {
    lock_sem (Power_State_Sem [i]) ;
    (*Power_State)[i].owner = test_msgq ;
    unlock_sem (Power_State_Sem [i]) ;
  }

  lock_sem (Block_Allocation_Sem) ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
    BLK_OWNER(i+1) = test_msgq ;

  unlock_sem (Block_Allocation_Sem) ;

  /*
   * Set power supplies alternately to maximum and half voltage.
   * Set all blocks to NORMAL direction and the lower numbered power supply.
   */

  for (i=1 ; i <= NUM_POWER_SOURCES ; i++)
    send_power_msg (i, test_msgq, SET_SPEED_REQUEST, 0, (i%2)?MAX_SPEED:MAX_SPEED/2, IMMEDIATELY) ;

  for (i=1 ; i <= NUM_BLOCKS ; i++)
  {
    send_dispatcher_req_msg (test_msgq, i, BLOCK_DIR, NORMAL_POLARITY) ;
    send_dispatcher_req_msg (test_msgq, i, BLOCK_POWER_SRC, PSU_1) ;
  }

  for (i=1 ; i <= NUM_BLOCKS ; i++)
  {
    if (Test_Block != -1)
      num = Test_Block ;
    else
      num = i ;

    #ifdef BASIC_DEBUG
      log_msg (NORM, "Testing Block %d", num) ;
    #endif

    sleep (delay) ;
    send_dispatcher_req_msg (test_msgq, num, BLOCK_POWER_SRC, PSU_2) ;
    sleep (delay) ;
    send_dispatcher_req_msg (test_msgq, num, BLOCK_DIR, OPPOSITE_POLARITY) ;
    sleep (delay) ;
    send_dispatcher_req_msg (test_msgq, num, BLOCK_POWER_SRC, PSU_1) ;

    if (Test_Block != -1)
    {
      sleep (delay) ;
      send_dispatcher_req_msg (test_msgq, num, BLOCK_DIR, NORMAL_POLARITY) ;
    }
  }
}

void
do_points_test (void)
{
  Ushort               i ;
  Ushort               num ;

  /*
   * Perform the points test
   */

  log_msg (NORM, "Performing points test") ;
  sleep (2) ;

  lock_sem (Block_Allocation_Sem) ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
    BLK_OWNER(i+1) = test_msgq ;

  unlock_sem (Block_Allocation_Sem) ;

  for (i=1 ; i <= NUM_POINTS ; i++)
  {
    if (Test_Points != -1)
      num = Test_Points ;
    else
      num = i ;

    #ifdef BASIC_DEBUG
      log_msg (NORM, "Testing points #%d", num) ;
    #endif

    send_points_msg (num, OPEN,   0, test_msgq) ;
    send_points_msg (num, CLOSED, 0, test_msgq) ;
    send_points_msg (num, OPEN,   0, test_msgq) ;
    send_points_msg (num, CLOSED, 0, test_msgq) ;
    send_points_msg (num, OPEN,   0, test_msgq) ;
    send_points_msg (num, CLOSED, 0, test_msgq) ;
  }
}

/*-----------------------------------------------------------------------------
 * The following routines are used to parse the route map and find certain
 * characteristics.
 *
 * find_longest_exit checks all exits from a block in a particular direction
 * and returns the block number which is the LONGEST distance from the block's
 * mid point.
 *
 * find_shortest_exit does the opposite.
 */

Ushort
find_longest_exit (const Ushort block, const TRACK_POLARITY polarity)
{
  Ushort    i, len, blk ;
  ADJACENCY *X ;

  len = blk = 0 ;

  for (i=0 ; i < DEAD_END ; i++)
  {
    X = ADJ(block, i+1) ;

    if ((X == NULL) || (X->polarity != polarity))
      continue ;

    if (X->length > len)
    {
      len = X->length ;
      blk = X->adj_seg ;
    }
  }

  return (blk) ;
}

Ushort
find_shortest_exit (const Ushort block, const TRACK_POLARITY polarity)
{
  Ushort    i, len, blk ;
  ADJACENCY *X ;

  len = 0xffff ;
  blk = 0 ;

  for (i=0 ; i < DEAD_END ; i++)
  {
    X = ADJ(block, i+1) ;

    if ((X == NULL) || (X->polarity != polarity))
      continue ;

    if (X->length < len)
    {
      len = X->length ;
      blk = X->adj_seg ;
    }
  }

  return (blk) ;
}

Ushort
find_longest_exit_length (const Ushort block, const TRACK_POLARITY polarity)
{
  return (ADJ(block, find_longest_exit(block, polarity))->length) ;
}

Ushort
find_shortest_exit_length (const Ushort block, const TRACK_POLARITY polarity)
{
  return (ADJ(block, find_shortest_exit(block, polarity))->length) ;
}

/*
 * find_partial_len_of_allocated_blocks is a routine which adds the
 * length of all the blocks presently allocated to the train except
 * the last half of the last block (from the perspective of direction
 * of travel).
 *
 * This routine is used when the train is going to stop and change
 * direction and is intended to be used in the calculation of whether
 * the train can clear the last half of the last block before entering
 * the next block or not.
 */

Ushort
find_partial_len_of_allocated_blocks (volatile ROUTE_PLAN (*plan))
{
  Ushort len, i ;

  /*
   * Depending upon the direction we're heading, need to find the length of all
   * blocks that the train is on top of either without the first half of the
   * engine's block or without the last half of the last prev block.
   */

  len = (plan->direction == FORWARDS) ? ADJ(plan->blocks[0], plan->next[0])->length : 0 ;

  for (i=0 ; i < (plan->num_blocks - 1) ; i++)
  {
    len += ADJ(plan->blocks[i], plan->blocks[i+1])->length ;
    len += ADJ(plan->blocks[i+1], plan->blocks[i])->length ;
  }

  if (plan->direction == REVERSE)
    len += ADJ(plan->blocks[plan->num_blocks-1], plan->blocks[plan->num_blocks])->length ;

  return (len) ;
}

Ushort
find_length_of_leading_block (volatile ROUTE_PLAN (*plan))
{
  Ushort len ;

  if ((plan->direction == FORWARDS) || (plan->num_blocks == 1))
  {
    len = ADJ(plan->blocks[0], plan->next[0])->length ;
    len += (plan->blocks[1] != 0) ? ADJ(plan->blocks[0], plan->blocks[1])->length : find_shortest_exit_length (plan->blocks[0], OPPOSITE_BLOCKS_POLARITY(plan->blocks[0],plan->next[0])) ;
  }
  else
  {
    len = ADJ(plan->blocks[plan->num_blocks-1], plan->blocks[plan->num_blocks-2])->length ;
    len += (plan->blocks[plan->num_blocks] != 0) ? ADJ(plan->blocks[plan->num_blocks-1], plan->blocks[plan->num_blocks])->length : find_shortest_exit_length (plan->blocks[plan->num_blocks-1], OPPOSITE_BLOCKS_POLARITY(plan->blocks[plan->num_blocks-1], plan->blocks[plan->num_blocks-2])) ;
  }

log_msg (FILE_ONLY, "Len of leading block=%hu", len) ;
  return (len) ;
}

CONTROLLER_SUBCMD
allocate_psu (volatile ROUTE_PLAN (*plan), Ushort card_num)
{
  Ushort           i, j ;
  Ushort           psu_num ;
  POWER_STATE_INFO *ptr ;

  psu_num = card_num * 2 ;
  ptr = &(*Power_State)[psu_num] ;
  lock_sem (Power_State_Sem [psu_num]) ;

  if (ptr->owner != 0)
  {
    unlock_sem (Power_State_Sem [psu_num]) ;
    psu_num++ ;
    ptr = &(*Power_State)[psu_num] ;
    lock_sem (Power_State_Sem [psu_num]) ;
  }

  if (ptr->owner == 0)
  {
    /*
     * We've found a free PSU. Set the ownership of the PSU
     * here and now
     */

    ptr->owner = engineer_msgq[plan->train_num-1] ;
    unlock_sem (Power_State_Sem [psu_num]) ;

    for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
      if (plan->PSU[i] == 0)
	break ;

    if (i >= NUM_PSU_ALLOCATIONS)
    {
      log_msg (ERR, "Can't allocate PSU because no free entries in plan->PSU") ;
      snapshot_plan (plan) ;
      _exit (0) ;
    }

    for (j=0 ; j < NUM_PSU_ALLOCATIONS ; j++)
      if (plan->psu_grants[j] == 0)
	break ;

    if (j >= NUM_PSU_ALLOCATIONS)
    {
      log_msg (ERR, "Can't allocate PSU to train=%d. No space in plan->psu_grants", plan->train_num) ;
      snapshot_plan (plan) ;
      _exit (0) ;
    }

    plan->psu_grants[j] = plan->PSU [i] = psu_num + 1 ;

    #ifdef CONTROLLER_DEBUG
      log_msg (FILE_ONLY, "Allocating PSU=%d to train=%d", psu_num+1, plan->train_num) ;
    #endif

    return (RESPONSE) ;
  }
  else
  {
    /*
     * Both PSUs on that controller card are allocated.
     * Check to see if we can go on wait.
     */

    unlock_sem (Power_State_Sem [psu_num]) ;

    for (i=0 ; i < NUM_RESERVATIONS ; i++)
      if (PSU_Reservation [card_num][i] == 0)
	break ;

    if (i >= NUM_RESERVATIONS)
    {
      log_msg (ERR, "Train=%d requested psu. PSUs owned by %s and %s. No free reservations.", plan->train_num, proc_from_msgq((*Power_State)[psu_num].owner), proc_from_msgq((*Power_State)[psu_num-1].owner)) ;
      return (0) ;
    }

    PSU_Reservation [card_num][i] = plan->train_num ;
    plan->num_reservations++ ;

    #ifdef CONTROLLER_DEBUG
      log_msg (FILE_ONLY, "Reservation=%d for psu on card=%d for train=%d", plan->num_reservations, card_num, plan->train_num) ;
    #endif

    return (DELAY) ;
  }
}

CONTROLLER_SUBCMD
check_and_allocate_psu (volatile ROUTE_PLAN (*plan))
{
  Ushort j, block ;

  /*
   * If the train's direction is FORWARDS, need to allocate PSU
   * for ->blocks[0] and ->next[0]. If the direction is REVERSE,
   * need to allocate PSU for ->blocks[0] and ->blocks[1].
   *
   * But if a PSU has already been allocated for either of
   * these blocks, then no need to allocate another.
   */

  memset ((void *)(plan->psu_grants), 0, sizeof(plan->psu_grants)) ;

  for (j=0 ; j < NUM_PSU_ALLOCATIONS ; j++)
    if ((plan->PSU[j] != 0) && (CARD_FROM_PSU(plan->PSU[j]) == CARD_FROM_BLOCK(plan->blocks[0])))
      break ;

  if ((j >= NUM_PSU_ALLOCATIONS) && (allocate_psu (plan, CARD_FROM_BLOCK(plan->blocks[0])) == DELAY))
  {
    log_msg (ERR, "No PSUs available for train=%d's engine's block=%d", plan->train_num, plan->blocks[0]) ;
    snapshot_plan (plan) ;
    _exit (0) ;
  }

  block = (plan->direction == FORWARDS) ? plan->next[0] : plan->blocks[1] ;

  for (j=0 ; j < NUM_PSU_ALLOCATIONS ; j++)
    if ((plan->PSU[j] != 0) && (CARD_FROM_PSU(plan->PSU[j]) == CARD_FROM_BLOCK(block)))
      break ;

  if ((j >= NUM_PSU_ALLOCATIONS) && (allocate_psu (plan, CARD_FROM_BLOCK(block)) == DELAY))
    return (DELAY) ;

  return (RESPONSE) ;
}

void
check_and_rm_psu (volatile ROUTE_PLAN (*plan))
{
  Ushort           i, j ;
  Ushort           card_num, psu_num ;
  POWER_STATE_INFO *ptr ;

  memset ((void *)(plan->psu_grants), 0, sizeof(plan->psu_grants)) ;

  /*
   * For every power supply which is presently owned, check if
   * there is a corresponding block which is being controlled
   * by that power supply.
   *
   * If direction is FORWARDS, only need to have the engine and
   * next[0] block controlled by a power supply. In the REVERSE
   * direction, the engine and all blocks[0] blocks need to have
   * a power supply.
   *
   * Return the list of released PSUs in psu_grants[]
   */

  for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
  {
    if ((psu_num = plan->PSU[i]) == 0)
      continue ;

    card_num = CARD_FROM_PSU(psu_num) ;

    if (CARD_FROM_BLOCK(plan->blocks[0]) == card_num)
      continue ;

    if ((plan->direction == FORWARDS) && (CARD_FROM_BLOCK(plan->next[0]) == card_num))
      continue ;

    if ((plan->direction == REVERSE) && (CARD_FROM_BLOCK(plan->blocks[1]) == card_num))
      continue ;

    for (j=0 ; j < NUM_PSU_ALLOCATIONS ; j++)
      if (plan->psu_grants[j] == 0)
	break ;

    if (j >= NUM_PSU_ALLOCATIONS)
    {
      log_msg (ERR, "Deallocating too many PSUs for train %hu", plan->train_num) ;
      snapshot_plan (plan) ;
      _exit (0) ;
    }

    #ifdef CONTROLLER_DEBUG
      log_msg (FILE_ONLY, "train=%hu returning PSU=%hu", plan->train_num, psu_num) ;
    #endif

    ptr = &(*Power_State)[psu_num-1] ;
    lock_sem (Power_State_Sem [psu_num-1]) ;
    ptr->owner = 0 ;
    unlock_sem (Power_State_Sem [psu_num-1]) ;
    plan->psu_grants[j] = psu_num ;
    plan->PSU[i] = 0 ;
    send_power_msg (psu_num, engineer_msgq[plan->train_num-1], SHUTDOWN_REQUEST, 0, 0, 0) ;

    if (plan->num_stop > 0)
      (plan->num_stop)-- ;
  }
}

/*
 * find_stub is the routeine which is called every time a train needs to change
 * its direction or to temporarily divert to a different route.
 *
 * It recursively searches for a route from a specicfied starting block in a
 * specified direction. The train is intended to divert along this route.
 *
 * It takes into account the length of the train so that once the train has
 * been completely diverted onto the stub route and off its original starting
 * block, the search is complete.
 *
 * There are two possible types of situation: the first is called an EXIT
 * solution and the second an ENTRY solution. The exit solution is where the
 * train needs to completely exit the originating block before it stops to
 * turn back again. The entry solution is where the train only needs to
 * touch the last block on the stub and can still be hanging over the last
 * few centimetres of the original block.
 */

void
generate_forbidden_list (Ushort *list, Ushort *num)
{
  Ushort i ;

  /*
   * It is assumed that Block_Allocation_Sem has been siezed
   */

  *num = 0 ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
    if (BLK_OWNER(i+1) != 0)
      list[(*num)++] = i + 1 ;
}

void
add_to_forbidden_list (Ushort *list, Ushort *num, Ushort block)
{
  Ushort i ;

  /*
   * It is assumed that Block_Allocation_Sem has been siezed
   */

  for (i=0 ; i < NUM_BLOCKS ; i++)
    if (list[i] == block)
      return ;

  list[(*num)++] = block ;
}

void
find_stub (Ushort *forbidden_list, Ushort num_forbidden, Ushort remaining_train_len, Ushort len_first_blk, Ushort current_blk, Ushort prev_blk,
	   TRACK_POLARITY polarity, Ushort num_stubs, Ushort *calculated_num_stubs, Ushort *calculated_stub_list,
	   Ushort *calculated_overshoot, DIRCHNG_STATE *calculated_solution_type)
{
  Ushort        i, j ;
  Ushort        temp_len ;
  static Ushort stub_list [NUM_STUB_BLOCKS] ;

static int num_recurs ;
log_msg (ERR, "here I am") ;
log_msg (ERR, "num_forb=%hu, rem_len=%hu, len_first=%hu curr=%hu, prev=%hu, num_stub=%hu", num_forbidden, remaining_train_len, len_first_blk, current_blk, prev_blk, num_stubs) ;

  if (prev_blk == 0)
  {
    /*
     * If this is the first call, initialise the result variables.
     * This saves us having to do it in the caller.
     */
num_recurs=1 ;
    memset ((void *)stub_list, 0, sizeof(Ushort)*NUM_STUB_BLOCKS) ;
    memset ((void *)calculated_stub_list, 0, sizeof(Ushort)*NUM_STUB_BLOCKS) ;
    *calculated_num_stubs = 0xffff ;
    *calculated_overshoot = 0xffff ;
    len_first_blk = 0 ;

    #ifdef CONTROLLER_DEBUG
      log_msg (FILE_ONLY, "About to find stubs from block=%hu, polarity=%hu", current_blk, polarity) ;
    #endif
  }
else log_msg (ERR, "entring for %dth time", ++num_recurs) ;

  for (i=0 ; i < DEAD_END ; i++) /* search all blocks from current block */
  {
    if ((ADJ(current_blk, i+1) == 0) || (ADJ(current_blk, i+1)->polarity != polarity))
      continue ;

    for (j=0 ; j < num_forbidden ; j++)
      if (forbidden_list[j] == (i+1))
	break ;

    if (j < num_forbidden) /* this block has been disallowed */
      continue ;

    forbidden_list [num_forbidden] = i+1 ;
    stub_list [num_stubs] = i+1 ;
    temp_len = ADJ(current_blk, i+1)->length ;

    if (prev_blk != 0) /* don't consider back half of 1st block in recursion */
    {
      temp_len += ADJ(current_blk, prev_blk)->length ;

      if ((remaining_train_len < temp_len) && (num_stubs <= *calculated_num_stubs))
      {
	/*
	 * We have a solution of the EXIT form. (i+1) will be entered.
	 * This is the best solution if it involves fewer blocks to
	 * request or if it involves a smaller overshoot
	 */

	if ((num_stubs < *calculated_num_stubs) || (len_first_blk < *calculated_overshoot))
	{
	  memcpy (calculated_stub_list, stub_list, sizeof(Ushort)*(num_stubs+1)) ;
	  *calculated_num_stubs = num_stubs ;
	  *calculated_overshoot = len_first_blk ;
	  *calculated_solution_type = EXIT ;

          #ifdef CONTROLLER_DEBUG
	    log_msg (FILE_ONLY, "Found EXIT solution. num_stubs=%d, overshoot=%d, block=%d", num_stubs, len_first_blk, current_blk) ;
          #endif
	}

	continue ; /* continue searching to try and get better solution */
      }
    }

    if (((len_first_blk + temp_len) > remaining_train_len) && (num_stubs <= *calculated_num_stubs))
    {
      /*
       * We have a solution of the ENTRY form. (i+1) needs to be touched.
       */

      temp_len = len_first_blk + temp_len - remaining_train_len ;

      if ((num_stubs < *calculated_num_stubs) || (temp_len < *calculated_overshoot))
      {
	memcpy (calculated_stub_list, stub_list, sizeof(Ushort)*(num_stubs+1)) ;
	*calculated_num_stubs = num_stubs + 1 ;
	*calculated_overshoot = temp_len ;
	*calculated_solution_type = ENTRY ;

        #ifdef CONTROLLER_DEBUG
log_msg (FILE_ONLY, "Recursed %d times", num_recurs) ;
	  log_msg (FILE_ONLY, "Found ENTRY solution. num_stubs=%d, overshoot=%d, block=%d", num_stubs+1, temp_len, i+1) ;
        #endif
      }

      continue ;
    }

    if ((num_stubs + 1) > *calculated_num_stubs)
      continue ; /* no point in continuing if a better solution exists */

    /*
     * We've not yet found a solution. Need to recurse.
     */

    if (i == DEAD_END)
      continue ;

    if (prev_blk == 0)
      len_first_blk = temp_len ;

    find_stub (forbidden_list, num_forbidden+1, remaining_train_len-temp_len, (prev_blk == 0) ? temp_len : len_first_blk, i+1, current_blk,
	       OPPOSITE_BLOCKS_POLARITY(i+1, current_blk), num_stubs+1, calculated_num_stubs, calculated_stub_list, calculated_overshoot,
	       calculated_solution_type) ;
  }
}

/*
 * determine route is the thinking part of the controller. Here is where
 * deadlocks will be detected and an avoidance strategy devised.
 */

int
determine_route (volatile ROUTE_PLAN (*ptr))
{
  Ushort        i, j ;
  Ushort        block ;
  Ushort        num_alloc ;

  Ushort        forbidden_list [NUM_BLOCKS] ;
  Ushort        num_forbidden ;
  Ushort        stub_list [NUM_STUB_BLOCKS] ;
  Ushort        num_stubs ;
  Ushort        overshoot ;
  DIRCHNG_STATE solution ;

  memset ((void *)ptr->block_grants, 0, NUM_GRANTS*sizeof(ptr->block_grants[0])) ;
  num_alloc = 0 ;
  solution = NO_STATE ;

  #ifdef PARANOID
    if (ptr->next[0] == ptr->blocks[0])
    {
      log_msg (ERR, "Nowhere to go! plan.blocks[0]=%d == plan.next[0]", ptr->blocks[0]) ;
      _exit (0) ;
    }
  #endif

  /*
   * Check to see if there will be a direction change happening. If so, ensure
   * that the entire train can fit onto the stub before the changed direction
   * otherwise, need to add some blocks into the path.
   *
   * The number of look ahead blocks (the top value of the loop counter below)
   * is debatable. Don't want too far ahead because we could miss the
   * opportunity to benefit from a newly (reduced) calculated train length and
   * so add additional blocks into the route unnecessarily. On the other hand,
   * it is better to pick up the direction change ASAP and adjust the route.
   */

  block = ptr->blocks[0] ;

  for (i=0 ; i < REVERSE_LOOKAHEAD ; i++)
  {
    if (!HEADING_SAME_DIRECTION(block, ptr->next[i], ptr->next[i+1]))
      break ;

    block = ptr->next[i] ;
  }

  if (i < REVERSE_LOOKAHEAD)
  {
    /*
     * ptr->next[i] is the furthest block in current direction of travel.
     * Press further into next and determine commone part of stub (all the
     * way to the point of divergence)
     */

    for (num_stubs=1 ; (num_stubs <= i) && ((i+num_stubs) < ptr->num_next) ; num_stubs++)
      if (ptr->next[i-num_stubs] != ptr->next[i+num_stubs])
	break ;
      else
	stub_list[num_stubs] = ptr->next[i+num_stubs] ;

    /*
     * At present, I assume that the number of REVERSE_LOOKAHEAD blocks
     * is smaller than half the number of MIN_NEXT_BLOCKS. The following
     * if test will sometimes break if this is not the case.
     */

for(j=0 ; j<num_stubs;j++)
log_msg (FILE_ONLY, "FF stub[%hu]=%hu", j, stub_list[j]) ;
for(j=0 ; j<ptr->num_blocks;j++)
log_msg (FILE_ONLY, "FF block[%hu]=%hu", j, ptr->blocks[j]) ;
for(j=0 ; j<ptr->num_next;j++)
log_msg (FILE_ONLY, "FF next[%hu]=%hu", j, ptr->next[j]) ;
log_msg(FILE_ONLY, "i=%hu, num_stubs=%hu, num_next=%hu, num_blocks=%hu", i, num_stubs, ptr->num_next, ptr->num_blocks) ;

    if ((i+num_stubs) < ptr->num_next)
    {
      if (i == (num_stubs-1))
      {
      	for (j=0 ; (j < ptr->num_blocks) && ((i+num_stubs) < ptr->num_next) ; j++)
	{
	  if (ptr->blocks[j] != ptr->next[i+num_stubs])
	    break ;
	  else
	  {
	    stub_list[num_stubs] = ptr->next[i+num_stubs] ;
	    num_stubs++ ;
	  }
	}
      }

      stub_list[0] = ptr->next[i] ;

      #ifdef CONTROLLER_DEBUG
        log_msg (FILE_ONLY, "Dir chng taking place. # common blocks in rev path=%hu", num_stubs-1) ;
      #endif

for(j=0 ; j<num_stubs;j++)
log_msg (FILE_ONLY, "XX stub[%hu]=%hu", j, stub_list[j]) ;
      if (num_stubs > 1)
      {
	/*
	 * See if we can find a solution of type ENTRY.
	 * [Engine just touching the stub[0] block, tail of train
	 * past the mid point of the stub[num_stubs] block]
	 */

	for (overshoot=0,j=1 ; j < (num_stubs-1) ; j++)
        {
	  overshoot += ADJ(stub_list[j], stub_list[j-1])->length ;
	  overshoot += ADJ(stub_list[j], stub_list[j+1])->length ;
	}

	overshoot += ADJ(stub_list[num_stubs-1], stub_list[num_stubs-2])->length ;

	if (overshoot >= ptr->length)
	{
	  solution = ENTRY ;

          #ifdef CONTROLLER_DEBUG
            log_msg (FILE_ONLY, "Present route OK for ENTRY solution. overshoot=%hu, train len=%hu", overshoot, ptr->length) ;
          #endif
	}
      }

      if ((num_stubs > 1) && (solution == NO_STATE))
      {
	/*
	 * See if we can find a solution of type EXIT.
	 * [Engine within the stub[0] block, tail of train
	 * exited the stub[num_stubs] block]
	 */

	overshoot = ADJ(stub_list[0], stub_list[1])->length ;
	overshoot += find_longest_exit_length (stub_list[0], OPPOSITE_BLOCKS_POLARITY(stub_list[0], stub_list[1])) ;

	for (j=1 ; j < (num_stubs-1) ; j++)
        {
	  overshoot += ADJ(stub_list[j], stub_list[j-1])->length ;
	  overshoot += ADJ(stub_list[j], stub_list[j+1])->length ;
	}

	overshoot += ADJ(stub_list[num_stubs-1], stub_list[num_stubs-2])->length ;
	overshoot += find_longest_exit_length (stub_list[num_stubs-1], OPPOSITE_BLOCKS_POLARITY(stub_list[num_stubs-1], stub_list[num_stubs-2])) ;

	if (overshoot >= (ptr->length + STOPPING_UNCERTAINTY))
	{
	  solution = EXIT ;

          #ifdef CONTROLLER_DEBUG
            log_msg (FILE_ONLY, "Present route OK for EXIT solution. overshoot=%hu, train len=%hu", overshoot, ptr->length) ;
          #endif
	}
      }

      if (solution == NO_STATE)
      {
	generate_forbidden_list (forbidden_list, &num_forbidden) ; /* from all allocated blocks */

	for (j=0 ; j <= i ; j++)
	  add_to_forbidden_list (forbidden_list, &num_forbidden, ptr->next[j]) ;

log_msg (ERR, "i=%hu, num_forbidden=%hu, cur_blk=%hu", i, num_forbidden, ptr->next[i]) ;
for(j=0 ; j<num_forbidden ; j++) log_msg (ERR, "forb[%hu]=%hu", j, forbidden_list[j]) ;
/*
find_stub (pointer to Ushort forbidden_list, num_forbidden entries, CURRENT TRAIN LEN, 0, STARTING BLOCK, 0, POLARITY TO GO IN, 0,
   pointer to Ushort calculated_num_stubs, pointer to Ushort calculated_stub_list[NUM_STUB_BLOCKS], pointer to Ushort calculated_overshoot,
   pointer to DIRCHNG_STATE calculated_solution_type) ;

find_stub (Ushort *forbidden_list, Ushort num_forbidden, Ushort remaining_train_len, Ushort len_first_blk, Ushort current_blk, Ushort prev_blk,
	   TRACK_POLARITY polarity, Ushort num_stubs, Ushort *calculated_num_stubs, Ushort *calculated_stub_list,
	   Ushort *calculated_overshoot, DIRCHNG_STATE *calculated_solution_type)
*/

        find_stub (forbidden_list, num_forbidden, ptr->length, 0, ptr->next[i], 0, ADJ(block, ptr->next[i])->polarity, 0, &num_stubs, stub_list, &overshoot, &solution) ;
/***** PUT IN A CHECK FOR NO SOLUTION ******/

        #ifdef CONTROLLER_DEBUG
          log_msg (FILE_ONLY, "Calculated new stub route of length=%hu, overshoot=%hu", num_stubs, overshoot) ;

for (i=0 ; i < num_stubs ; i++)
log_msg (FILE_ONLY, "stub[%hu]=%hu", i, stub_list[i]) ;
        #endif

	if (ptr->num_next <= (NUM_NEXT_BLOCKS-2))
        {
	  /*
	   * Add ONE additional stub block (and a return) into the next
	   * array - a total of two new blocks
	   */

	  for (j=NUM_NEXT_BLOCKS-1 ; j > (i+2) ; j--)
	  {
	    ptr->next [j]       = ptr->next       [j-2] ;
	    ptr->next_speed [j] = ptr->next_speed [j-2] ;
	  }

	  ptr->next [i+1] = stub_list [1] ;
	  ptr->next [i+2] = ptr->next [i] ;
	  ptr->next_speed [i+1] = ptr->next_speed [i+2] = ptr->next_speed [i] ;
	  ptr->num_next += 2 ;
	}
      }
    }
  }

  if (!HEADING_SAME_DIRECTION(ptr->blocks[0], ptr->next[0], ptr->next[1]))
  {
    /*
     * If we're about to change direction, we need to set the ptr->dirchng state
     * to tell the engineer how to handle the direction change.
     */

    ptr->dirchng = solution ;
    ptr->expect_block = (solution == ENTRY) ? stub_list[0] : stub_list[num_stubs-1] ;
  }

  if (ptr->direction == REVERSE)
  {
    /*
     * When we need to reverse, we need to step backwards carefully.
     * If the train is sufficiently long, its rear will overhang the
     * elbow in the route onto one or more additional blocks. We need
     * to determine which those blocks are going to be and add them into
     * the route.
     */

    log_msg (ERR, "B: Reverse not yet implemented. Train=%d", ptr->train_num) ;
    _exit (0) ;
  }

  /*
   * Which block do we need? It depends on direction of travel.
   */

  block = (ptr->direction == FORWARDS) ? ptr->next[0] : ptr->next[ptr->num_blocks-1] ; /* should this be +1 instead of -1? */
#ifdef ENGINEER_DEBUG
  log_msg (FILE_ONLY, "train=%d requesting next block=%d", ptr->train_num, block) ;
#endif

ptr->block_grants[0] = block ;
return (1) ;

  do
  {
    for (i=0 ; i < ptr->num_rev ; i++)
      if (ptr->blocks[i] == block)
	break ;

    if (i >= ptr->num_rev)
    {
      /*
       * We need to request the next block before anything else can
       * take place.
       */

      #ifdef ENGINEER_DEBUG
        log_msg (FILE_ONLY, "train=%d requesting next block=%d", ptr->train_num, block) ;
      #endif

      ptr->block_grants[0] = block ;
      num_alloc++ ;
    }
  }
  while (block-- > 0) ;

  return (num_alloc) ;
}

CONTROLLER_SUBCMD
allocate_blocks (volatile ROUTE_PLAN (*plan))
{
  Ushort            i, j, block ;
  CONTROLLER_SUBCMD ret_val ;

  ret_val = RESPONSE ;

  for (i=0 ; i < NUM_GRANTS ; i++)
  {
    if ((block = plan->block_grants[i]) != 0)
    {
      #ifdef PARANOID
        if (block > NUM_BLOCKS)
        {
	  log_msg (ERR, "Block request from train=%d referenced illegal block[%d] number=%d", plan->train_num, i, block) ;
	  snapshot_plan (plan) ;
	  _exit (0) ;
	}
      #endif

      #ifdef PARANOID
	if (BLK_OWNER(block) == engineer_msgq [plan->train_num-1])
	{
	  log_msg (ERR, "Train=%d has requested a block=%d which it already owns", plan->train_num, block) ;
	  snapshot_plan (plan) ;
	  _exit(1) ;
	}
      #endif

      if (BLK_OWNER(block) != 0)
      {
	/*
	 * The block is already owned
	 */

	for (j=0 ; j < NUM_RESERVATIONS ; j++)
	  if (Block_Reservation [block-1][j] == 0)
	    break ;

	if (j >= NUM_RESERVATIONS)
        {
	  log_msg (ERR, "Request from train=%d for blk=%d owned by %s. All reservations busy", plan->train_num, block, proc_from_msgq(BLK_OWNER(block))) ;
	  snapshot_plan (plan) ;
	  _exit (0) ;
	}

	Block_Reservation [block-1][j] = plan->train_num ;
	plan->num_reservations++ ;

        #ifdef CONTROLLER_DEBUG
          log_msg (FILE_ONLY, "Reservation=%d for block=%d for train=%d (owned by %s)", plan->num_reservations, block, plan->train_num, proc_from_msgq(BLK_OWNER(block))) ;
        #endif

	plan->block_grants[i] = 0 ; /* set to zero - the block was not granted */
	ret_val = DELAY ;
      }
      else
      {
	/*
	 * The block is not already owned
	 */

        #ifdef CONTROLLER_DEBUG
          log_msg (FILE_ONLY, "Allocating block=%d for train=%d", block, plan->train_num) ;
        #endif

        BLK_OWNER(block) = engineer_msgq [plan->train_num-1] ;
      }
    }
  }

  return (ret_val) ;
}

void
check_and_rm_blocks (volatile ROUTE_PLAN (*plan))
{
  Ushort i ;

  /*
   * Scan through all blocks listed in ->block_grants and
   * remove ownership. Don't change the ->block_grants array
   * because it will be used by the caller to check for
   * reservations which may become grantable.
   */

  for (i=0 ; i < NUM_GRANTS ; i++)
  {
    if (plan->block_returns[i] == 0)
      continue ;

    #ifdef PARANOID
      if (plan->block_returns[0] > NUM_BLOCKS)
      {
	log_msg (ERR, "Received CONTROLLER RETURN with bad blk=%hu from train=%d", plan->block_returns[i], plan->train_num) ;
	snapshot_plan (plan) ;
	_exit(0) ;
      }
    #endif

    if (BLK_OWNER(plan->block_returns[i]) != engineer_msgq[plan->train_num-1])
    {
      log_msg (ERR, "train=%hu returned block=%hu but marked as owned by %hu", plan->train_num, plan->block_returns[i], BLK_OWNER(plan->block_returns[i])) ;
      snapshot_plan (plan) ;
      _exit (0) ;
    }

    #ifdef CONTROLLER_DEBUG
      log_msg (FILE_ONLY, "train=%hu returning block=%hu", plan->train_num, plan->block_returns[i]) ;
    #endif

    BLK_OWNER(plan->block_returns[i]) = 0 ;
  }
}

void
check_and_grant_reservations (volatile ROUTE_PLAN (*plan))
{
  Ushort           i, j, k, item, recipient ;
  Ushort           card_num ;
  POWER_STATE_INFO *ptr ;
  ROUTE_PLAN       *plans = &(*Plan)[0] ;

  /*
   * On entry, the block_returns and psu_grants lists contain all of the
   * blocks and PSUs which have been returned.
   *
   * Scan the list of block_returns and psu_grants and see if there are
   * any other processes waiting on the released items.
   * Grant the released item to ONE of the waiting trains
   */

  #ifdef CONTROLLER_DEBUG
    for (i=0 ; i<NUM_GRANTS ; i++)
      for (j=0 ; j < NUM_RESERVATIONS ; j++)
	if (Block_Reservation[i][j] != 0)
	  log_msg (FILE_ONLY, "chk_res: Current reservation for block=%d for train=%d", i+1, Block_Reservation[i][j]) ;
  #endif

  for (i=0 ; i < NUM_GRANTS ; i++)
  {
    if ((item = plan->block_returns[i]) == 0)
      continue ;

    for (j=0 ; j < NUM_RESERVATIONS ; j++)
      if ((recipient = Block_Reservation[item-1][j]) != 0)
	break ;

    if (j < NUM_RESERVATIONS)
    {
      /*
       * Found a block which has been released and a train
       * waiting for exactly this block
       */

      #ifdef CONTROLLER_DEBUG
        log_msg (FILE_ONLY, "Delayed allocation of block=%d to train=%d", item, recipient) ;
      #endif

      #ifdef PARANOID
	if (item != plans[recipient-1].next[0])
	{
	  log_msg (ERR, "Discrepency between next[0]=%d and granted block=%d", plans[recipient-1].next[0], item) ;
	  _exit(1) ; 
	}
      #endif

      /*
       * find space in the train's block_grants array and
       * inform the train that the block is granted
       */

      lock_sem (Plan_Sem [recipient-1]) ;

      for (k=0 ; k < NUM_GRANTS ; k++)
	if (plans[recipient-1].block_grants[k] == 0)
	  break ;

      if (k >= NUM_GRANTS)
      {
	log_msg (ERR, "No more room in train=%d's block_grants list to receive allocation", recipient) ;
	unlock_sem (Plan_Sem [recipient-1]) ;
	snapshot_plan (plan) ;
	_exit (0) ;
      }

      plans[recipient-1].block_grants[k] = item ;
      BLK_OWNER(item) = engineer_msgq[recipient-1] ;

      if (--plans[recipient-1].num_reservations == 0)
	send_controller_msg (engineer_msgq[recipient-1], controller_msgq, CONTROLLER_MESSAGE, RESPONSE, recipient) ;

      Block_Reservation[item-1][j] = 0 ;
      unlock_sem (Plan_Sem [recipient-1]) ;
    }

    plan->block_returns[i] = 0 ;
  }

  /*
   * Now check for any PSUs which can be reallocated
   */

  #ifdef CONTROLLER_DEBUG
    for (i=0 ; i<NUM_PSU_ALLOCATIONS ; i++)
      for (j=0 ; j < NUM_RESERVATIONS ; j++)
	if (PSU_Reservation[i][j] != 0)
	  log_msg (FILE_ONLY, "chk_res: Current PSU reservation for card=%d for train=%d", i, PSU_Reservation[i][j]) ;
  #endif

  for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
  {
    if ((item = plan->psu_grants[i]) == 0)
      continue ;

    card_num = CARD_FROM_PSU(item) ;

    for (j=0 ; j < NUM_RESERVATIONS ; j++)
      if ((recipient = PSU_Reservation[card_num][j]) != 0)
	break ;

    if (j < NUM_RESERVATIONS)
    {
      #ifdef CONTROLLER_DEBUG
        log_msg (FILE_ONLY, "Delayed allocation of PSU=%d to train=%d", item, recipient) ;
      #endif

      lock_sem (Plan_Sem [recipient-1]) ;

      for (k=0 ; k < NUM_PSU_ALLOCATIONS ; k++)
	if (plans[recipient-1].psu_grants[k] == 0)
	  break ;

      if (k >= NUM_PSU_ALLOCATIONS)
      {
	log_msg (ERR, "No more room in train=%d's psu_grants list to receive allocation", recipient) ;
	unlock_sem (Plan_Sem [recipient-1]) ;
	snapshot_plan (plan) ;
	_exit (0) ;
      }

      PSU_Reservation[card_num][j] = 0 ;
      plans[recipient-1].psu_grants[k] = item ;

      for (k=0 ; k < NUM_PSU_ALLOCATIONS ; k++)
	if (plans[recipient-1].PSU[k] == 0)
	  break ;

      if (k >= NUM_PSU_ALLOCATIONS)
      {
	log_msg (ERR, "No more room in train=%d's PSU list to receive allocation", recipient) ;
	unlock_sem (Plan_Sem [recipient-1]) ;
	snapshot_plan (plan) ;
	_exit (0) ;
      }

      plans[recipient-1].PSU[k] = item ;
      ptr = &(*Power_State)[item-1] ;
      lock_sem (Power_State_Sem [item-1]) ;
      ptr->owner = engineer_msgq[recipient-1] ;
      unlock_sem (Power_State_Sem [item-1]) ;

      if (--plans[recipient-1].num_reservations == 0)
	send_controller_msg (engineer_msgq[recipient-1], controller_msgq, CONTROLLER_MESSAGE, RESPONSE, recipient) ;

      unlock_sem (Plan_Sem [recipient-1]) ;
    }

    plan->psu_grants[i] = 0 ;
  }
}

int
controller_process (void)
{
           CONTROLLER_MSG_UNION msg ;
  volatile ROUTE_PLAN           (*ptr) ;
           int                  i, j, k ;
	   CONTROLLER_SUBCMD    ret_val ;

  #ifdef PARANOID
    int                conf_err ;
  #endif

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), controller_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), controller_msgq, test_msgq) ;
  memset ((void *)Adjacency, 0, sizeof(ADJACENCY *)*NUM_BLOCKS*DEAD_END) ;
  memset ((void *)Block_Allocation, 0, sizeof(int)*NUM_BLOCKS) ;
  memset ((void *)Block_Reservation, 0, sizeof(Block_Reservation)) ;
  memset ((void *)Block_From_Points, 0, sizeof(int)*NUM_POINTS) ;
  memset ((void *)PSU_Reservation, 0, sizeof(PSU_Reservation)) ;
  unlock_sem (Block_Allocation_Sem) ;

  /*
   * Parse the Block[] structure defined in config.h and populate the 2
   * dimensional pointer array (in shared memory) so that references to the
   * layout plan can be made without having to parse the entire structure.
   * At the same time, populate the block_from_points array which
   * establishes a relationship between a point number and its host
   * block.
   *
   * The outer loop (i) scans past information for each track block.
   */

  #ifdef PARANOID
    if ((sizeof(Block) / sizeof (BLOCK_STRUCT)) != NUM_BLOCKS)
    {
      log_msg (ERR, "%d entries in Block[] when expecting %d", (sizeof(Block) / sizeof (BLOCK_STRUCT)), NUM_BLOCKS) ;
      return (0) ;
    }

    conf_err = 0 ;
  #endif

  for (i=0 ; i < NUM_BLOCKS ; i++)
  {
    /*
     * The next loop (j) scans the exits from the block
     */

    for (j=0 ; (j < MAX_ADJACENCIES) && (Block[i].adj_seg[j].adj_seg != 0) ; j++)
    {
      #ifdef PARANOID
        if (Block[i].adj_seg[j].adj_seg > DEAD_END)
        {
	  log_msg (ERR, "Block(#%d %s) adj_seg[%d].adj_seg too large (%d)", i+1, Block[i].name, j, Block[i].adj_seg[j].adj_seg+1) ;
	  conf_err++ ;
	  continue ;
        }

        if ((Block[i].adj_seg[j].length == 0) || (Block[i].adj_seg[j].length > MAX_BLOCK_LENGTH))
	{
	  log_msg (ERR, "Block (#%d %s) adj_seg[%d].length too large or small (%d)", i+1, Block[i].name, j, Block[i].adj_seg[j].length) ;
	  conf_err++ ;
	}

        if ((Block[i].adj_seg[j].polarity != NORMAL_POLARITY) && (Block[i].adj_seg[j].polarity != OPPOSITE_POLARITY))
	{
	  log_msg (ERR, "Block (#%d %s) adj_seg[%d].polarity has strange polarity (%d)", i+1, Block[i].name, j, Block[i].adj_seg[j].polarity) ;
	  conf_err++ ;
	}

	if (ADJ(i+1, Block[i].adj_seg[j].adj_seg) != NULL)
	{
	  log_msg (ERR, "Route from %d to %d defined more than once in Block(%s)", i+1, Block[i].adj_seg[j].adj_seg, Block[i].name) ;
	  conf_err++ ;
	}
      #endif

      #ifdef CONTROLLER_DEBUG
        log_msg (FILE_ONLY, "route defined from %d (%s) to %d", i+1, Block[i].name, Block[i].adj_seg[j].adj_seg) ;
      #endif

      ADJ(i+1, Block[i].adj_seg[j].adj_seg) = &Block[i].adj_seg[j] ;

      /*
       * The next loop (k) scans the point list for this particular exit
       */

      for (k=0 ; (k < MAX_CASCADED_POINTS) && (Block[i].adj_seg[j].points[k].point_num != 0) ; k++)
      {
	#ifdef PARANOID
	  if (Block[i].adj_seg[j].points[k].point_num > NUM_POINTS)
	  {
	    log_msg (ERR, "Block(#%d %s) adj_seg[%d].points[%d].point_num too large (%d)", i+1, Block[i].name, j, k, Block[i].adj_seg[j].points[k].point_num) ;
	    conf_err++ ;
	  }

	  if (((*Block_From_Points)[Block[i].adj_seg[j].points[k].point_num-1] != 0) && ((*Block_From_Points)[Block[i].adj_seg[j].points[k].point_num-1] != i))
	  {
	    log_msg (ERR, "Point number %d is present in block %d and has appeared again in block %d", Block[i].adj_seg[j].points[k].point_num, (*Block_From_Points)[Block[i].adj_seg[j].points[k].point_num-1], i+1) ;
	    conf_err++ ;
	  }

	  if ((Block[i].adj_seg[j].points[k].direction != OPEN) && (Block[i].adj_seg[j].points[k].direction != CLOSED))
	  {
	    log_msg (ERR, "Block(#%d %s) adj_seg[%d].points[%d].direction is illegal (%d)", i+1, Block[i].name, j, k, Block[i].adj_seg[j].points[k].direction) ;
	    conf_err++ ;
	  }
	#endif

        #ifdef CONTROLLER_DEBUG
          log_msg (FILE_ONLY, "Point %d is defined in block %d", Block[i].adj_seg[j].points[k].point_num, i+1) ;
        #endif

	(*Block_From_Points) [Block[i].adj_seg[j].points[k].point_num-1] = i ;
      }
    }
  }

  #ifdef PARANOID
    for (i=0 ; i<NUM_BLOCKS ; i++)
      for (j=0 ; j<NUM_BLOCKS ; j++)
	if (ADJ(i+1, j+1) != NULL)
	  if (ADJ(j+1, i+1) == NULL)
	  {
	    log_msg (ERR, "Route defined from %d to %d but not in reverse direction", i+1, j+1) ;
	    conf_err++ ;
	  }

    if (conf_err != 0)
    {
      log_msg (ERR, "%d errors in Block[]. Aborting.", conf_err) ;
      return (0) ;
    }
  #endif

  unlock_sem (Adjacency_Sem) ;
  unlock_sem (Block_From_Points_Sem) ;

  /*
   * Initialisation is complete. Perform any global tests
   * if these have been requested.
   */

  if (Test_Supply != 0)
    do_power_supply_test() ;

  if (Test_Block != 0)
    do_block_test (Test_Block_Delay) ;

  if (Test_Points != 0)
    do_points_test() ;

  if ((Test_Supply != 0) || (Test_Block != 0) || (Test_Points != 0))
  {
    log_msg (SCREEN, "All test messages have been sent") ;
    sleep (500) ;
    return (0) ;
  }

  /*
   * Request status notification for power supply overloads
   */

  for (i=1 ; i <= NUM_OVERLOAD_SOURCES ; i++)
    request_notification (POWER_OVERLOAD, i, controller_msgq, ON, 0) ;

  for (;;)
  {
    read_mesg (controller_msgq, (LINUX_MSG_BUF *)&msg, sizeof(msg)) ;

    switch (msg.command.mtype)
    {
      case CONTROLLER_MESSAGE:
	lock_sem (Plan_Sem [msg.controller.train_number-1]) ;
	lock_sem (Block_Allocation_Sem) ;
	ptr = &(*Plan)[msg.controller.train_number-1] ;

	if (msg.command.sub_command == REQUEST)
	{
	  /*
	   * Firstly, need to allocate the blocks.
	   * Secondly, need to allocate the PSUs (if necessary)
	   */

	  #ifdef CONTROLLER_DEBUG
	    log_msg (FILE_ONLY, "Received CONTROLLER REQUEST from train=%d", msg.controller.train_number) ;
	  #endif

	  ret_val = RESPONSE ;

	  if (determine_route (ptr))
	    ret_val = allocate_blocks (ptr) ;

	  if (check_and_allocate_psu (ptr) == DELAY)
	    ret_val = DELAY ;

	  send_controller_msg (msg.command.reply_to, controller_msgq, CONTROLLER_MESSAGE, ret_val, msg.controller.train_number) ;
	}
	else if (msg.command.sub_command == NOTIFICATION)
	{
	  /*
	   * Need to parse the plan structure and reserve the train's blocks
	   */

	  #ifdef CONTROLLER_DEBUG
	    log_msg (FILE_ONLY, "Received CONTROLLER NOTIFICATION from train=%d", msg.controller.train_number) ;
	  #endif

	  for (i=0 ; i < ptr->num_blocks ; i++)
	    if (BLK_OWNER(ptr->blocks[i]) != 0)
	    {
	      log_msg (ERR, "train=%d requested ownership of block=%d but already owned by %d", msg.controller.train_number, ptr->blocks[i], BLK_OWNER(ptr->blocks[i])) ;
	      unlock_sem (Block_Allocation_Sem) ;
	      unlock_sem (Plan_Sem [msg.controller.train_number-1]) ;
	      return (0) ;
	    }
	    else
	    {
	      #ifdef CONTROLLER_DEBUG
	        log_msg (FILE_ONLY, "Pre allocating block=%d to train=%d", ptr->blocks[i], msg.controller.train_number) ;
	      #endif

	      BLK_OWNER(ptr->blocks[i]) = msg.command.reply_to ;
	    }

	  send_controller_msg (msg.command.reply_to, controller_msgq, CONTROLLER_MESSAGE, allocate_psu (ptr, CARD_FROM_BLOCK(ptr->blocks[0])), msg.controller.train_number) ;
	}
	else if (msg.command.sub_command == RETURN)
	{
	  /*
	   * return blocks to the free pool
	   */

	  #ifdef CONTROLLER_DEBUG
	    log_msg (FILE_ONLY, "Received CONTROLLER RETURN blk=%hu from train=%d", ptr->block_returns[0], msg.controller.train_number) ;
	  #endif

	  check_and_rm_blocks (ptr) ;
	  check_and_rm_psu (ptr) ;
	  check_and_grant_reservations (ptr) ;
	}
	#ifdef PARANOID
	  else
	  {
	    log_msg (ERR, "Received unsupported BLK sub command (%d)", msg.command.sub_command) ;
	    unlock_sem (Block_Allocation_Sem) ;
	    unlock_sem (Plan_Sem [msg.controller.train_number-1]) ;
	    return (0) ;
	  }
	#endif

	#ifdef CONTROLLER_DEBUG
	  show_blocks (ptr) ;
	#endif

	unlock_sem (Block_Allocation_Sem) ;
	unlock_sem (Plan_Sem [msg.controller.train_number-1]) ;
	break ;

      case BLOCK_NOTIFICATION:
	#ifdef CONTROLLER_DEBUG
	  log_msg (FILE_ONLY, "Received BLOCK_NOTIFICATION for type=%d, id=%d, state=%d", msg.status.type, msg.status.id, msg.status.state) ;
	#endif

	/*
	 * The Status Scanner process is telling us something
	 */

	if ((msg.status.type == POWER_OVERLOAD) && (msg.status.state != 0))
	{
	  /*
	   * One of the power supplies has ungracefully shorted and shut down.
	   * Ungracefully direct all power processes to shut down.
	   *
	   * Send the shutdown request with a ZERO response queue. This forces
	   * the power process to skip the routine task of checking for ownership
	   */

	  log_msg (ERR, "Power Supply %hu has shut down", msg.status.id) ;

	  for (i=1 ; i <= NUM_POWER_SOURCES ; i++)
	    send_power_msg (i, 0, SHUTDOWN_REQUEST, 0, 0, IMMEDIATELY) ;

	  sleep (1) ; /* wait for psus to shut down. Then quit. */
	  _exit (0) ;
	}
	else
	{
	  log_msg (ERR, "Received unknown BLOCK_NOTIFICATION for type=%d, id=%d, state=%d", msg.status.type, msg.status.id, msg.status.state) ;
	  _exit (1) ;
	}

	break ;

      default:
	#ifdef PARANOID
	  log_msg (ERR, "Bad msg recd type: %d, subcmd: %#0x, reply_to: %d", msg.command.mtype, msg.command.sub_command, msg.command.reply_to) ;
	  return (0) ;
	#endif
	break ;
    }
  }
}

/******************************************************************************
 * user_input_process
 *
 * The user input process has a couple of roles in the train system. Firstly,
 * it is responsible for initialising the engineer processes at start up time.
 * It does this by asking the user questions, getting information and then
 * kicking the engineer processes off.
 *
 * The second responsibility of the user input process is to wait for the
 * user to give the command to shut the system down. When a shut down commend
 * is received, the engineers are again kicked with the prompt to gracefully
 * bring their locomotives to a halt.
 *
 * When parsing the block occupancy table to identify trains which are
 * positioned on multiple blocks, a routine called discover_train_recursively
 * is used.
 */

#define IN_SIZE 30

void
input_line (char *line, int size)
{
  if (fgets (line, size, stdin) == NULL)
  {
    log_msg (ERR, "Error reading input string from standard input") ;
    _exit (0) ;
  }
}

void
discover_train_recursively (Ushort *num, Ushort *first, Ushort *train, Uchar *occupancy, Ushort direction, TRACK_POLARITY polarity)
{
  ADJACENCY *X ;
  Ushort    i, j, this_block, this_num ;
  char      line [IN_SIZE] ;

  this_block = (direction) ? train[*num-1] : train[0] ;
  this_num = *num ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
  {
    /*
     * Adjacent blocks with carriages are those which are NOT
     *  NULL in the Adjacency matrix
     *  flagged as being empty within the occupancy array
     *  the opposite polarity setting that we are interested in
     *  already in the list of blocks identified in this train
     */

    X = ADJ(this_block, i+1) ;

    if ((X == NULL) || (occupancy[i] == 0) || (X->polarity != polarity))
      continue ;

    for (j=0 ; j < *num ; j++)
      if (train[j] == i)
	continue ;

    if (*first != 0)
    {
      *first = 0 ;
      printf ("Some adjacent blocks are also occupied.\n") ;
      fflush (stdout) ;
    }

    for (;;)
    {
      printf ("Is a part of this train on block %d (%s)? >", i+1, Block[i].name) ;
      fflush (stdout) ;
      input_line (line, IN_SIZE) ;

      if ((tolower(*line) == 'y') || (tolower(*line) == 'n'))
	break ;

      printf ("Y or N please\n") ;
      fflush (stdout) ;
    }

    if (tolower(*line) == 'n')
      continue ;

    /*
     * We have a new block to add to the train. Depending upon the direction we
     * are searching, add to the begining or end of the existing train list.
     * Then recursively look for more blocks connected to this train.
     */

    if (this_num >= MAX_TRAIN_BLOCKS)
    {
      log_msg (ERR, "Compiled to support max %d blocks per train", MAX_TRAIN_BLOCKS) ;
      _exit (0) ;
    }

    if (direction == 0)
    {
      for (j=*num ; j > 0 ; j--)
	train[j] = train[j-1] ;

      train[0] = i+1 ;
      (*num)++ ;
    }
    else
      train[(*num)++] = i+1 ;

    discover_train_recursively (num, first, train, occupancy, direction, OPPOSITE_BLOCKS_POLARITY(i+1, this_block)) ;

    if (*num != this_num)
      break ;
  }
}

int
user_input_process (void)
{
  Ushort            i, j, rear, first, occupancy_count ;
  Uchar             copy_of_occupancy [NUM_BLOCKS] ;
  char              line [IN_SIZE] ;
  ACCELERATION      acc ;
  INIT_ENGINEER_MSG msg ;

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), user_msgq) ;
  #endif

  set_process_info (Process_Name, getpid(), user_msgq, 0) ;
  set_process_priority (SCHED_RR, 1) ;
  sleep (USER_SLEEP) ;
  lock_sem (Status_Indicators_Sem) ;

  for (i=0 ; i < NUM_OVERLOAD_SOURCES ; i++)
    if ((*Overload_Status)[i] != 0)
    {
      log_msg (ERR, "Can't start train. Controller %hu indicating PSU tripped", i+1) ;
      return (0) ;
    }

  for (occupancy_count=0, i=0 ; i < NUM_BLOCKS ; i++)
  {
    if ((copy_of_occupancy[i] = (*Block_Occupancy)[i]) != 0)
    {
      #ifdef BASIC_DEBUG
        log_msg (NORM, "Block %d (%s) is occupied", i+1, Block[i].name) ;
      #endif

      occupancy_count++ ;
    }
  }

  #ifdef BASIC_DEBUG
    sleep(1) ; /* let the messages get printed */
  #endif

  unlock_sem (Status_Indicators_Sem) ;

  if ((Test_Ring != 0) || (Test_Supply != 0) || (Test_Block != 0) || (Test_Points != 0))
  {
    char  dummy [2] ;

    log_msg (SCREEN, "Press <Enter> to quit or when test complete") ;

    for (;;)
    {
      fgets (dummy, sizeof(dummy), stdin) ;

      if (dummy[strlen(dummy)-1] == '\n')
	break ;
    }

    log_msg (SCREEN, "OK - Shutting down programme") ;
    return (0) ;
  }

  #ifdef SIMULATE_PORT
    for (i=0 ; i < NUM_BLOCKS ; i++)
      copy_of_occupancy[i] = 1 ;
  #else
    if (occupancy_count == 0)
    {
      log_msg (ERR, "No engines detected on any track block! Aborting!") ;
      return (0) ;
    }
    else
    {
      printf ("%d track blocks are occupied", occupancy_count) ;
      fflush (stdout) ;
    }
  #endif

  /*
   * The following loop is a little convoluted.
   *
   * The copy of the occupancy array that we made earlier flags each track
   * block which contains an electrical load on it. We need to scan from the
   * start of this copied array towards the end until we find an occupied block
   *
   * When we find the first block, we will continue to find all blocks which
   * are connected at either end of it (on the layout).
   *
   * Then we'll ask the user if this is an entire train and if so, which of the
   * two ends contains the front.
   *
   * When we know which blocks contain the train, we can send an initialise
   * engineer message. Then we'll null out those blocks, reset our counters and
   * start scanning the copy of the occupancy array from the start again.
   *
   * Only when all elements in the copied array have been nulled out will we be
   * ready to start the train set.
   */

  for (i=occupancy_count=0 ; i < NUM_BLOCKS ; i++)
  {
    /*
     * i gets reset to -1 at the end of each pass
     */

    if (copy_of_occupancy[i] == 0)
      continue ;

    #ifdef SIMULATE_PORT
      for (;;)
      {
	printf ("\nDo you wish to simulate a train on block %d (%s)? >", i+1, Block[i].name) ;
	fflush (stdout) ;
	input_line (line, IN_SIZE) ;

	if ((tolower(*line) == 'y') || (tolower(*line) == 'n'))
	  break ;

	printf ("Y or N please\n") ;
	fflush (stdout) ;
      }

      if (tolower(*line) == 'n')
      {
	copy_of_occupancy[i] = 0 ;
	continue ;
      }
    #else
      printf ("\n\nA train is located on block %d (%s)\n", i+1, Block[i].name) ;
      fflush (stdout) ;
    #endif

    first = 1 ; /* index at which to write discovered blocks */
    memset ((void *)msg.block, 0, sizeof(msg.block)) ;
    msg.num_blocks = 1 ;
    msg.block[0] = i+1 ;

    for (j=1 ; j < NUM_PREV_BLOCKS ; j++)
      msg.block[j] = 0 ;

    discover_train_recursively (&msg.num_blocks, &first, &msg.block[0], &copy_of_occupancy[0], 0, NORMAL_POLARITY) ;
    discover_train_recursively (&msg.num_blocks, &first, &msg.block[0], &copy_of_occupancy[0], 1, OPPOSITE_POLARITY) ;

    if (msg.num_blocks == 1)
    {
      ADJACENCY *X ;
      Ushort    side1 ;
      Ushort    side2 ;

      /*
       * Identify a block at either side of the train
       */

      side1 = side2 = 0xffff ;

      for (j=0 ; j < NUM_BLOCKS ; j++)
      {
	X = ADJ(msg.block[0], j+1) ;

	if (X == NULL)
	  continue ;

	if (X->polarity == NORMAL_POLARITY)
	  side1 = X->adj_seg ;
	else
	  side2 = X->adj_seg ;
      }

      if (side1 == 0xffff)
      {
	side1 = side2 ;
	side2 = 0xffff ;
      }

      for (;;)
      {
	printf ("\nWhich direction is the REAR of the train?\nTowards block %d (%s) or ", side1, Block[side1-1].name) ;

	if (side2 == 0xffff)
	  printf ("the buffer (b)") ;
	else
	  printf ("block %d (%s)", side2, Block[side2-1].name) ;

	printf (" >") ;
	fflush (stdout) ;
	input_line (line, IN_SIZE) ;

	if ((sscanf (line, "%hu", &rear) == 1) && ((rear == side1) || (rear == side2)))
	  break ;

	if ((side2 == 0xffff) && (tolower(*line) == 'b'))
	{
	  rear = 0xffff ;
	  break ;
	}
      }

      msg.block[1] = rear ;

      if (rear == 0xffff)
	log_msg (FILE_ONLY, "Train on block %d and departing buffers", msg.block[0]) ;
      else
	log_msg (FILE_ONLY, "Train on block %d and heading away from block %d", msg.block[0], rear) ;
    }
    else
    {
      for (;;)
      {
	printf ("\nWhich end of train is the rear end?\nBlock %d (%s) or block %d (%s)? >", msg.block[0], Block[msg.block[0]-1].name, msg.block[msg.num_blocks-1], Block[msg.block[msg.num_blocks-1]-1].name) ;
	fflush (stdout) ;
	input_line (line, IN_SIZE) ;

	if ((sscanf (line, "%hu", &rear) == 1) && ((rear == msg.block[0]) || (rear == msg.block[msg.num_blocks-1])))
	  break ;

	printf ("The only valid responses are %d and %d\n", msg.block[0], msg.block[msg.num_blocks-1]) ;
	fflush (stdout) ;
      }

      if (rear == msg.block[0])
      {
	Ushort swap ;
	Ushort temp ;

	for (swap=0 ; swap < (msg.num_blocks-swap) ; swap++)
	{
	  temp = msg.block[swap] ;
	  msg.block[swap] = msg.block[msg.num_blocks-swap-1] ;
	  msg.block[msg.num_blocks-swap-1] = temp ;
	}
      }

      log_msg (FILE_ONLY, "Engine located on block %d. %d blocks occupied.", msg.block[0], msg.num_blocks) ;
    }

    sleep (1) ;

    for (;;)
    {
      printf ("\nEnter the route file number to use for train %d >", occupancy_count+1) ;
      fflush (stdout) ;
      input_line (line, IN_SIZE) ;

      if ((sscanf (line, "%hu", &msg.route_id) == 1) && (msg.route_id < 100))
	break ;

      printf ("Route ID must be between 0 and 99 - try again\n") ;
      fflush (stdout) ;
    }

    for (;;)
    {
      printf ("\nSpeed scaling factor (default=1.0 [no scaling]) >") ;
      fflush (stdout) ;
      input_line (line, IN_SIZE) ;

      if ((strlen(line) == 1) && (*line == '\n'))
      {
	/*
	 * set value to default
	 */

	msg.speed_scale = 1.0 ;
	break ;
      }
 
      if ((sscanf (line, "%f", &msg.speed_scale) == 1) && (msg.speed_scale >= 0.1) && (msg.speed_scale <= 3.0))
	break ;

      printf ("Scaling factor must be between 0.1 and 3.0 - try again\n") ;
      fflush (stdout) ;
    }

    msg.command.mtype = INITIALISE_ENGINEER ;
    msg.command.reply_to = user_msgq ;

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Engineer initialisation msg being sent") ;
    #endif

    /*
     * Send the initialisation message and then null out the occupied blocks
     * from the copy of the occupancy array. Finally, we can reset the
     * loop counter to null and scan the occupancy array from the start
     */

    send_mesg (engineer_msgq[occupancy_count++], (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;

    for (j=0 ; j < msg.num_blocks ; j++)
      copy_of_occupancy[msg.block[j]-1] = 0 ;

    i = 0xffff ; /* reset the loop counter and scan occupancy from start */
    sleep (1) ;

    #ifdef SIMULATE_PORT
      for (;;)
      {
	printf ("\n%d dummy trains defined. Continue defining another? >", occupancy_count) ;
	fflush (stdout) ;
	input_line (line, IN_SIZE) ;

	if ((tolower(*line) == 'y') || (tolower(*line) == 'n'))
	  break ;

	printf ("Y or N please\n") ;
	fflush (stdout) ;
      }

      if (tolower(*line) == 'n')
	break ;
    #endif
  }

  #ifdef SIMULATE_PORT
    if (occupancy_count == 0)
    {
      log_msg (ERR, "No trains have been defined!") ;
      return (0) ;
    }
  #endif

  log_msg (FILE_ONLY, "Total of %d engineer processes have been defined", occupancy_count) ;

  /*
   * All occupied blocks have been initialised with routes
   * and the required number of engineer processes are
   * waiting for the green light to start.
   *
   * Give the green light to just enough processes. Some engineer
   * processes may not be started - these will end up waiting
   * for their initialialisation messages forever.
   */

  sleep (3) ;

  #ifdef BASIC_DEBUG
    log_msg (SCREEN, "Sending START message to the engineers") ;
    snapshot_allocation (FILE_ONLY) ;
    sleep (1) ;
  #else
    printf ("\nTrain set STARTING!\n") ;
  #endif

  for (i=0 ; i < occupancy_count ; i++)
    send_engineer_msg (engineer_msgq[i], user_msgq, START_ENGINEER, 0) ;

  /*
   * wait for the user's commands
   */

  printf ("Enter '?' for help\n") ;

  for (;;)
  { 
    printf ("> ") ;
    fflush (stdout) ;
    input_line (line, IN_SIZE) ;

    if (*line == '\n')
      continue ;

    switch (tolower(*line))
    {
      default:
	printf ("*****Unknown command (%s)", line) ;

      case '?': /* Help */
	printf ("Available commands:\n") ;
	printf ("\t?: Display this help message\n") ;
	printf ("\tc <eng num> <new speed scaler>: Change speed scaling factor\n") ;
	printf ("\tg: Gracefully halt all engines and exit programme\n") ;
	printf ("\th <eng num>: Halt a particular engine\n") ;
	printf ("\tq: Quickly (ungracefully) halt all engines and exit programme\n") ;
	printf ("\ts <eng num>: Restart a particular engine\n") ;
	printf ("\to: Display resource allocations\n") ;
	fflush (stdout) ;
	break ;

      case 'c':  /* Change speed scaling */
	if (sscanf (&line[1], "%hu %f", &i, &msg.speed_scale) != 2)
	{
	  printf ("Badly formed command\n") ;
	  printf ("Usage: c <train num> <new speed scaler>\n") ;
	  fflush (stdout) ;
	  break ;
	}

	if ((msg.speed_scale < 0.1) || (msg.speed_scale > 3.0))
	{
	  printf ("Scaling factor must be between 0.1 and 3.0\n") ;
	  fflush (stdout) ;
	  break ;
	}

	if ((i < 1) || (i > occupancy_count))
	{
bad_engine_number:
	  if (occupancy_count == 1)
	    printf ("Engineer number must be 1 as there's only one train\n") ;
	  else
	    printf ("Engineer number must be between 1 and %hu\n", occupancy_count) ;

	  fflush (stdout) ;
	  break ;
	}

	msg.command.mtype = CHANGE_SPEED_SCALE ;
send_eng_msg:
	msg.command.reply_to = user_msgq ;
	send_mesg (engineer_msgq[i-1], (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;
	break ;

      case 'g': /* Graceful shut down */
        log_msg (SCREEN, "Stopping engines gracefully and exiting") ;
	acc = V_SLOWLY ;
shut_down_train_set:

        #ifdef BASIC_DEBUG
	  snapshot_allocation (FILE_ONLY) ;
        #endif

	/*
	 * Send a shut down message to each engineer and expect to receive
	 * a reply from each when their train finally grinds to a halt
	 */

	for (i=0 ; i < occupancy_count ; i++)
	  send_engineer_msg (engineer_msgq[i], user_msgq, STOP_ENGINEER, acc) ;

	for (i=0 ; i < occupancy_count ; i++)
	{
	  read_mesg (user_msgq, (LINUX_MSG_BUF *)(&msg), sizeof(msg)) ;

	  if (msg.command.mtype != STOP_ENGINEER)
	  {
	    log_msg (ERR, "Received unexpected message from %s type=%d", proc_from_msgq(msg.command.reply_to), msg.command.mtype) ;
	    return (0) ;
	  }

	  log_msg (FILE_ONLY, "Train=%d has stopped its train", msg.command.sub_command) ;
	}

	log_msg (SCREEN, "All engines now stopped - bye bye!") ;
	return (0) ; /* a value of 0 is a healthy return */

      case 'h': /* Halt a particular engine */
	if (sscanf (&line[1], "%hu", &i) != 1)
	{
	  printf ("Badly formed command\n") ;
	  printf ("Usage: h <eng num>\n") ;
	  fflush (stdout) ;
	  break ;
	}

	if ((i < 1) || (i > occupancy_count))
	  goto bad_engine_number ;

        log_msg (SCREEN, "Halting engine %hu", i) ;
	usleep (100000) ;
	msg.command.mtype = HALT_ENGINEER ;
	goto send_eng_msg ;

      case 'q': /* Quickly shut down */
        log_msg (SCREEN, "Stopping engines quickly and exiting") ;
	acc = IMMEDIATELY ;
	goto shut_down_train_set ;

      case 's': /* Start a particular engine */
	if (sscanf (&line[1], "%hu", &i) != 1)
	{
	  printf ("Badly formed command\n") ;
	  printf ("Usage: s <eng num>\n") ;
	  fflush (stdout) ;
	  break ;
	}

	if ((i < 1) || (i > occupancy_count))
	  goto bad_engine_number ;

        log_msg (SCREEN, "Starting engine %hu", i) ;
	usleep (100000) ;
	msg.command.mtype = START_ENGINEER ;
	goto send_eng_msg ;

      case 'o': /* Display resource allocation */
	snapshot_allocation (SCREEN) ;
	usleep (100000) ; /* Let message process run and empty its buffers */
	break ;
    }
  }
}

/******************************************************************************
 * engineer_process
 *
 * An instance of this process is started for each train which can possibly be
 * controlled in the layout. The number of engineer processes is therefore
 * related to the number of controller cards in the system. For one controller
 * card, two trains can be controlled. For two cards, three, for three cards,
 * five and so on.
 */

#define LINE_SIZE 100

#ifdef SIMULATE_PORT
  #define Entry_Constant (5 * TICK_FREQ) /* seconds */
  #define Exit_Constant  (1 * TICK_FREQ) /* seconds */
#endif

void
iterate_train_length (volatile ROUTE_PLAN (*p))
{
  Ushort len ;
  Ushort i ;

  len = (p->next[0] != p->blocks[1]) ? ADJ(p->blocks[0], p->next[0])->length : find_longest_exit_length (p->blocks[0], OPPOSITE_BLOCKS_POLARITY(p->blocks[0], p->next[0])) ;

  for (i=1 ; i < p->num_blocks ; i++)
  {
    len += ADJ(p->blocks[i], p->blocks[i-1])->length ;
    len += ADJ(p->blocks[i-1], p->blocks[i])->length ;
  }

  if ((p->num_blocks == 1) && (p->blocks[1] != 0) && ((p->next[0] != p->blocks[1])))
    len += ADJ(p->blocks[0], p->blocks[1])->length ;
  else if ((p->num_blocks == 1) && (p->blocks[1] != 0))
    len += find_longest_exit_length (p->blocks[0], BLOCKS_POLARITY(p->blocks[0], p->next[0])) ;
  else if (p->num_blocks == 1)
    len += find_longest_exit_length (p->blocks[0], OPPOSITE_BLOCKS_POLARITY(p->blocks[0], p->next[0])) ;
  else if (p->blocks[p->num_blocks] != 0)
    len += ADJ(p->blocks[p->num_blocks-1], p->blocks[p->num_blocks])->length ;
  else
    len += find_longest_exit_length (p->blocks[p->num_blocks-1], OPPOSITE_BLOCKS_POLARITY(p->blocks[p->num_blocks-1], p->blocks[p->num_blocks-2])) ;

  if (p->length > len)
  {
    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Train length now approximated to %d", len) ;
    #endif

    p->length = len ;
  }
}

void
arrange_points (Ushort current, Ushort next, DIR_STATE dir, int msgq, volatile ROUTE_PLAN (*p))
{
  ADJACENCY          *ptr ;
  POINT_SETTING      *point ;

  /*
   * This route is called for every block transition. But blocks do not need to
   * have points at every transition. In this case, this routine does nothing
   * because there will be no points to be set.
   *
   * If the direction is forwards, the points are set in the order they're defined
   * but if reverse, they are set backwards. The intention is that the point
   * closest to the engine will be set before the others.
   */

  lock_sem (Adjacency_Sem) ;
  ptr = ADJ(current, next) ;
  unlock_sem (Adjacency_Sem) ;

  #ifdef PARANOID
    if (ptr == NULL)
    {
      log_msg (ERR, "Can't arrange points. block=%hu is not adjacent to block=%hu", next, current) ;
      _exit (0) ;
    }
  #endif

  if (dir == FORWARDS)
    for (point=ptr->points ; (*point).point_num != 0 ; point++)
      send_points_msg ((*point).point_num, (*point).direction, ++p->point_index, msgq) ;
  else
  {
    /* find the last point */
    for (point=ptr->points ; (*point).point_num != 0 ; point++) ;

    for (point-- ; point >= ptr->points ; point--)
      send_points_msg ((*point).point_num, (*point).direction, ++p->point_index, msgq) ;
  }
}

/*
 * configure_power_supplies sets all power supplies which have been allocated
 * to a train to accelerate towards a new speed value.
 *
 * The returned value is the total number of PSU messages which should be
 * expected IF THE SPEED HAS BEEN TARGETTED TOWARDS 0. Otherwise, the returned
 * value should be 0.
 */

void
configure_power_supplies (volatile ROUTE_PLAN (*ptr), ACC_STATE new_acc, float new_speed, ACCELERATION acc)
{
  Ushort i ;
  Ushort scaled_speed ;
  Ushort scaled_acc ;
  Ushort num=0 ;

  #ifdef ENGINEER_DEBUG
    char *a, *b, *d ;

    log_msg (FILE_ONLY, "Config_pwr_sup called with new_speed = %2.2f, acc=%hu", new_speed, acc) ;
  #endif

  if (ptr->wait_fields != 0)
  {
    new_acc = DECELERATING ;
    new_speed = 0.0 ;
  }

  if (ptr->acc_state > STOPPED) /* DECELERATING_TO_HALT or STOP */
  {
    new_acc = ptr->acc_state ;
    new_speed = 0.0 ;

    if (acc > ptr->acc) /* choose the most aggressive acceleration */
      acc = ptr->acc ;  /* the lower the number the more aggressive */
  }

  if ((ptr->acc_state == DECELERATING) && (new_acc == DECELERATING) && (acc > ptr->acc))
    acc = ptr->acc ; /* choose the most aggressive acceleration */

  scaled_speed = (Ushort)(new_speed * ptr->speed_scale * MAX_SPEED) ;
  scaled_acc = (Ushort)((float)acc / (ptr->speed_scale * ptr->next_speed[0])) ;

  if ((new_speed > 0.0) && (scaled_speed == 0))
    scaled_speed = 1 ; /* rounding errors */

  if (scaled_speed > MAX_SPEED)
    scaled_speed = MAX_SPEED ;

  if ((new_acc == ACCELERATING) && (new_speed < ptr->last_speed))
    new_acc = DECELERATING ;

  #ifdef ENGINEER_DEBUG
    if (new_acc == ACCELERATING) a="ACCEL" ;
    else if (new_acc == AT_SPEED) a="AT_SPEED" ;
    else if (new_acc == DECELERATING) a="DECEL" ;
    else if (new_acc == STOPPED) a="STOPPED" ;
    else a="DECEL_TO_HLT" ;

    if (acc == IMMEDIATELY) b = "IMMED" ;
    else if (acc <= QUICKLY) b = "QUICK" ;
    else if (acc <= MODERATELY) b = "MOD" ;
    else if (acc <= SLOWLY) b = "SLOW" ;
    else b = "VSLOW" ;

    d = (ptr->direction == FORWARDS) ? "FWD" : "REV" ;

    log_msg (FILE_ONLY, "Config_pwr_sup: new_speed=%2.2f (scaled speed=%hu) direction=%s", new_speed, scaled_speed, d) ;
    log_msg (FILE_ONLY, "Config_pwr_sup: acc_state=%s %s (scaled acc=%hu)", a, b, scaled_acc) ;
  #endif

  for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
    if (ptr->PSU[i] != 0)
    {
      send_power_msg (ptr->PSU[i], engineer_msgq[ptr->train_num-1], SET_SPEED_REQUEST, 0, scaled_speed, scaled_acc) ;
      num++ ;
    }

  if (new_speed != 0.0)
    num = 0 ; /* only non zero if decelerating to 0 */

  ptr->last_speed = new_speed ;
  ptr->acc_state = new_acc ;
  ptr->acc = acc ;
  ptr->num_stop = num ;
}

Ushort
get_next_block (FILE **f, int state, Ushort *block, float *speed_fraction)
{
  char      line [LINE_SIZE] ;
  float     fraction ;

  for (;;)
  {
    if (fgets (line, LINE_SIZE, *f) == NULL)
    {
      if (state == 0)    /* 0 means initialising or already rewound */
      {
	*block = 0 ;     /* the default error return value */
	return (1) ;
      }

      /*
       * We must have reached the end of file. Recommence at the start
       */

      rewind (*f) ;
      state = 0 ;
      continue ;
    }

    if (sscanf (line, "%hu", block) == 1)
    {
      if ((*block == 0) || (*block > NUM_BLOCKS))
      {
	log_msg (ERR, "Read an illegal block number=%hu from route file", *block) ;
	*block = 0 ;
	return (1) ;
      }

      if (sscanf (line, "%hu %f", block, &fraction) == 2)
      {
	if ((fraction < 0.05) || (fraction > 1.0))
	{
	  log_msg (ERR, "Read illegal speed setting=%3.3f for block=%hu from route file", fraction, *block) ;
	  *block = 0 ;
	  return (1) ;
	}

	*speed_fraction = fraction ;
      }

      return (0) ;
    }

    /*
     * Anything other than 1 means a comment or blank line
     */
  }
}

ADJACENCY *
get_next_adjacency (FILE **f, Ushort current_block, float *speed_fraction)
{
  Ushort    next_block ;
  ADJACENCY *X ;

  /*
   * Firstly, read the next block from the route file. Then make sure that the
   * next block is attached to the current block (in the layout plan)
   */

  if (get_next_block (f, 1, &next_block, speed_fraction))
  {
    log_msg (ERR, "Couldn't read next block from route file") ;
    return (NULL) ;
  }

  lock_sem (Adjacency_Sem) ;
  X = ADJ(current_block, next_block) ;
  unlock_sem (Adjacency_Sem) ;

  if (X == NULL)
    log_msg (ERR, "Next block (%hu) is not adjacent to current block (%hu)", next_block, current_block) ;

  return (X) ;
}

Ushort
prime_file (volatile ROUTE_PLAN (*ptr), FILE **file)
{
  Ushort i, j, block ;
  float  speed_fraction ;

  /*
   * read the route file and find an ordered set of values which correspond to
   * the train's blocks. Try to locate all of the train's blocks as if the
   * train was facing the FORWARDS direction, if an exact match can't be found,
   * try in the REVERSE direction. If an exact match still can't be found, try
   * to see if the train can smoothly enter the route in the FORWARDS
   * direction. If this fails, try for a REVERSE smooth entry. If this fails,
   * there's an error!
   *
   * There are four separate file search loops below. Each loop exits with
   * block==0 if search failed or block=="head of train" if OK.
   */

  ptr->direction = FORWARDS ;
  ptr->num_next = 0 ;

  #ifdef ENGINEER_DEBUG
    log_msg (FILE_ONLY, "Searching for FORWARDS route match") ;
  #endif

  for (speed_fraction=1.0, i=ptr->num_blocks ; (i > 0) && (get_next_block(file, 0, &block, &speed_fraction) == 0) ;)
  {
    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file - looking for %hu", block, speed_fraction, ptr->blocks[i-1]) ;
    #endif

    if (block == ptr->blocks[i-1])
      i-- ;
    else
      i = ptr->num_blocks ;
  }

  ptr->circumstantial_speed = speed_fraction ;

  if ((block == 0) && (ptr->num_blocks > 1))
  {
    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Searching for a smooth FORWARD entry onto route") ;
    #endif

    ptr->direction = FORWARDS ;
    rewind (*file) ;

    for (speed_fraction=1.0 ; (get_next_block (file, 0, &block, &speed_fraction)) != 0 ;)
    {
      #ifdef ENGINEER_DEBUG
        log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file - looking for %hu", block, speed_fraction, ptr->blocks[0]) ;
      #endif

      if (block != ptr->blocks[0])
	continue ;

      if (get_next_block(file, 0, &block, &speed_fraction) && HEADING_SAME_DIRECTION(ptr->blocks[1], ptr->blocks[0], block))
      {
	ptr->next[0] = block ;
	ptr->num_next = 1 ;
	break ;
      }

      #ifdef ENGINEER_DEBUG
        log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file - not a smooth forwards entry", block, speed_fraction) ;
      #endif
    }
  }

  if ((block == 0) && (ptr->num_blocks > 1))
  {
    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Searching for REVERSE route match") ;
    #endif

    ptr->direction = REVERSE ;
    rewind (*file) ;

    for (speed_fraction=1.0, i=0 ; (i < ptr->num_blocks) && (get_next_block (file, 0, &block, &speed_fraction) != 0) ;)
    {
      #ifdef ENGINEER_DEBUG
        log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file - looking for %hu", block, speed_fraction, ptr->blocks[i]) ;
      #endif

      if (block == ptr->blocks[i])
      {
	if (i++ > 0)
	  ptr->next[ptr->num_next++] = block ;
      }
      else
	ptr->num_next = i = 0 ;
    }
  }

  if ((block == 0) && (ptr->num_blocks > 1))
  {
    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Searching for a smooth REVERSE entry onto route") ;
    #endif

    ptr->direction = REVERSE ;
    rewind (*file) ;

    for (speed_fraction=1.0 ; get_next_block (file, 0, &block, &speed_fraction) != 0 ;)
    {
      #ifdef ENGINEER_DEBUG
        log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file - looking for %hu", block, speed_fraction, ptr->blocks[ptr->num_blocks-1]) ;
      #endif

      if (block != ptr->blocks[ptr->num_blocks-1])
	continue ;

      if (get_next_block(file, 0, &block, &speed_fraction) && HEADING_SAME_DIRECTION(ptr->blocks[ptr->num_blocks-2], ptr->blocks[ptr->num_blocks-1], block))
      {
	for (j=0 ; j < (ptr->num_blocks-1) ; j++)
	  ptr->next[j] = ptr->blocks[j+1] ;

	ptr->next[j++] = block ;
	ptr->num_next = j ;

        #ifdef ENGINEER_DEBUG
          log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file", block, speed_fraction) ;
        #endif
	break ;
      }

      #ifdef ENGINEER_DEBUG
        log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file - not a smooth reverse entry", block, speed_fraction) ;
      #endif
    }
  }

  if (block == 0)
  {
    log_msg (ERR, "Couldn't find a route from train's present location into specified route") ;
    _exit (0) ;
  }

  #ifdef ENGINEER_DEBUG
    for (i=0 ; i < ptr->num_blocks ; i++)
      log_msg (FILE_ONLY, "blocks[%hu] = %hu", i, ptr->blocks[i]) ;

    for (i=0 ; i < ptr->num_next ; i++)
      log_msg (FILE_ONLY, "next[%hu] = %hu, speed[%hu] = %2.2f", i, ptr->next[i], i, ptr->next_speed[i]) ;
  #endif

  return (block) ;
}

void
replenish_cache (volatile ROUTE_PLAN (*ptr), int *block, FILE **file)
{
  Ushort      j ;
  ADJACENCY   *route_ptr ;
  float       speed_fraction ;

  /*
   * Cache the next MIN_NEXT_BLOCKS into the route array
   */

  for (j=ptr->num_next ; j < MIN_NEXT_BLOCKS ; j++)
  {
    speed_fraction = -1.0 ; /* something impossible */

    if ((route_ptr = get_next_adjacency (file, *block, &speed_fraction)) == NULL)
    {
      log_msg (ERR, "Error reading next block[%hu]", j) ;
      _exit (0) ;
    }

    *block = ptr->next[j] = route_ptr->adj_seg ;

    if (j == 0)
      ptr->next_speed[0] = ptr->circumstantial_speed ;
    else ptr->next_speed[j] = (speed_fraction <= 0.0) ? ptr->next_speed[j-1] : speed_fraction ;

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "read block=%hu, speed_frac=%2.2f from route file", *block, ptr->next_speed[j]) ;
    #endif
  }

  ptr->num_next = j ;
}

/*
 * stop_on_exit() is called if train must stop due to a delay.
 * returns 0 if the train must stop immediately.
 * returns block number if train may continue until this block is
 * exited.
 */

Ushort
stop_on_exit (volatile ROUTE_PLAN (*plan))
{
  Ushort len, min_len, i, j, blk ;

  /*
   * The degenerate case is when the train is fully residing on a single
   * block. The answer to our question is "yes", the train CAN fully
   * reside on this block!
   */

  j = plan->num_blocks ;
  blk = 0 ;

  if (j > 1)
  {
    min_len = plan->length + STOPPING_UNCERTAINTY ;

    if (plan->direction == FORWARDS)
    {
      blk = plan->next[0] ;

      for (len=i=0 ; i < (j-1) ; i++)
      {
	len += ADJ(plan->blocks[i], blk)->length ;
	len += ADJ(plan->blocks[i], plan->blocks[i+1])->length ;

	if (len >= min_len)
	  break ;

	blk = plan->blocks[i] ;
      }

      blk = (i < (j-1)) ? plan->blocks[i+1] : 0 ;
    }
    else
    {
      /* same as above but in reverse direction */

      for (len=0, i=j-1 ; i > 0 ; i--)
      {
	blk = plan->blocks[i+1] ;

	if (blk == 0)
	{
	  len += ADJ(plan->blocks[i], find_shortest_exit_length (plan->blocks[i], OPPOSITE_BLOCKS_POLARITY(plan->blocks[i], plan->blocks[i-1])))->length ;
	  len += ADJ(plan->blocks[i], plan->blocks[i-1])->length ;
	}
	else
	{
	  len += ADJ(plan->blocks[i], blk)->length ;
	  len += ADJ(plan->blocks[i], plan->blocks[i-1])->length ;
	}

	if (len >= min_len)
	  break ;
      }

      blk = (i > 0) ? plan->blocks[i-1] : 0 ;
    }
  }

  #ifdef ENGINEER_DEBUG
    log_msg (FILE_ONLY, (blk == 0) ? "Train must stop immediately" : "Train can wait until block %hu exited before stopping") ;
  #endif

  return (blk) ;
}

void
process_PSU_grants (volatile ROUTE_PLAN (*ptr), const int train_number)
{
  Ushort i, j ;

  /*
   * Go through list of PSUs which have been allocated and hand over
   * to them from the PSU presently controlling the engine's block.
   */

  for (j=0 ; j < NUM_PSU_ALLOCATIONS ; j++)
    if ((ptr->PSU[j] != 0) && (CARD_FROM_BLOCK(ptr->blocks[0]) == CARD_FROM_PSU(ptr->PSU[j])))
      break ;

  #ifdef PARANOID
    if (j >= NUM_PSU_ALLOCATIONS)
    {
      log_msg (ERR, "Can't find engine's PSU in allocated PSU list!") ;
      show_blocks (ptr) ;
      _exit (0) ;
    }
  #endif

  for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
  {
    if (ptr->psu_grants[i] == 0)
      continue ;

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Received PSU allocation=%d", ptr->psu_grants[i]) ;
    #endif

    send_power_msg (ptr->psu_grants[i], engineer_msgq[train_number], HANDOVER_REQUEST, ptr->PSU[j], 0, 0) ;
    ptr->psu_grants[i] = 0 ;

    if (ptr->num_stop > 0)
      (ptr->num_stop)++ ;
  }
}

void
process_block_grants (volatile ROUTE_PLAN (*ptr), const int train_number)
{
  Ushort i, j, k ;

  /*
   * For each block which has been granted, we must ask to be notified
   * of status change events. The block must have the right PSU selected
   * and the PSU polarity needs to be configured.
   */

  for (i=0 ; i < NUM_GRANTS ; i++)
  {
    if (ptr->block_grants[i] == 0)
      continue ;

    request_notification (BLOCK_DET, ptr->block_grants[i], engineer_msgq[train_number], ON, 0) ;

    /*
     * Must find out the previous block so that polarity can be set correctly.
     */

    for (j=1 ; j < ptr->num_rev ; j++)
      if (ptr->blocks[j] == ptr->block_grants[i])
	break ;

    if (j < ptr->num_rev)
      j = ptr->blocks[j-1] ;
    else
    {
      for (j=0 ; j < ptr->num_next ; j++)
	if (ptr->next[j] == ptr->block_grants[i])
	  break ;

      if (j >= ptr->num_next)
      {
	log_msg (ERR, "Can't find adjacency to block=%hu in route", ptr->block_grants[i]) ;
	_exit (0) ;
      }

      j = (j == 0) ? ptr->blocks[0] : ptr->next[j-1] ;
    }

    for (k=0 ; k < NUM_PSU_ALLOCATIONS ; k++)
      if ((ptr->PSU[k] != 0) && (CARD_FROM_BLOCK(ptr->block_grants[i]) == CARD_FROM_PSU(ptr->PSU[k])))
	break ;

    #ifdef PARANOID
      if (k >= NUM_PSU_ALLOCATIONS)
      {
	log_msg (ERR, "Can't find PSU for block=%hu", ptr->block_grants[i]) ;
	_exit (0) ;
      }
    #endif

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Setting direction on block=%hu to %s using PSU=%hu", ptr->block_grants[i], (OPPOSITE_BLOCKS_POLARITY(ptr->block_grants[i], j) == NORMAL_POLARITY) ? "NORMAL" : "OPPOSITE", ptr->PSU[k]) ;
    #endif

    send_dispatcher_req_msg (engineer_msgq[train_number], ptr->block_grants[i], BLOCK_DIR, OPPOSITE_BLOCKS_POLARITY(ptr->block_grants[i], j)) ;
    send_dispatcher_req_msg (engineer_msgq[train_number], ptr->block_grants[i], BLOCK_POWER_SRC, PSU_NUMBER(ptr->PSU[k])) ;
    ptr->block_grants[i] = 0 ;
  }
}

void
configure_points_to_move (volatile ROUTE_PLAN (*ptr))
{
  Ulong initial_index ;
  Ushort h, i, j, k, cur_len, next_len ;

  /*
   * If we are going to be continuing further from the next block then set the
   * points to the next anticipated block. Otherwise we will need to be
   * stopping on the next block and changing direction... Just set the points
   * to the longest possible exit.
   */

  if (ptr->direction == FORWARDS)
  {
    h = (ptr->num_blocks >= 2) ? ptr->blocks[1] : 0 ;
    i = ptr->blocks[0] ;
    j = ptr->next[0] ;
    k = ptr->next[1] ;
  }
  else
  {
    h = (ptr->num_blocks >= 2) ? ptr->blocks[ptr->num_blocks-2] : 0 ;
    i = ptr->blocks[ptr->num_blocks-1] ;
    j = ptr->blocks[ptr->num_blocks] ;
    k = ptr->blocks[ptr->num_blocks+1] ;
  }

  if (!HEADING_SAME_DIRECTION(ptr->blocks[0], ptr->next[0], ptr->next[1]))
    k = find_longest_exit (j, OPPOSITE_BLOCKS_POLARITY(j, i)) ;

  cur_len = ADJ(i,j)->length + ((h == 0) ? 0 : ADJ(i,h)->length) ;
  next_len = ADJ(j,i)->length + ((k == 0) ? 0 : ADJ(j,k)->length) ;

  #ifdef ENGINEER_DEBUG
    log_msg (FILE_ONLY, "Setting points for %hu->%hu, %hu->%hu, %hu->%hu", i, j, j, i, j, k) ;
  #endif

  initial_index = ptr->point_index ;
  arrange_points (i, j, FORWARDS, engineer_msgq[ptr->train_num-1], ptr) ;

  /*
   * If there's a risk of overshoot, we'd better wait for the points
   */

  if ((initial_index < ptr->point_index) && (cur_len < (STOPPING_UNCERTAINTY * ptr->speed_scale * 0.5)))
  {
    ptr->point_wait = ptr->point_index ;
    ptr->wait_fields |= POINTS_WAIT ;
    configure_power_supplies (ptr, DECELERATING, 0, SLOWLY) ;

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Delaying train to wait for points. Cause=current block") ;
    #endif
  }

  arrange_points (j, i, REVERSE, engineer_msgq[ptr->train_num-1], ptr) ;

  if (k != 0)
    arrange_points (j, k, FORWARDS, engineer_msgq[ptr->train_num-1], ptr) ;

  if ((initial_index < ptr->point_index) && ((cur_len + next_len) < (STOPPING_UNCERTAINTY * ptr->speed_scale)) && ((ptr->wait_fields & POINTS_WAIT) == 0))
  {
    ptr->point_wait = ptr->point_index ;
    ptr->wait_fields |= POINTS_WAIT ;
    configure_power_supplies (ptr, DECELERATING, 0, SLOWLY) ;

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Delaying train to wait for points. Cause=next block") ;
    #endif
  }
}

void
process_halt (volatile ROUTE_PLAN (*ptr))
{
  Ushort i, j, k;

  memset ((void *)ptr->block_returns, 0, sizeof(ptr->block_returns)) ;
  lock_sem (Block_Allocation_Sem) ;
  k = 0 ;

  for (i=0 ; i < NUM_BLOCKS ; i++)
  {
    if (BLK_OWNER(i+1) != engineer_msgq[ptr->train_num-1])
      continue ; /* only look for blocks owned by this engineer */

    for (j=0 ; j < ptr->num_blocks ; j++)
      if (ptr->blocks[j] == (i+1))
	break ;

    if (j != ptr->num_blocks)
      continue ; /* This block is part of the train */

    #ifdef PARANOID
      if (k > NUM_GRANTS)
      {
	log_msg (ERR, "Trying to free up more blocks than NUM_GRANTS") ;
	break ;
      }
    #endif

    #ifdef ENGINEER_DEBUG
      log_msg (FILE_ONLY, "Returning block=%hu, owned but not occupied", i+1) ;
    #endif

    ptr->block_returns[k++] = i+1 ;
    request_notification (BLOCK_DET, ptr->blocks[i], engineer_msgq[ptr->train_num-1], OFF, 0) ;
  }

  unlock_sem (Block_Allocation_Sem) ;
  ptr->num_rev = ptr->num_blocks ;
  ptr->acc_state = STOPPED ;
  ptr->acc = IMMEDIATELY ;
  send_controller_msg (controller_msgq, engineer_msgq[ptr->train_num-1], CONTROLLER_MESSAGE, RETURN, ptr->train_num) ;
  log_msg (SCREEN, "Engine halted and waiting for user") ;
}

int
engineer_process (const int train_number)
{
           ENGINEER_MSG_UNION msg ;
  volatile ROUTE_PLAN         (*ptr) ;
           FILE               *route ;
           Ushort             i, j ;
	   int                block ;
	   ENGINEER_STATE     en_state ;
	   Ushort             expect_block ;
	   Ushort             chng_dir_flag ;
	   char               file [strlen(DEFAULT_ROUTE_FILENAME)+1] ;

  #ifdef ENGINEER_DEBUG
    char             *e, *s ;
  #endif

  #ifdef SIMULATE_PORT
    int              status_entry ;
    int              status_exit ;
  #endif

  #ifdef PARANOID
    if (train_number >= NUM_ENGINEER_PROCESSES)
    {
      log_msg (ERR, "Process started with illegal train number") ;
      return (0) ;
    }
  #endif

  #ifdef BASIC_DEBUG
    log_msg (FILE_ONLY, "STARTING. PID=%d, Msg queue id=%d", getpid(), engineer_msgq[train_number]) ;
  #endif

  set_process_info (Process_Name, getpid(), engineer_msgq[train_number], 0) ;
  set_process_priority (SCHED_RR, 0) ;
  ptr = &(*Plan)[train_number] ;
  memset ((void *)ptr, 0, sizeof(ROUTE_PLAN)) ;
  ptr->train_num = train_number+1 ;
  ptr->acc_state = STOPPED ;
  ptr->acc = IMMEDIATELY ;
  ptr->dirchng = NO_STATE ;
  ptr->num_stop = 0 ;
  ptr->point_index = 0 ;
  ptr->point_wait = 0 ;
  ptr->wait_fields = 0 ;
  ptr->stat_chng_event = NO_STATE ;

  /* shut up those damned useless compiler warnings */

  expect_block = block = 0 ;
  en_state = UNINITIALISED ;
  chng_dir_flag = 0 ;

  for (;;)
  { 
    #ifdef ENGINEER_DEBUG
      if (en_state == UNINITIALISED) e="UNINIT" ;
      else if (en_state == READY_TO_START) e="READY" ;
      else e="RUN" ;

      if (ptr->stat_chng_event == NO_STATE) s="NO_STATE" ;
      else if (ptr->stat_chng_event == ENTRY) s="ENTRY" ;
      else s="EXIT" ;

      show_blocks (ptr) ;
      log_msg (FILE_ONLY, "en_stat=%s, st_stat=%s, resource_del=%c, chng_dir=%c", e, s, (ptr->wait_fields & CONTROLLER_WAIT) ? 'Y' : 'N', (chng_dir_flag) ? 'Y' : 'N') ;
    #endif

    unlock_sem (Plan_Sem [train_number]) ;
    read_mesg (engineer_msgq[train_number], (LINUX_MSG_BUF *)&msg, sizeof(msg)) ;
    lock_sem (Plan_Sem [train_number]) ;

    switch (msg.command.mtype)
    {
      case INITIALISE_ENGINEER:
	/*---------------------------------------------------------------------
	 * This is the very first message that a new train process should
	 * receive. It is from the User Input Process which will be telling us
	 * which route file to use and which blocks our train is presently
	 * sitting on top of.
	 *
	 * So we'll open the route file, fill out the 'plan' structure with the
	 * blocks that we're on top of, and try to locate these block numbers
	 * (in the correct sequence) within the route file.
	 */

	#ifdef PARANOID
	  if (en_state != UNINITIALISED)
	  {
	    log_msg (ERR, "Received second INITIALISE message") ;
	    return (0) ;
	  }

	  if ((msg.init.num_blocks == 0) || (msg.init.num_blocks > MAX_TRAIN_BLOCKS))
	  {
	    log_msg (ERR, "Received init message with bad num blocks specified (%d)", msg.init.num_blocks) ;
	    return (0) ;
	  }
	#endif

	/*
	 * Locate and open the route file.
	 */

	sprintf (file, DEFAULT_ROUTE_FILENAME, msg.init.route_id) ;

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received INITIALISE_ENGINEER (route file=%s)", file) ;
	#endif

	if ((route = fopen (file, "r")) == (FILE *)0)
	{
	  log_msg (ERR, "Unable to locate and open route file: %s", file) ;
	  _exit (0) ;
	}

	ptr->num_blocks = ptr->num_rev = ptr->num_rev_route =  msg.init.num_blocks ;
	memcpy ((void *)(&ptr->blocks), &msg.init.block, sizeof(msg.init.block)) ;
	log_msg (FILE_ONLY, "Engine located on block %hu", ptr->blocks[0]) ;

	for (i=1 ; i < ptr->num_blocks ; i++)
	  log_msg (FILE_ONLY, "Block %hu contains section %hu of train", ptr->blocks[i], i+1) ;

	ptr->circumstantial_speed = 1.0 ;
	ptr->speed_scale = msg.init.speed_scale ;
	block = prime_file (ptr, &route) ;
	replenish_cache (ptr, &block, &route) ;
	ptr->length = 0xffff ;
	ptr->num_reservations = 0 ;
	ptr->acc_state = STOPPED ;
	iterate_train_length ((ROUTE_PLAN *)ptr) ;

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Train initially headed in %s direction", (ptr->direction == FORWARDS) ? "FORWARDS" : "REVERSE") ;
	#endif

	if (ptr->direction == REVERSE)
	{
	  if (ptr->num_blocks > 1)
	    for (i=1 ; i < ptr->num_blocks ; i++)
	      if (ptr->blocks[i] != ptr->next[i-1])
	      {
		log_msg (ERR, "Not all parts of reversing train located on specified route") ;
		return (0) ;
	      }

	  ptr->blocks[ptr->num_blocks] = ptr->next[ptr->num_blocks-1] ; /* set the block "behind" the last carriage */
	}

	memset ((void *)ptr->block_grants, 0, sizeof(ptr->block_grants)) ;
	memset ((void *)ptr->block_returns, 0, sizeof(ptr->block_returns)) ;
	memset ((void *)ptr->psu_grants, 0, sizeof(ptr->psu_grants)) ;
	send_controller_msg (controller_msgq, engineer_msgq[train_number], CONTROLLER_MESSAGE, NOTIFICATION, train_number+1) ;
	en_state = READY_TO_START ;

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Entering READY_TO_START state") ;
	#endif

	break ;

      case CHANGE_SPEED_SCALE:
	/*---------------------------------------------------------------------
	 * Message from the human (user process) asking to change the speed
	 * scaling value.
	 */

	#ifdef ENGINEER_DEBUG
	log_msg (FILE_ONLY, "Received CHANGE_SPEED_SCALE") ;
	#endif

	log_msg (SCREEN,  "Old scaler=%2.2f New scaler=%2.2f", ptr->speed_scale, msg.init.speed_scale) ;
	ptr->speed_scale = msg.init.speed_scale ;
	break ;

      case START_ENGINEER:
	/*---------------------------------------------------------------------
	 * After all the engineer processes have been initialised by the user
	 * input process, the user input process sends a start_engineer msg
	 * to all engineers.
	 *
	 * Prior to receipt of this start engineer message, the controller
	 * process should have sent the PSU confirmation for the ptr->blocks[0]
	 * block.
	 *
	 * Upon receipt of the start_engineer message, we need to request
	 * granting of the next block from the controller (or if we already own
	 * that block, then request granting of another PSU, or if we already
	 * have the PSU, we can start!)
	 */

	if (en_state == RUNNING)
	{
	  log_msg (ERR, "Received START_ENGINEER message but already running") ;
	  break ;
	}

	#ifdef PARANOID
	  if (en_state != READY_TO_START)
	  {
	    log_msg (ERR, "Received START_ENGINEER message but not ready to start") ;
	    return (0) ;
	  }
	#endif

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received START_ENGINEER") ;
	#endif

	en_state = RUNNING ;
	send_controller_msg (controller_msgq, engineer_msgq[train_number], CONTROLLER_MESSAGE, REQUEST, ptr->train_num) ;
	break ;

      case HALT_ENGINEER:
	/*---------------------------------------------------------------------
	 * The user has asked us to gracefully stop this train. The user could
	 * ask us to start it again in the future.
	 */

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received HALT_ENGINEER") ;
	#endif

	if (en_state != RUNNING)
	    break ;

	configure_power_supplies (ptr, DECELERATING_TO_HALT, 0, V_SLOWLY) ;

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Preparing to halt. Expecting %hu halt message(s)", ptr->num_stop) ;
	#endif
	break ;

      case STOP_ENGINEER:
	/*---------------------------------------------------------------------
	 * The user has asked us to gracefully shut down.
	 *
	 * This message can be received at any time after the engineer has
	 * started. The effect is that all trains slowly come to a halt.
	 * While the trains are slowing down, the remainder of the engineer's
	 * normal operations continue - ie, blocks are still requested,
	 * PSUs are still handed over, blocks are still released etc etc.
	 * When all trains are stopped, we send a message back to the
	 * controller and the controller coordinates the exiting from the
	 * programme.
	 */

	#ifdef PARANOID
	  if (en_state != RUNNING)
	  {
	    log_msg (ERR, "Received STOP_ENGINEER message but not running") ;
	    return (0) ;
	  }
	#endif

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received STOP_ENGINEER") ;
	#endif

	if (ptr->acc_state == STOPPED)
	  goto shut_engineer_down ;

	configure_power_supplies (ptr, DECELERATING_TO_STOP, 0, V_SLOWLY) ;

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Preparing to shut down. Expecting %hu halt message(s)", ptr->num_stop) ;
	#endif

	break ;

      case CONTROLLER_MESSAGE:
	/*---------------------------------------------------------------------
	 * The controller process has responded to a request for a block / PSU.
	 */

	if ((msg.command.sub_command == RESPONSE) && (en_state == READY_TO_START))
	{
	  #ifdef ENGINEER_DEBUG
	    log_msg (FILE_ONLY, "Setting direction on block=%hu to %s using PSU=%hu", ptr->blocks[0], (BLOCKS_POLARITY(ptr->blocks[0], ptr->next[0]) == NORMAL_POLARITY) ? "NORMAL" : "OPPOSITE", ptr->PSU[0]) ;
	  #endif

	  send_dispatcher_req_msg (engineer_msgq[train_number], ptr->blocks[0], BLOCK_DIR, BLOCKS_POLARITY(ptr->blocks[0], ptr->next[0])) ;
	  send_dispatcher_req_msg (engineer_msgq[train_number], ptr->blocks[0], BLOCK_POWER_SRC, PSU_NUMBER(ptr->PSU[0])) ;

	  for (i=0 ; i < ptr->num_rev ; i++)
	    request_notification (BLOCK_DET, ptr->blocks[i], engineer_msgq[train_number], ON, 0) ;

	  for (i=1 ; i < ptr->num_rev ; i++)
	  {
	    arrange_points (ptr->blocks[i-1], ptr->blocks[i], FORWARDS, engineer_msgq[train_number], ptr) ;
	    arrange_points (ptr->blocks[i], ptr->blocks[i-1], REVERSE, engineer_msgq[train_number], ptr) ;
	  }

	  #ifdef ENGINEER_DEBUG
	    log_msg (FILE_ONLY, "Received initial PSU allocation=%d for block=%d", ptr->PSU[0], ptr->blocks[0]) ;
	  #endif

	  break ;
	}

	if (msg.command.sub_command == RESPONSE)
	{
	  Ushort       total_len ;
	  float        speed ;
	  ACCELERATION acceleration ;

	  ptr->direction = HEADING_SAME_DIRECTION(ptr->blocks[1], ptr->blocks[0], ptr->next[0]) ? FORWARDS : REVERSE ;

	  #ifdef ENGINEER_DEBUG
	    log_msg (FILE_ONLY, "Received CONTROLLER response") ;
	    log_msg (FILE_ONLY, "Headed in %s direction towards block=%d, speed=%2.2f", (ptr->direction == FORWARDS) ? "FORWARDS" : "REVERSE", ptr->next[0], ptr->next_speed[0]) ;
	  #endif

	  process_PSU_grants (ptr, train_number) ;
	  process_block_grants (ptr, train_number) ;
	  configure_points_to_move (ptr) ;

	  /*
	   * Calculate correct acceleration and target speed
	   */

	  speed = ptr->circumstantial_speed ;
	  acceleration = V_SLOWLY ;

	  if (!HEADING_SAME_DIRECTION(ptr->next[0], ptr->next[1], ptr->next[2]))
	  {
	    #ifdef ENGINEER_DEBUG
	      log_msg (FILE_ONLY, "DECELERATING for dir change in 2") ;
	    #endif

	    if (speed > 0.5)
	      speed = 0.5 ;
	  }

	  if (!HEADING_SAME_DIRECTION(ptr->blocks[0], ptr->next[0], ptr->next[1]))
	  {
	    #ifdef ENGINEER_DEBUG
	      log_msg (FILE_ONLY, "DECELERATING for dir change in 1") ;
	    #endif

	    acceleration = SLOWLY ;
	    chng_dir_flag = 1 ;
	    total_len = find_partial_len_of_allocated_blocks (ptr) ;

	    if (speed > 0.25)
	      speed = 0.25 ;

	    #ifdef PARANOID
	      if (ptr->dirchng == NO_STATE)
	      {
		log_msg (ERR, "controller has not set ptr->dirchng") ;
		_exit(0) ;
	      }
	    #endif

	    ptr->stat_chng_event = ptr->dirchng ;
	    expect_block = ptr->expect_block ;
	    ptr->dirchng = NO_STATE ;

	    #ifdef ENGINEER_DEBUG
	      log_msg (FILE_ONLY, "Will stop and change direction upon %s block %hu", (ptr->stat_chng_event == ENTRY) ? "ENTRY to" : "EXIT from", expect_block) ;
	    #endif
	  }

	  ptr->wait_fields &= ~CONTROLLER_WAIT ;
	  ptr->stat_chng_event = NO_STATE ;
	  configure_power_supplies (ptr, ACCELERATING, speed, acceleration) ;

	  #ifdef SIMULATE_PORT
	    send_timeout_req_msg (engineer_msgq[train_number], Entry_Constant, &status_entry) ;
	  #endif
	}
	else if (msg.command.sub_command == DELAY)
	{
	  float speed ;

	  #ifdef ENGINEER_DEBUG
	    log_msg (FILE_ONLY, "Received CONTROLLER delay") ;
	  #endif

	  process_PSU_grants (ptr, train_number) ;
	  process_block_grants (ptr, train_number) ;

	  /*
	   * We need to stop very quickly (before we reach next block) because
	   * the next[0] block has not yet been allocated to us. WE HOWEVER CAN
	   * ASSUME THAT THE BLOCK SPECIFIED IN NEXT[0] WILL BE ALLOCATED IN
	   * THE FUTURE and so we can set the points correctly to head towards
	   * it.
	   */

	  if (ptr->acc_state != STOPPED)
	  {
	    /*
	     * decide whether to stop immediately or whether to slow and wait
	     * until train exits the previous block
	     */

	    speed = (ptr->circumstantial_speed > 0.25) ? 0.25 : ptr->circumstantial_speed ;
	    expect_block = stop_on_exit (ptr) ;

	    if (expect_block != 0)
	    {
	      configure_power_supplies (ptr, DECELERATING, speed, (ptr->acc_state < DECELERATING) ? QUICKLY : MODERATELY) ;
	      ptr->stat_chng_event = EXIT ;
	    }
	    else
	    {
	      ACCELERATION acc ;
	      Ushort       first_block_len ;

	      /*
	       * Must slow down and stop before reaching next block
	       */

	      first_block_len = find_length_of_leading_block (ptr) ;

	      if (first_block_len <= ((1.0+ptr->speed_scale)*STOPPING_UNCERTAINTY))
		acc = IMMEDIATELY ;
	      else if (first_block_len <= (STOPPING_UNCERTAINTY*2.5))
		acc = QUICKLY ;
	      else acc = MODERATELY ;

	      #ifdef ENGINEER_DEBUG
	        log_msg (FILE_ONLY, "Decelerating to stop. Acc=%d", acc) ;
	      #endif

	      configure_power_supplies (ptr, DECELERATING, 0, acc) ;
	    }

	    ptr->wait_fields |= CONTROLLER_WAIT ;
	  }

	  arrange_points (ptr->blocks[0], ptr->next[0], FORWARDS, engineer_msgq[train_number], ptr) ; /* just in case its a short block */

	  #ifdef SIMULATE_PORT
	    if (ptr->stat_chng_event != EXIT)
	      send_timeout_req_msg (engineer_msgq[train_number], 0, &status_exit) ;

	    send_timeout_req_msg (engineer_msgq[train_number], 0, &status_entry) ;
	  #endif
	}
	#ifdef PARANOID
	  else
	  {
	    log_msg (ERR, "Received unknown type of CONTROLLER response (%d)", msg.command.sub_command) ;
	    return (0) ;
	  }
	#endif
	break ;

      #ifdef SIMULATE_PORT
        case TIMEOUT_NOTIFICATION:
	  if (msg.command.sub_command == (int)&status_entry)
	  {
	    log_msg (FILE_ONLY, "Received timeout - converting to NOTIFICATION->ENTRY") ;
	    send_timeout_req_msg (engineer_msgq[train_number], Exit_Constant, &status_exit) ;
	    msg.status.state = 0xff ; /* entry event */
	    msg.status.id = (ptr->direction == FORWARDS) ? ptr->next[0] : ptr->blocks[ptr->num_blocks] ;
	  }
	  else if (msg.command.sub_command == (int)&status_exit)
	  {
	    log_msg (FILE_ONLY, "Received timeout - converting to NOTIFICATION->EXIT") ;
	    msg.status.state = 0 ; /* exit event */
	    msg.status.id = (ptr->direction == FORWARDS) ? ptr->blocks[ptr->num_blocks-1] : ptr->blocks[0] ;
	  }
	  else
	  {
	    log_msg (ERR, "Received unknown timeout") ;
	    return (0) ;
	  }

	  msg.status.type = BLOCK_DET ;
      #endif

      case BLOCK_NOTIFICATION:
	/*---------------------------------------------------------------------
	 * The Status Scanner process is telling us that a block is being
	 * released or occupied. Logically, if a block is being released, it
	 * MUST either be the ptr->blocks[0] or ptr->blocks[ptr->num_blocks-1]
	 * segments. If a block is being entered, it MUST be ptr->next[0] or
	 * ptr->blocks[ptr->num_blocks].
	 *
	 * When a block is released, we need to inform the controller that the
	 * block is available for another train. We also need to check to see
	 * if a PSU is able to be released and if so, also inform the
	 * controller. The releasing of a block could also mean that it is
	 * time to stop the train.
	 *
	 * If we are being informed that a block is occupied, we may need to
	 * parse the route file to get the next future block. We will also
	 * need to check if the controller needs to be asked for permission
	 * to enter another block or to find another PSU.
	 */

	#ifdef PARANOID
	  if (msg.status.type != BLOCK_DET)
	  {
	    log_msg (ERR, "Status change notification received for unexpected type %d", msg.status.type) ;
	    return (0) ;
	  }
	#endif

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received STAT_CHNG_NOTIF for type=%d, id=%d, state=%d", msg.status.type, msg.status.id, msg.status.state) ;
	#endif

	if (msg.status.state == 0)
	{
	  /*
	   * A state of 0 means that a block has been released. Need to
	   * remove ourselves from the notification list and inform controller.
	   */

	  #ifdef PARANOID
	    if ((ptr->direction == REVERSE) && (msg.status.id != ptr->blocks[0]))
	    {
	      log_msg (ERR, "Notified of release of %d when expecting to release %d", msg.status.id, ptr->blocks[0]) ;
	      return (0) ;
	    }
	    else if (msg.status.id != ptr->blocks[ptr->num_blocks-1])
	    {
	      log_msg (ERR, "Notified of release of %d when expecting to release %d", msg.status.id, ptr->blocks[ptr->num_blocks-1]) ;
	      return (0) ;
	    }

	    if (ptr->num_blocks <= 1)
	    {
	      log_msg (ERR, "Received block release when train only occupying one block") ;
	      return (0) ;
	    }
	  #endif

	  log_msg (FILE_ONLY, "Leaving block=%d", msg.status.id) ;

	  if (ptr->direction == FORWARDS)
	  {
	    #ifdef PARANOID
	      if ((ptr->num_rev != ptr->num_blocks) || (ptr->num_rev_route != ptr->num_blocks))
	      {
		log_msg (ERR, "num_rev=%hu, num_rev_route=%hu and num_blocks=%hu should be same value", ptr->num_rev, ptr->num_rev_route, ptr->num_blocks) ;
		return (0) ;
	      }
	    #endif

	    ptr->num_rev = ptr->num_rev_route = --ptr->num_blocks ;
	  }
	  else
	  {
	    #ifdef PARANOID
	      if ((ptr->num_rev <= ptr->num_blocks) || (ptr->num_rev_route < ptr->num_rev))
	      {
		log_msg (ERR, "num_block=%hu, num_rev=%hu and num_rev_route=%hu should be monotonic increasing", ptr->num_blocks, ptr->num_rev, ptr->num_rev_route) ;
		return (0) ;
	      }
	    #endif

	    for (i=0 ; i < (NUM_PREV_BLOCKS-2) ; i++)
	      ptr->blocks[i] = ptr->blocks[i+1] ;

	    ptr->num_blocks-- ;
	    ptr->num_rev-- ;
	  }

	  iterate_train_length (ptr) ;
	  request_notification (BLOCK_DET, msg.status.id, engineer_msgq[train_number], OFF, 0) ;
	  memset ((void *)ptr->block_returns, 0, sizeof(ptr->block_returns)) ;
/* ???????? memset ((void *)ptr->block_grants, 0, sizeof(ptr->block_grants)) ; */
	  ptr->block_returns[0] = msg.status.id ;
	  send_controller_msg (controller_msgq, engineer_msgq[train_number], CONTROLLER_MESSAGE, RETURN, train_number+1) ;

	  if (ptr->stat_chng_event == EXIT)
	  {
	    if (msg.status.id == expect_block)
	    {
	      /*
	       * Stop the train. Could be to change direction
	       * or could be just because train is delaying.
	       */

	      configure_power_supplies (ptr, DECELERATING, 0, (chng_dir_flag == 0) ? QUICKLY : MODERATELY) ;
	      ptr->stat_chng_event = NO_STATE ;
	    }
	    #ifdef ENGINEER_DEBUG
	      else
	        log_msg (FILE_ONLY, "Block %hu exited but waiting to exit %hu before stopping", msg.status.id, expect_block) ;
	    #endif
	  }
	}
	else
	{
	  /*
	   * A status message of non zero means that a block has been entered.
	   * Need to shuffle along all elements of the blocks[] and next[]
	   * lists. Check if we need to refresh our cache of next blocks.
	   */

	  if (msg.status.id != ptr->next[0])
	  {
	    log_msg (ERR, "Got block entry msg for unexpected block %d (expected %d)", msg.status.id, ptr->next[0]) ;
	    return (0) ;
	  }

	  log_msg (FILE_ONLY, "Entering block=%d", msg.status.id) ;

	  for (i=NUM_PREV_BLOCKS-1 ; i > 0 ; i--)
	    ptr->blocks[i] = ptr->blocks[i-1] ;

	  ptr->blocks[0] = ptr->next[0] ;
	  ptr->circumstantial_speed = ptr->next_speed[0] ;
	  ptr->num_blocks++ ;
	  ptr->num_rev++ ;
	  ptr->num_rev_route++ ;

	  for (i=0 ; i < (ptr->num_next-1) ; i++)
	  {
	    ptr->next[i] = ptr->next[i+1] ;
	    ptr->next_speed[i] = ptr->next_speed[i+1] ;
	  }

	  ptr->num_next-- ;

	  if (ptr->num_blocks > MAX_OCCUPIED_BLOCKS)
	  {
	    log_msg (ERR, "Num blocks in train=%d more than allowed", ptr->num_blocks) ;
	    return (0) ;
	  }

	  replenish_cache (ptr, &block, &route) ;

	  if (ptr->stat_chng_event == ENTRY)
	  {
	    if (msg.status.id == expect_block)
	    {
	      /*
	       * Stop and change direction
	       */

	      configure_power_supplies (ptr, DECELERATING, 0, MODERATELY) ;
	      ptr->stat_chng_event = NO_STATE ;
	    }
	    #ifdef ENGINEER_DEBUG
	      else
	        log_msg (FILE_ONLY, "Block %hu entered but waiting to enter %hu before stopping", msg.status.id, expect_block) ;
	    #endif
	  }

	  #ifdef SIMULATE_PORT
	    send_timeout_req_msg (engineer_msgq[train_number], Entry_Constant, &status_entry) ;
	  #endif

	  send_controller_msg (controller_msgq, engineer_msgq[train_number], CONTROLLER_MESSAGE, REQUEST, ptr->train_num) ;
	}

	break ;

      case SPEED_NOTIFICATION:
	/*---------------------------------------------------------------------
	 * One of the power processes is telling us that the train has reached
	 * the speed we requested.
	 *
	 * If we are in the process of changing the train's direction, the
	 * polarity on the tracks needs to be swapped and another speed
	 * message sent to the power process.
	 *
	 * If the engineer is shutting down to end the session, we need to
	 * tell the user input process when out train has stopped.
	 */

	#ifdef PARANOID
	  if ((msg.engineer.speed != 0) && (ptr->num_stop != 0))
	  {
	    log_msg (ERR, "Received speed=%hu msg when expecting to stop. num_stop=%hu, acc_state=%hu", msg.engineer.speed, ptr->num_stop, ptr->acc_state) ;
	    return (0) ;
	  }

	  if ((msg.engineer.speed == 0) && (ptr->num_stop == 0))
	  {
	    log_msg (ERR, "Received zero speed msg when not expecting to stop. acc_state=%d", ptr->acc_state) ;
	    return (0) ;
	  }
	#endif

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received SPEED_NOTIFICATION for PSU=%hu, speed=%hu", msg.engineer.psu, msg.engineer.speed) ;
	#endif

	if (msg.engineer.speed == 0)
	{
	  if (--(ptr->num_stop) == 0)
	  {
	    if (ptr->acc_state == DECELERATING_TO_STOP)
	    {
shut_engineer_down:
	      log_msg (FILE_ONLY, "Stopped and shut down") ;
	      msg.command.mtype = STOP_ENGINEER ;
	      msg.command.sub_command = train_number+1 ;
	      msg.command.reply_to = engineer_msgq [train_number] ;
	      send_mesg (user_msgq, (LINUX_MSG_BUF *)(&msg), sizeof(msg.command)) ;
	      /* wait forever */
	      sleep (10000) ;
	    }
	    else if (ptr->acc_state == DECELERATING_TO_HALT)
	    {
	      /*
	       * send msg to controller to free all unoccupied blocks. The
	       * controller will free PSUs. Then return to READY_TO_START state
	       */

	      #ifdef SIMULATE_PORT
	        send_timeout_req_msg (engineer_msgq[train_number], 0, &status_entry) ;
	        send_timeout_req_msg (engineer_msgq[train_number], 0, &status_exit) ;
	      #endif

	      if (ptr->wait_fields & POINTS_WAIT)
		ptr->wait_fields |= HALT_WAIT ;
	      else
	      {
		process_halt (ptr) ;
		en_state = READY_TO_START ;
	      }
	    }
	    else if (chng_dir_flag != 0)
	    {
	      /*
	       * We've stopped and need to complete the reversing process.
	       * Set all the track blocks to the correct direction and
	       * polarities. Then, set all power supplies to accelerate.
	       */

	      chng_dir_flag = 0 ;
	      ptr->direction = (ptr->direction == FORWARDS) ? REVERSE : FORWARDS ;

	      #ifdef ENGINEER_DEBUG
	        log_msg (FILE_ONLY, "Changing direction. Train now headed in %s direction", (ptr->direction == FORWARDS) ? "FORWARDS" : "REVERSE") ;
	      #endif

	      for (i=0 ; i < NUM_PSU_ALLOCATIONS ; i++)
	      {
		if (ptr->PSU[i] != 0)
		{
		  for (j=0 ; j < ptr->num_rev ; j++)
		  {
		    if (CARD_FROM_BLOCK(ptr->blocks[j]) == CARD_FROM_PSU(i))
		    {
		      if ((ptr->direction == FORWARDS) && (j == 0))
			block = ptr->next[0] ;
		      else if (ptr->direction == FORWARDS)
			block = ptr->blocks[j-1] ;
		      else
			block = ptr->blocks[j+1] ;

		      send_dispatcher_req_msg (engineer_msgq[train_number], ptr->blocks[j], BLOCK_DIR, BLOCKS_POLARITY(ptr->blocks[j], block)) ;
		      send_dispatcher_req_msg (engineer_msgq[train_number], ptr->blocks[j], BLOCK_POWER_SRC, PSU_NUMBER(ptr->PSU[i])) ;
		    }
		  }
		}
	      }

	      ptr->acc_state = STOPPED ;
	      ptr->last_speed = 0 ;
	      configure_power_supplies (ptr, ACCELERATING, ptr->circumstantial_speed, V_SLOWLY) ;
	    }
	    else if (ptr->acc_state == DECELERATING)
	    {
	      /*
	       * we've stopped waiting for some other action
	       */

	      #ifdef SIMULATE_PORT
	        send_timeout_req_msg (engineer_msgq[train_number], 0, &status_entry) ;
	        send_timeout_req_msg (engineer_msgq[train_number], 0, &status_exit) ;
	      #endif

	      #ifdef ENGINEER_DEBUG
	        log_msg (FILE_ONLY, "Train stopped and waiting for controller") ;
	      #endif

	      ptr->acc_state = STOPPED ;
	      ptr->last_speed = 0 ;
	    }
	    #ifdef PARANOID
	      else
	      {
	        log_msg (ERR, "Notified speed=0 but ptr->direction=%d", ptr->direction) ;
	        return (0) ;
	      }
	    #endif
	  }
	}
	else
	{
	  #ifdef ENGINEER_DEBUG
	    log_msg (FILE_ONLY, "At speed=%d. Train headed in %s direction", msg.engineer.speed, (ptr->direction == FORWARDS) ? "FORWARDS" : "REVERSE") ;
	  #endif

	  ptr->acc_state = AT_SPEED ;
	}

	break ;

      case POINT_NOTIFICATION:
	/*---------------------------------------------------------------------
	 * One of the points processes is telling us that a point has been
	 * thrown.
	 *
	 * If we're waiting on this particular message to start a halted train,
	 * then start it. Otherwise, keep waiting.
	 */

	#ifdef ENGINEER_DEBUG
	  log_msg (FILE_ONLY, "Received point message. Message index=%u, point_index=%u", msg.points.index, ptr->point_index) ;
        #endif

	if (msg.points.index >= ptr->point_wait)
	{
	  if ((ptr->wait_fields & POINTS_WAIT) != 0)
	  {
	    #ifdef ENGINEER_DEBUG
	      log_msg (FILE_ONLY, "Taking train out of points wait state") ;
            #endif

	    ptr->wait_fields &= ~POINTS_WAIT ;
	    configure_power_supplies (ptr, ACCELERATING, ptr->circumstantial_speed, V_SLOWLY) ;
	  }

	  ptr->point_wait = 0 ;

	  if (ptr->wait_fields & HALT_WAIT)
	  {
	    ptr->wait_fields &= ~HALT_WAIT ;
	    process_halt (ptr) ;
	    en_state = READY_TO_START ;
	  }
	}
	break ;

    default:
      break ;
    }
  }
}

/******************************************************************************
 * parse - parses the command line parameters
 *
 * This routine could in fact have been included within main(). It has been
 * brought out as a simplification of the source code.
 */

#define BAD_ERROR 100

void
parse (int argc, char **argv)
{
  char *p ;

  while ((Error < BAD_ERROR) && (--argc > 0))
  {
    /*
     * point to next argument on the command line (in argv[]).
     */

    p = *++argv ;

    if (*p == '-')
    {
      p++ ;             /* point to first character after the '-' */

      while ((Error < BAD_ERROR) && *p)	switch (toupper(*p++))
      {
        case 'L': 		/* log file */
	  if (Logfilename != (char *)0)
	  {
	    fputs ("More than one log filename specified in command line!\n", stderr) ;
	    Error++ ;
	  }

	  if (*p != '\0')       /*remainder of this element is the filename */
	    Logfilename = p ;
	  else if ((argc > 1) && (*(*(argv+1)) != '-')) /* check next element in argv */
	  {
	    argc-- ;
	    Logfilename = *(++argv) ;
	  }
	  else    /* there is no filename specified - use the default */
	    Logfilename = DEFAULT_LOG_FILENAME ;

	  p = "" ;
	  break ;

        case 'R': 		/* reset the ring */
	  Reset_Ring = 1 ;
	  p = "" ;
	  break ;

        case 'T': 		/* test action */
	  if (*p == '\0')
	  {
	    if ((argc > 1) && (*(*(argv+1)) != '-')) /* check next element in argv */
	    {
	      argc-- ;
	      p = *(++argv) ;
	    }
	    else
	    {
	      fputs ("Test type not specified!\n", stderr) ;
	      Error++ ;
	      p = "" ;
	      break ;
	    }
	  }

	  if ((toupper(*p) == 'A') || (toupper(*p) == 'B') || (toupper(*p) == 'P') || (toupper(*p) == 'R') || (toupper(*p) == 'S'))
	  {
	    long val ;

	    val = -1 ;

	    if (*(p+1) != '\0')
	    {
	      if (sscanf (p+1, "%ld", &val) != 1)
	      {
		val = -1 ;
		fprintf (stderr, "Error reading numeric parameter: '-t%s'!\n", p) ;
		Error++ ;
	      }
	    }

	    if (toupper(*p) == 'A') /* test block relay */
	    {
	      Test_Block_Delay = 1 ;

	      if ((val == -1) || ((val >= 1) && (val <= NUM_BLOCKS)))
		Test_Block = val ;
	      else
	      {
		fprintf (stderr, "Block number must be between 1 and %d!\n", NUM_BLOCKS) ;
		Error++ ;
	      }
	    }
	    else if (toupper(*p) == 'B') /* test blocks */
	    {
	      Test_Block_Delay = 4 ;

	      if ((val == -1) || ((val >= 1) && (val <= NUM_BLOCKS)))
		Test_Block = val ;
	      else
	      {
		fprintf (stderr, "Block number must be between 1 and %d!\n", NUM_BLOCKS) ;
		Error++ ;
	      }
	    }
	    else if (toupper(*p) == 'P') /* test points */
	    {
	      if ((val == -1) || ((val >= 1) && (val <= NUM_POINTS)))
		Test_Points = val ;
	      else
	      {
		fprintf (stderr, "Point number must be between 1 and %d!\n", NUM_POINTS) ;
		Error++ ;
	      }
	    }
	    else if (toupper(*p) == 'R') /* test ring */
	    {
	      if ((val == -1) || ((val >= 1) && (val <= 2)))
		Test_Ring = val ;
	      else
	      {
		fprintf (stderr, "Ring test sequence mustg be between 1 and 2!\n") ;
		Error++ ;
	      }
	    }
	    else if (toupper(*p) == 'S') /* test supply */
	    {
	      if ((val == -1) || ((val >= 1) && (val <= NUM_POWER_SOURCES)))
		Test_Supply = val ;
	      else
	      {
		fprintf (stderr, "Supply number must be between 1 and %d!\n", NUM_POWER_SOURCES) ;
		Error++ ;
	      }
	    }
	  }
	  else
	  {
	    fprintf (stderr, "Unknown test type specified: %c\n", *p) ;
	    Error++ ;
	  }

	  p = "" ;
	  break ;

        case 'V': 		/* verbose */
	  Verbose = 1 ;
	  break ;

        default:
	  fprintf (stderr, "Unrecognised command line option: %c\n", *(p-1)) ;

        case '?':
	  Error = BAD_ERROR ;
	  break ;
      }
    }
    else             /* parameter doesn't commence with a '-' */
    {
      fprintf (stderr, "Command line parameter does not begin with a '-' (%s)\n", p) ;
      Error++ ;
    }
  }
}

/******************************************************************************
 * Here is where the programme starts. Remember, the executable needs to be
 * SUID root or it needs to be run by the root user. (So that the port driver
 * process can access the parallel port register and also change its priority)
 */

int
main (int argc, char **argv)
{
  int          pid[NUM_PROC] ;
  Ushort       i ;
  int          ret_pid ;
  char         index ;
  GENERIC_MSG  close_msg ;
  int          ret_status ;
  SYS_TIME     dummy_time ;

  ret_pid = Error = Test_Block = Test_Points = Test_Ring = Test_Supply = Reset_Ring = 0 ;
  Sys_Time = &dummy_time ;
  Sys_Time->seconds = 0 ;
  Sys_Time->ticks = 0 ;

  index = '!' ;
  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Main") ;

  /*
   * Open the message queues. These are global variables for all processes to
   * access.
   */

  logfile_msgq     = create_mesg (index++) ;
  port_driver_msgq = create_mesg (index++) ;
  scan_msgq        = create_mesg (index++) ;
  status_recv_msgq = create_mesg (index++) ;
  timer_msgq       = create_mesg (index++) ;
  dispatcher_msgq  = create_mesg (index++) ;
  controller_msgq  = create_mesg (index++) ;
  test_msgq        = create_mesg (index++) ;

  for (i=0 ; i < NUM_POINTS_SUPPLIES ; i++)
    points_msgq [i] = create_mesg (index++) ;

  for (i=0 ; i < NUM_POWER_SOURCES ; i++)
    power_msgq [i] = create_mesg (index++) ;

  for (i=0 ; i < NUM_ENGINEER_PROCESSES ; i++)
    engineer_msgq [i] = create_mesg (index++) ;

  user_msgq = create_mesg (index++) ;

  /*
   * log information about the flags which have been defined
   */

  #ifdef BASIC_DEBUG
    log_msg (SCREEN, "Compiled with BASIC_DEBUG defined") ;
  #endif

  #ifdef MESSAGE_TRACE
    log_msg (SCREEN, "Compiled with MESSAGE_TRACE defined") ;
  #endif

  #ifdef SEM_TRACE
    log_msg (SCREEN, "Compiled with SEM_TRACE defined") ;
  #endif

  #ifdef SHM_TRACE
    log_msg (SCREEN, "Compiled with SHM_TRACE defined") ;
  #endif

  #ifdef PORT_DEBUG
    log_msg (SCREEN, "Compiled with PORT_DEBUG defined") ;
  #endif

  #ifdef STATUS_SCAN_DEBUG
    log_msg (SCREEN, "Compiled with STATUS_SCAN_DEBUG defined") ;
  #endif

  #ifdef TIMER_DEBUG
    log_msg (SCREEN, "Compiled with TIMER_DEBUG defined") ;
  #endif

  #ifdef DISPATCHER_DEBUG
    log_msg (SCREEN, "Compiled with DISPATCHER_DEBUG defined") ;
  #endif

  #ifdef POINTS_DEBUG
    log_msg (SCREEN, "Compiled with POINTS_DEBUG defined") ;
  #endif

  #ifdef POWER_DEBUG
    log_msg (SCREEN, "Compiled with POWER_DEBUG defined") ;
  #endif

  #ifdef SIG_HANDLER_DEBUG
    log_msg (SCREEN, "Compiled with SIG_HANDLER_DEBUG defined") ;
  #endif

  #ifdef CONTROLLER_DEBUG
    log_msg (SCREEN, "Compiled with CONTROLLER_DEBUG defined") ;
  #endif

  #ifdef ENGINEER_DEBUG
    log_msg (SCREEN, "Compiled with ENGINEER_DEBUG defined") ;
  #endif

  #ifdef PARANOID
    log_msg (SCREEN, "Compiled with PARANOID defined") ;
  #endif

  #ifdef SIMULATE_PORT
    log_msg (SCREEN, "Compiled with SIMULATE_PORT defined") ;
  #endif

  /*
   * Declare the semaphores and shared memory
   */

  Sys_Time_id       = create_shm (index++, sizeof(Ulong), (void **)(&Sys_Time)) ;
  Sys_Time->seconds = 0 ;
  Sys_Time->ticks = 0 ;
  Notifier_Sem        = create_sem (index++) ;
  Not_Sem             = create_sem (index++) ;
  Notifier_id         = create_shm (index++, sizeof(NOTIFIER)*NUM_SERIAL_IN_BITS, (void **)(&Notifier)) ;
  Not_Block_Det_id    = create_shm (index++, sizeof(NOTIFIER *)*NUM_BLOCKS, (void **)(&Not_Block_Det)) ;
  Not_GP_Input_id     = create_shm (index++, sizeof(NOTIFIER *)*NUM_GP_INPUTS, (void **)(&Not_GP_Input)) ;
  Not_Pow_Overload_id = create_shm (index++, sizeof(NOTIFIER *)*NUM_OVERLOAD_SOURCES, (void **)(&Not_Pow_Overload)) ;
  Status_Indicators_Sem = create_sem (index++) ;
  Block_Occupancy_id  = create_shm (index++, sizeof(Uchar)*NUM_BLOCKS, (void **)(&Block_Occupancy)) ;
  GP_Inputs_id        = create_shm (index++, sizeof(Uchar)*NUM_GP_INPUTS, (void **)(&GP_Inputs)) ;
  Overload_Status_id  = create_shm (index++, sizeof(Uchar)*NUM_OVERLOAD_SOURCES, (void **)(&Overload_Status)) ;

  Block_Allocation_Sem = create_sem (index++) ;
  Block_Allocation_id  = create_shm (index++, sizeof(int)*NUM_BLOCKS, (void **)(&Block_Allocation)) ;

  Block_From_Points_Sem = create_sem (index++) ;
  Block_From_Points_id= create_shm (index++, sizeof(int)*NUM_POINTS, (void **)(&Block_From_Points)) ;

  Adjacency_Sem       = create_sem (index++) ;
  Adjacency_id        = create_shm (index++, sizeof(ADJACENCY *)*NUM_BLOCKS*DEAD_END, (void **)(&Adjacency)) ;

  for (i=0 ; i < NUM_POWER_SOURCES ; i++)
    Power_State_Sem[i]= create_sem (index++) ;

  Power_State_id      = create_shm (index++, sizeof(POWER_STATE_INFO)*NUM_POWER_SOURCES, (void **)(&Power_State)) ;

  for (i=0 ; i < NUM_ENGINEER_PROCESSES ; i++)
    Plan_Sem[i]       = create_sem (index++) ;

  Plan_id             = create_shm (index++, sizeof(ROUTE_PLAN)*NUM_ENGINEER_PROCESSES, (void **)(&Plan)) ;

  Process_Info_Sem    = create_sem (index++) ;
  Process_Info_id     = create_shm (index++, sizeof(PROC_INFO)*NUM_PROC, (void **)(&Proc_Info)) ;
  memset ((void *)(*Proc_Info), 0, sizeof(PROC_INFO)*NUM_PROC) ;
  unlock_sem (Process_Info_Sem) ;

  /*
   * Create the required processes
   */

  parse (argc, argv) ;

  if (((Test_Block != 0) || (Test_Points != 0) || (Test_Ring != 0) || (Test_Supply != 0)) && (Reset_Ring != 0))
  {
    fprintf (stderr, "Can't specify test and reset ring at the same time\n") ;
    Test_Block = Test_Points = Test_Ring = Test_Supply = 0 ;
    Error++ ;
  }

  if (Verbose != 0)
  {
    puts ("train - Copyright (C) 2000 Stefan Keller-Tuberg\n") ;
    puts ("This program is free software; you can redistribute it and/or modify") ;
    puts ("it under the terms of the GNU General Public License Version 2 as published") ;
    puts ("by the Free Software Foundation.\n") ;
    puts ("This program is distributed in the hope that it will be useful,") ;
    puts ("but WITHOUT ANY WARRANTY; without even the implied warranty of") ;
    puts ("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the") ;
    puts ("GNU General Public License for more details.\n") ;
    puts ("\thttp://www.gnu.org/copyleft/gpl.html\n") ;
    puts ("You should have received a copy of the GNU General Public License") ;
    puts ("along with this program; if not, write to the Free Software") ;
    puts ("Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n") ;
  }

  if (Error == 0) 
  {
    for (i=0 ; i < NUM_PROC ; i++)
    {
      /*
       * The way that this loop behaves is a little tricky because processes
       * are created but contexts start to change. All processes will inherit
       * the pid array, but only the parent process will end up with a
       * completed array. The children will have partial arrays (which they
       * never use again).
       *
       * While it might appear that the parent and child will both set the
       * value of pid[i] differently, you need to remember that they will be
       * setting their OWN instance of the pid array - and only the parent's
       * instance really matters!
       */

      if (((Test_Block != 0) || (Test_Points != 0) || (Test_Ring != 0) || (Test_Supply != 0) || (Reset_Ring != 0)) && (i >= FIRST_ENGINEER_PROC))
	break ;

      ret_pid = pid[i] = fork() ;

      if (ret_pid == 0)
	break ; /* I am a child */
      else if (ret_pid == -1)
      {
	perror ("main: fork call failed") ;
	fprintf (stderr, "Failed when trying to start %s\n", Process_Name) ;
	exit (80) ;
      }
    }

    if (ret_pid == 0)  /* I am a child */
    {
      switch (i)
      {
        case LOG_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Logger") ;
	  ret_status = log_process (Logfilename) ;
	  break ;

        case PORT_DRIVER_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Port Driver") ;
	  ret_status = port_driver_process() ;
	  break ;

        case STATUS_SCAN_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Status Scanner") ;
	  ret_status = status_scanner_process() ;
	  break ;

        case DISPATCHER_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Dispatcher") ;
	  ret_status = dispatcher_process() ;
	  break ;

        case TIMEKEEPER_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Timekeeper") ;
	  ret_status = timekeeper_process() ;
	  break ;

        case CONTROLLER_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "Controller") ;
	  ret_status = controller_process() ;
	  break ;

        case USER_INPUT_PROC:
	  sprintf (Process_Name, "%.*s", PROCESS_NAME_SIZE, "User Input") ;
	  ret_status = user_input_process() ;
	  break ;

        default:
	  if ((i >= FIRST_POINTS_PROC) && (i <= LAST_POINTS_PROC))
	  {
	    sprintf (Process_Name, "%.*s %d", PROCESS_NAME_SIZE, "Points", i-FIRST_POINTS_PROC+1) ;
	    ret_status = points_process (i-FIRST_POINTS_PROC) ;
	  }
	  else if ((i >= FIRST_POWER_PROC) && (i <= LAST_POWER_PROC))
	  {
	    sprintf (Process_Name, "%.*s %d", PROCESS_NAME_SIZE, "Power", i-FIRST_POWER_PROC+1) ;
	    ret_status = power_process (i-FIRST_POWER_PROC) ;
	  }
	  else if ((i >= FIRST_ENGINEER_PROC) && (i <= LAST_ENGINEER_PROC))
	  {
	    sprintf (Process_Name, "%.*s %d", PROCESS_NAME_SIZE, "Engineer", i-FIRST_ENGINEER_PROC+1) ;
	    ret_status = engineer_process (i-FIRST_ENGINEER_PROC) ;
	  }
	  else
	  {
	    log_msg (ERR, "Tried to run illegal process number %d", i) ;
	    _exit (0) ;
	  }
      }

      if (ret_status > 1)
	log_msg (ERR, "Process subroutine has returned! ERROR!") ;

      _exit (ret_status) ;
    }
    else  /* I am the parent process */
    {
      #ifdef BASIC_DEBUG
        log_msg (NORM, "Parent is waiting for the first child to quit") ;
      #endif

      ret_pid = wait (&ret_status) ;

      if (WIFSIGNALED(ret_status) || (WEXITSTATUS(ret_status) != 0))
      {
	log_msg (ERR, "The first child has quit (%s)", proc_from_pid (ret_pid)) ;
	log_msg (ERR, "Killing the remainder") ;

	if (WIFEXITED(ret_status))
	{
	  if (WEXITSTATUS(ret_status) > 1)
	    log_msg (ERR, "The dying process indicated status %d", WEXITSTATUS(ret_status)) ;
	}
	else if (WIFSIGNALED(ret_status))
	{
	  log_msg (ERR, "Exited because of an uncaught signal: %d (%#02x)", ret_status, ret_status) ;
	  log_msg (ERR, "(execute 'man 7 signal' for details of this crash)") ;
	}
	else log_msg (ERR, "Something unanticipated caused child to quit!") ;
      }

      /*
       * Close up and quit.
       */

      snapshot_allocation (FILE_ONLY) ;

      for (i=NUM_PROC-1 ; i > 0 ; i--) /* don't kill logger */
      {
	if (i == PORT_DRIVER_PROC)
	  send_port_req_msg (0, INITIALISE_PORT_DRIVER, (void *)0) ;

	/*
	 * The port driver process will receive message and process it
	 * immediately because it is a higher priority task than this one.
	 */

	if (pid[i] != 0)
	  kill (pid[i], SIGTERM) ;
      }

      close_msg.mtype = CLOSE_LOG ;
      send_mesg (logfile_msgq, (LINUX_MSG_BUF *)&close_msg, sizeof(close_msg.mtype)) ;
      sleep (2) ;
	    
      if (logfile_msgq != 0)
	rm_mesg (logfile_msgq) ;

      if (port_driver_msgq != 0)
	rm_mesg (port_driver_msgq) ;

      if (scan_msgq != 0)
	rm_mesg (scan_msgq) ;

      if (status_recv_msgq != 0)
	rm_mesg (status_recv_msgq) ;

      if (timer_msgq != 0)
	rm_mesg (timer_msgq) ;

      if (dispatcher_msgq != 0)
	rm_mesg (dispatcher_msgq) ;

      if (controller_msgq != 0)
	rm_mesg (controller_msgq) ;

      if (test_msgq != 0)
	rm_mesg (test_msgq) ;

      for (i=0 ; i < NUM_POINTS_SUPPLIES ; i++)
	if (points_msgq[i] != 0)
	  rm_mesg (points_msgq[i]) ;

      for (i=0 ; i < NUM_POWER_SOURCES ; i++)
	if (power_msgq[i] != 0)
	  rm_mesg (power_msgq[i]) ;

      for (i=0 ; i < NUM_ENGINEER_PROCESSES ; i++)
	if (engineer_msgq[i] != 0)
	  rm_mesg (engineer_msgq[i]) ;

      if (user_msgq != 0)
	rm_mesg (user_msgq) ;

      if (Sys_Time_id != 0)
	rm_shm (Sys_Time_id) ;

      if (Notifier_Sem != 0)
	rm_sem (Notifier_Sem) ;

      if (Not_Sem != 0)
	rm_sem (Not_Sem) ;

      if (Notifier_id != 0)
	rm_shm (Notifier_id) ;

      if (Not_Block_Det_id != 0)
	rm_shm (Not_Block_Det_id) ;

      if (Not_GP_Input_id != 0)
	rm_shm (Not_GP_Input_id) ;

      if (Not_Pow_Overload_id != 0)
	rm_shm (Not_Pow_Overload_id) ;

      if (Status_Indicators_Sem != 0)
	rm_sem (Status_Indicators_Sem) ;

      if (Block_Occupancy_id != 0)
	rm_shm (Block_Occupancy_id) ;

      if (GP_Inputs_id != 0)
	rm_shm (GP_Inputs_id) ;

      if (Overload_Status_id != 0)
	rm_shm (Overload_Status_id) ;

      if (Block_Allocation_Sem != 0)
	rm_sem (Block_Allocation_Sem) ;

      if (Block_Allocation_id != 0)
	rm_shm (Block_Allocation_id) ;

      if (Block_From_Points_Sem != 0)
	rm_sem (Block_From_Points_Sem) ;

      if (Block_From_Points_id != 0)
	rm_shm (Block_From_Points_id) ;

      if (Adjacency_Sem != 0)
	rm_sem (Adjacency_Sem) ;

      if (Adjacency_id != 0)
	rm_shm (Adjacency_id) ;

      for (i=0 ; i < NUM_POWER_SOURCES ; i++)
	if (Power_State_Sem[i] != 0)
	  rm_sem (Power_State_Sem[i]) ;

      if (Power_State_id != 0)
	rm_shm (Power_State_id) ;

      for (i=0 ; i < NUM_ENGINEER_PROCESSES ; i++)
	if (Plan_Sem[i] != 0)
	  rm_sem (Plan_Sem[i]) ;

      if (Plan_id != 0)
	rm_shm (Plan_id) ;

      if (Process_Info_Sem != 0)
	rm_sem (Process_Info_Sem) ;

      if (Process_Info_id != 0)
	rm_shm (Process_Info_id) ;
    }
  }

  if (Error != 0)
  {
    extract_filename (argv[0]) ;
    fprintf (stderr, "%s - train set control programme\n", Filename) ;
    fprintf (stderr, "Written and Copyright by Stefan Keller-Tuberg, 2000\n") ;
    fprintf (stderr, "\nUsage: %s {-l {filename}} {-r} {-ta#} {-tb#} {-tp#} {-tr#} {-ts#} {-v} {-?}\n", Filename) ;
    fprintf (stderr, "-l\tsave log messages in a file rather than to stdout\n") ;
    fprintf (stderr, "-r\treset the ring and exit the programme\n") ;
    fprintf (stderr, "-ta#\tperform block relay test (#=optional block number)\n") ;
    fprintf (stderr, "-tb#\tperform block test (#=optional block number)\n") ;
    fprintf (stderr, "-tp#\tperform points test (#=optional point number)\n") ;
    fprintf (stderr, "-tr#\tperform ring test (test type 1 or 2)\n") ;
    fprintf (stderr, "-ts#\tperform power supply test (#=optional supply number)\n") ;
    fprintf (stderr, "-v\tbe verbose\n") ;
    fprintf (stderr, "-?\tdisplay this message\n") ;
  }

  exit (0) ;
}
